#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/slab.h>
#include <asm/uaccess.h>
#include <linux/list.h>
#include <linux/types.h>
#include <linux/netfilter_ipv4.h>
#include <linux/in.h>
#include <linux/ip.h>
#include <linux/tcp.h>
#include <linux/udp.h>
#include <net/sock.h>
#include <linux/netlink.h>
#include <linux/sched.h>


#define NETLINK_TEST     30
#define USER_PORT        100
int netlink_count = 0;
char netlink_kmsg[1024];

struct sock *nlsk = NULL;
extern struct net init_net;

int send_usrmsg(char *pbuf, uint16_t len)
{
    struct sk_buff *nl_skb;
    struct nlmsghdr *nlh;

    int ret;

    //Create sk_buff using nlmsg_new().
    nl_skb = nlmsg_new(len, GFP_ATOMIC);
    if(!nl_skb)
    {
        printk("netlink alloc failure\n");
        return -1;
    }

    //Set up nlmsghdr.
    nlh = nlmsg_put(nl_skb, 0, 0, NETLINK_TEST, len, 0);
    if(nlh == NULL)
    {
        printk("nlmsg_put failaure \n");
        nlmsg_free(nl_skb);  //If nlmsg_put() failed, nlmsg_free() will free sk_buff.
        return -1;
    }

    //Copy pbuf to nlmsghdr payload.
    memcpy(nlmsg_data(nlh), pbuf, len);
    ret = netlink_unicast(nlsk, nl_skb, USER_PORT, MSG_DONTWAIT);

    return ret;
}

static void netlink_rcv_msg(struct sk_buff *skb)
{
    struct nlmsghdr *nlh = NULL;
    char *umsg = NULL;
    char *kmsg = "";

    if(skb->len >= nlmsg_total_size(0))
    {
        netlink_count++;
	    snprintf(netlink_kmsg, sizeof(netlink_kmsg), "hello users count=%d", netlink_count);
	    send_usrmsg("wubw123456", 10);
	    kmsg = netlink_kmsg;
        nlh = nlmsg_hdr(skb);  //Get nlmsghdr from sk_buff.
        umsg = NLMSG_DATA(nlh); //Get payload from nlmsghdr.
        if(umsg)
        {
            printk("kernel recv from user: %s\n", umsg);
            send_usrmsg(kmsg, strlen(kmsg));
        }
    }
}

struct netlink_kernel_cfg cfg = { 
        .input  = netlink_rcv_msg, /* set recv callback */
};  

struct list_head In_lhead;	/* Head of inbound-rule list */
struct list_head Out_lhead;	/* Head of outbound-rule list */

/*
 * General filter uses exact match algorithm based on the given rule list.
 */
static unsigned int deepin_general_filter(void *priv, struct sk_buff *skb,
	       const struct nf_hook_state *state,
	       struct list_head *rule_list_head)
{
	//判断黑,白名单list与current数据的关系
	//1.白名单数据,放行 NF_ACCEPT
	//2.黑名单数据,禁止 NF_DROP
	//3.没有记录的数据,使用socket通知用户,等待用户回复消息后再决定是否放行(用户态界面需要获取鼠标,键盘焦点,必须选择后"禁止"/"放行"后才能继续操作)
	// printk("---- hello packets.");
	// printk("---- hello packets. %d", current->pid);

	 //  struct task_struct *task;

  // for_each_process(task)
  // {
  //   printk(KERN_INFO "pid: %d | pname: %s | state: %ld\n", task->pid, task->comm, task->state);
  // }

  // pr_info("sec-open-fn: %s comm: %s\n", fn, current ? current->comm : 0);
  	pr_info("partition , comm[%d]: %s , cpu : %d , flags : %d /n", current->pid, current->comm, current->cpu, current->flags);

	//在安全中心里面结束socket数据
	// if (1)//当黑白名单没记录时
		send_usrmsg("wubowen send user msg ", 30);
	
	return NF_ACCEPT;
}

/*
 * Inbound filter is applied to all inbound packets.
 */
static unsigned int deepin_in_filter(void *priv, struct sk_buff *skb, const struct nf_hook_state *state)
{
	return deepin_general_filter(priv, skb, state, &In_lhead);
}

/*
 * Outbound filter is applied to all outbound packets.
 */
static unsigned int deepin_out_filter(void *priv, struct sk_buff *skb, const struct nf_hook_state *state)
{
	return deepin_general_filter(priv, skb, state, &Out_lhead);
}

/* Inbound hook configuration for netfilter */
struct nf_hook_ops mfw_in_hook_ops = {
	.hook = deepin_in_filter,
	.pf = PF_INET,
	.hooknum = NF_INET_PRE_ROUTING,
	.priority = NF_IP_PRI_FIRST
};

/* Outbound hook configuration for netfilter */
struct nf_hook_ops mfw_out_hook_ops = {
	.hook = deepin_out_filter,
	.pf = PF_INET,
	.hooknum = NF_INET_LOCAL_OUT,
	.priority = NF_IP_PRI_FIRST
};

/*
 * The MiniFirewall kernel module is initialized by this function.
 */
static int __init deepin_mod_init(void)
{
	/* Register netfilter inbound and outbound hooks */
    nf_register_net_hook(&init_net, &mfw_in_hook_ops);
    nf_register_net_hook(&init_net, &mfw_out_hook_ops);

    /* Create netlink socket */
    nlsk = (struct sock *)netlink_kernel_create(&init_net, NETLINK_TEST, &cfg);
    if(nlsk == NULL)
    {   
        printk("netlink_kernel_create error !\n");
        return -1; 
    }   
    printk("netlink_test_init\n");

	return 0;
}

/*
 * The MiniFirewall module is cleaned up by this function.
 */
static void __exit deepin_mod_cleanup(void)
{
    nf_unregister_net_hook(&init_net, &mfw_in_hook_ops);
    nf_unregister_net_hook(&init_net, &mfw_out_hook_ops);

    if (nlsk){
	    netlink_kernel_release(nlsk); /* release ..*/
	    nlsk = NULL;
	}   
    printk("netlink_test_exit!\n");
     
}

module_init(deepin_mod_init);
module_exit(deepin_mod_cleanup);
MODULE_LICENSE("GPL");
MODULE_AUTHOR("deepin-test");