#ifndef COMMON_H
#define COMMON_H

#include <QString>

//以下枚举,结构体实际由插件头文件提供,下面仅用于临时使用
enum Type {
// ⽂件类型
    Normal, //正常,
    Suspicious, //可疑,
    Danger, //危险,
    Malicious,//恶意程序,
//具体都是有什么类型需要商定
    ScanTaskError,// 扫描任务错误
    NetAnomaly,//⽹络异常,
    ServerAnomaly//服务器异常,
};

struct FileInfo {
    Type type;//扫描出的⽂件识别结果或任务错误信息
    QString detail;//⽂件识别结果描述或错误信息描述
};

struct dissectionStruct {
    int taskId;
    FileInfo info;
};

enum State {
    Pause,
    Continue,
    Stop,
};

struct VersionInfo {
    QString version;//版本号
    QString detail;//此版本的描述信息：⽐如新录⼊了xx⽊⻢
    quint64 time; //UTC时间对应的时间戳（从1970-01-01T00:00:00⾄今的毫秒数）
};

enum ErrorType
{
    NoError = 0,
    NetworkError,
};

struct UpdateState {
    double progress;
    VersionInfo info;
};

/*!
 * 概要:指定路径扫描、 进程扫描。
 * 第1个参数：任务id；
 * 第2个参数：扫描进度；
 * 第3个参数：扫描结果；
 */
typedef void (*DISSECTION_PROGRESS_CALLBACK)(int taskId, double progress, FileInfo *detail);

/*!
 * 概要:检查病毒库版本（是否有更新等）。
 * 第1个参数：参数给的是服务器最新版本病毒库的信息，其中时间戳记录的是此版本病毒库的⽣成时间，版本号为空时表示此请求失败，
 *          时间戳对应的值则存储失败类型，且版本描述字符串⽤来存储错误的描述信息；
 */
typedef void (*CHECK_CALLBACK)(VersionInfo *data);

/*!
 * 概要:更新病毒库。
 * 第1个参数：更新进度（0.0-1.0）；
 * 第2个参数：参数给的是服务器最新版本病毒库的信息，其中时间戳记录的是此版本病毒库的⽣成时间，版本号为空时表示此请求失败，
 *          时间戳对应的值则存储失败类型，且版本描述字符串⽤来存储错误的描述信息；
 */
typedef void (*UPATE_PROGRESS_CALLBACK)(double progress, VersionInfo *data);

/*!
 * 概要:指定路径扫描。
 * 第1个参数：扫描路径；
 * 第2个参数：扫描的回调函数；
 */
int dissection(QString path, DISSECTION_PROGRESS_CALLBACK funciton);

/*!
 * 概要:指定进程id扫描。
 * 第1个参数：扫描进程id；
 * 第2个参数：扫描的回调函数；
 */
int dissection(int taskId, DISSECTION_PROGRESS_CALLBACK funciton);

/*!
 * 概要:设置更新状态。
 * 第1个参数：任务id；
 * 第2个参数：将设置状态；
 */
bool setDissectionState(int taskId, State state);

/*!
 * 概要:获取版本信息。
 */
VersionInfo currentVersion();

/*!
 * 概要:检查更新。
 * 第1个参数：回调函数；
 */
void check(CHECK_CALLBACK function);

/*!
 * 概要:更新病毒库。
 * 第1个参数：更新病毒库的回调函数；
 */
void update(UPATE_PROGRESS_CALLBACK function);

#endif // COMMON_H
