/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     wubw <wubowen_cm@deepin.com>
 *
 * Maintainer: wubw <wubowen_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "dbussecuritycenterservice.h"
#include "mainwindow.h"

DBusSecurityCenterService* DBusSecurityCenterService::m_handle = nullptr;

DBusSecurityCenterService::DBusSecurityCenterService(MainWindow *parent, QString type)
    : QObject(parent)
    , m_loadLib(new loadLib(this, type))
    , m_strEngineType(type)//根据引擎类型去get不同的插件对象(getLibTest)
{
    m_handle = this;
    m_parent = parent;
#if USE_DBUS
    m_versionInfo.time = 0;
    m_versionInfo.version = "";
    m_versionInfo.description = "";

    m_updateInfo.progress = 0.0;
    m_updateInfo.info.version = "";
    m_updateInfo.info.description = "";
    m_updateInfo.info.time = 0;

    m_dissectionInfo.taskId = 0;
    m_dissectionInfo.progress = 0.0;
    m_dissectionInfo.fileInfo.type = 0;
    m_dissectionInfo.fileInfo.description = "";

    //注册自定义结构体
    registerDefenderFileInfoMetaType();
    registerDefenderDissectionMetaType();
    registerDefenderVersionInfoMetaType();
    registerDefenderUpdateStateMetaType();

    new DefenderAdaptor(this);
#endif
    QString use_svr = service;
    QString use_path = path;

    if ("" != m_strEngineType) {
        use_svr += QString(".%1").arg(m_strEngineType);
        use_path += QString("/%1").arg(m_strEngineType);
    }
    qDebug() << "use_svr : " << use_svr;
    qDebug() << "use_path : " << use_path;

#if USE_DBUS
    QDBusConnection::sessionBus().registerService(use_svr);
    QDBusConnection::sessionBus().registerObject(use_path, this);
#endif

    //启动的时候就check获取版本信息
    check();
}

DBusSecurityCenterService::~DBusSecurityCenterService()
{

}

//处理Dissection(扫描)返回的信息
void DBusSecurityCenterService::execDissectionCallback(int taskId, double progress, FileInfo *detail)
{
    Q_EMIT notifyDissectionStateChanged(taskId, back);

    if (detail) {
        qDebug() << Q_FUNC_INFO << taskId << progress << detail->type << detail->detail << QThread::currentThreadId();
#if USE_DBUS
        m_dissectionInfo.taskId = taskId;
        m_dissectionInfo.progress = progress;
        m_dissectionInfo.fileInfo.type = detail->type;
        m_dissectionInfo.fileInfo.description = detail->detail;
        Q_EMIT dissectionInfoChanged(m_dissectionInfo);
#endif
    } else {
        qDebug() << Q_FUNC_INFO << taskId << progress << " , FileInfo is nullptr.";
    }

    Q_EMIT notifyDissectionStateChanged(taskId, end);
}

//处理Check(检测病毒库是否有更新)返回的信息
void DBusSecurityCenterService::execCheckCallback(VersionInfo *info)
{
    Q_EMIT notifyCheckStateChanged(back);

    if (info) {
        qDebug() << Q_FUNC_INFO << info->version << info->detail << info->time << QThread::currentThreadId();
#if USE_DBUS
        m_versionInfo.time = info->time;
        m_versionInfo.version = info->version;
        m_versionInfo.description = info->detail;
        Q_EMIT versionInfoChanged(m_versionInfo);
#endif
    } else {
        qDebug() << Q_FUNC_INFO << " backing data is nullptr.";
    }

    Q_EMIT notifyCheckStateChanged(end);
}

//处理Update(更新病毒库)返回的信息
void DBusSecurityCenterService::execUpdateCallback(double progress, VersionInfo *info)
{
    Q_EMIT notifyUpdateStateChanged(back);

    if (info) {
        qDebug() << Q_FUNC_INFO << progress << info->version << info->detail << info->time << QThread::currentThreadId();
#if USE_DBUS
        m_updateInfo.progress = progress;
        m_updateInfo.info.time = info->time;
        m_updateInfo.info.version = info->version;
        m_updateInfo.info.description = info->detail;
        Q_EMIT updateInfoChanged(m_updateInfo);
#endif
    } else {
        qDebug() << Q_FUNC_INFO << progress;
    }

    Q_EMIT notifyUpdateStateChanged(end);
}

#if USE_DBUS
DefenderDissection DBusSecurityCenterService::getDissectionInfo() const
{
    return m_dissectionInfo;
}

DefenderVersionInfo DBusSecurityCenterService::getVersionInfo() const
{
    return m_versionInfo;
}

DefenderUpdateState DBusSecurityCenterService::getUpdateInfo() const
{
    return m_updateInfo;
}
#endif

//每次只加载一种类型的插件,这里传入需要加载插件后缀名称
void DBusSecurityCenterService::loadEngine(QString type)
{
    qDebug() << Q_FUNC_INFO << type;
}

void DBusSecurityCenterService::dissection(int processId)
{
    qDebug() << Q_FUNC_INFO << processId << QThread::currentThreadId();

    Q_EMIT requestDissectionId(processId);

    Q_EMIT notifyDissectionStateChanged(processId, start);
}

void DBusSecurityCenterService::dissection(QString path)
{
    qDebug() << Q_FUNC_INFO << path << QThread::currentThreadId();

    Q_EMIT requestDissectionPath(path);
}

//dissection的回调
void DBusSecurityCenterService::dissectionProgress(int taskId, double progress, FileInfo *detail)
{
    qDebug() << Q_FUNC_INFO << taskId << progress << QThread::currentThreadId();;

    m_handle->execDissectionCallback(taskId, progress, detail);
}

bool DBusSecurityCenterService::setDissectionState(int taskId, State state)
{
    qDebug() << Q_FUNC_INFO << taskId << state;
    return m_loadLib->getLibTest()->setDissectionState(taskId, state);
}

bool DBusSecurityCenterService::setDissectionState(int taskId, int state)
{
    qDebug() << Q_FUNC_INFO << taskId << state;
    return m_loadLib->getLibTest()->setDissectionState(taskId, static_cast<State>(state));
}

#if USE_DBUS
DefenderVersionInfo DBusSecurityCenterService::currentVersion()
{
    qDebug() << Q_FUNC_INFO << QThread::currentThreadId();

    return m_versionInfo;
}
#endif

void DBusSecurityCenterService::check()
{
    Q_EMIT requestCheck();

    Q_EMIT notifyCheckStateChanged(start);
}

//check的回调函数
void DBusSecurityCenterService::checkFinished(VersionInfo *info)
{
    if (info)
        qDebug() << Q_FUNC_INFO << info->version << info->time << info->detail << QThread::currentThreadId();
    else
        qDebug() << Q_FUNC_INFO;

    m_handle->execCheckCallback(info);
}

void DBusSecurityCenterService::update()
{
    Q_EMIT requestUpdate();

    Q_EMIT notifyUpdateStateChanged(start);
}

//update的回调
void DBusSecurityCenterService::updateProgress(double progress, VersionInfo *info)
{
    if (info)
        qDebug() << Q_FUNC_INFO << info->version << info->time << info->detail << QThread::currentThreadId();
    else
        qDebug() << Q_FUNC_INFO << progress;

    m_handle->execUpdateCallback(progress, info);
}
