/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     wubw <wubowen_cm@deepin.com>
 *
 * Maintainer: wubw <wubowen_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef DBUSSECURITYCENTERSERVICE_H
#define DBUSSECURITYCENTERSERVICE_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include <QThread>
#include <QMetaType>

#include "common.h"
#include "loadlib.h"

#define USE_DBUS 1

#if USE_DBUS
#include "defender_adaptor.h"
#endif
/*
 * Adaptor class for interface com.deepin.SecurityCenter
 */
class MainWindow;

const QString service = "com.deepin.defender.hmiscreen";
const QString path = "/com/deepin/defender/hmiscreen";

const QString start = "start";
const QString back = "backing";
const QString end = "end";

class DBusSecurityCenterService: public QObject
{
    Q_OBJECT
public:
    explicit DBusSecurityCenterService(MainWindow *parent = nullptr, QString type = "");
    virtual ~DBusSecurityCenterService();

    inline MainWindow *parent() const;

public:
    void execDissectionCallback(int taskId, double progress, FileInfo *detail);
    void execCheckCallback(VersionInfo *info);
    void execUpdateCallback(double progress, VersionInfo *info);
#if USE_DBUS
    Q_PROPERTY(DefenderDissection dissectionInfo READ getDissectionInfo NOTIFY dissectionInfoChanged)
    DefenderDissection getDissectionInfo() const;

    Q_PROPERTY(DefenderVersionInfo versionInfo READ getVersionInfo NOTIFY versionInfoChanged)
    DefenderVersionInfo getVersionInfo() const;

    Q_PROPERTY(DefenderUpdateState updateInfo READ getUpdateInfo NOTIFY updateInfoChanged)
    DefenderUpdateState getUpdateInfo() const;
#endif
public Q_SLOTS: // METHODS
    /*!
     * \brief 加载引擎
     */
    void loadEngine(QString type);

    /*!
     * \brief 扫描进程
     * \param processId
     * \return 扫描结果
     */
    void dissection(int processId);

    /*!
     * \brief 扫描文件
     * \param path
     * \return 扫描结果
     */
    void dissection(QString path);

    /*!
     * \brief dissection的回调函数
     * \param taskId,任务Id
     * \param progress,扫描进度
     * \param detail,详细信息
     */
    static void dissectionProgress(int taskId, double progress, FileInfo *detail);

    /*!
     * \brief 更改扫描状态
     * \param taskId:任务id
     * \param state:状态枚举值，包含：暂停、继续、停⽌等值，⽤于暂停和停⽌扫描任务
     * \return 更改状态，成功返回true，否则返回false
     */
    bool setDissectionState(int taskId, State state);

    bool setDissectionState(int taskId, int state);

    /*!
     * \brief 获取病毒库版本信息
     * \return 返回病毒库当前版本信息，其中时间戳记录着此版本病毒库更新的时间
     */
//    VersionInfo currentVersion();
#if USE_DBUS
    DefenderVersionInfo currentVersion();
#endif
    /*!
     * \brief 检查病毒库版本（是否有更新等），传⼊的是回调函数，在检查结束后调⽤ : 回调函数返回后通知病毒库当前版本信息
     * \return 返回病毒库当前版本信息
     */
    void check();

    /*!
     * \brief check的回调函数
     * \param 版本信息
     */
    static void checkFinished(VersionInfo *info);

    /*!
     * \brief 更新病毒库，传⼊的是回调函数，在检查结束后调⽤ : 回调函数返回后通知update状态
     */
    void update();

    /*!
     * \brief update的回调函数
     */
    static void updateProgress(double progress, VersionInfo *info);

Q_SIGNALS:
    void requestDissectionId(int);
    void requestDissectionPath(QString);
    void requestCheck();
    void requestUpdate();

    void notifyDissectionStateChanged(int, QString);
    void notifyCheckStateChanged(QString);
    void notifyUpdateStateChanged(QString);
#if USE_DBUS
    void dissectionInfoChanged(DefenderDissection);
    void versionInfoChanged(DefenderVersionInfo);
    void updateInfoChanged(DefenderUpdateState);
#endif
private:
    static DBusSecurityCenterService* m_handle;
    MainWindow *m_parent;
    loadLib *m_loadLib;
    QString m_strEngineType;
#if USE_DBUS
    DefenderDissection m_dissectionInfo;
    DefenderVersionInfo m_versionInfo;
    DefenderUpdateState m_updateInfo;
#endif
};

#endif // DBUSSECURITYCENTERSERVICE_H
