#include "netlicksocket.h"
#include <QDebug>

NetlinkSocket::NetlinkSocket(QObject *parent) : QThread(parent)
{
    init();
    start();
}

NetlinkSocket::~NetlinkSocket()
{
    if (m_nlh) {
        delete m_nlh;
        m_nlh = nullptr;
    }
}

void NetlinkSocket::init()
{
    /*Create netlink socket*/
    m_sockfd = socket(AF_NETLINK, SOCK_RAW, NETLINK_TEST); //Create a socket using user defined protocol NETLINK_TEST.
    if (m_sockfd == -1) {
        perror("create socket error\n");
        return;
    }

    m_timeout.tv_sec = 0;
    m_timeout.tv_usec = 1000 * 100;

    //Source address.
    memset(&m_saddr, 0, sizeof(m_saddr));
    m_saddr.nl_family = AF_NETLINK; //AF_NETLINK
    m_saddr.nl_pid = USER_PORT;  //netlink portid, same as kernel.
    m_saddr.nl_groups = 0;
    if (bind(m_sockfd, (struct sockaddr *)&m_saddr, sizeof(m_saddr)) != 0) { //bind to skfd with saddr.
        perror("bind() error\n");
        close(m_sockfd);
        return;
    }

    //Destination address.
    memset(&m_daddr, 0, sizeof(m_daddr));
    m_daddr.nl_family = AF_NETLINK;
    m_daddr.nl_pid = 0;    // to kernel
    m_daddr.nl_groups = 0;

    m_nlh = (struct nlmsghdr *)malloc(NLMSG_SPACE(MAX_PLOAD));
}

int NetlinkSocket::sendMsg(char *name, int pid, int value)
{
    char *msg = sendData;

    if (!msg)
        return -1;

    memset(msg, 0, strlen(msg));
    sprintf(msg, "%s-%d-%d end",name, pid, value);

//    m_nlh = (struct nlmsghdr *)malloc(NLMSG_SPACE(MAX_PLOAD));
    if (!m_nlh)
        return -1;

    memset(m_nlh, 0, sizeof(struct nlmsghdr));
    m_nlh->nlmsg_len = NLMSG_SPACE(MAX_PLOAD);
    m_nlh->nlmsg_flags = 0;
    m_nlh->nlmsg_type = 0;
    m_nlh->nlmsg_seq = 0;
    m_nlh->nlmsg_pid = m_saddr.nl_pid; //self port
    memcpy(NLMSG_DATA(m_nlh), msg, strlen(msg));

    qDebug() << "sendto kernel: " << msg;
    int ret = sendto(m_sockfd, m_nlh, m_nlh->nlmsg_len, 0, (struct sockaddr *)&m_daddr, sizeof(struct sockaddr_nl));
    if(!ret) {
        perror("sendto error , close socket\n");
        close(m_sockfd);
    }

//    if (m_nlh) {
//        delete m_nlh;
//        m_nlh = nullptr;
//    }

    return ret;
}

ssize_t NetlinkSocket::RioReadn(int fd, void *userbuf, size_t n)
{
    size_t nleft = n;
    ssize_t nread = 0;
    // std::cout<<"rio_readn"<<std::endl;
    char *bufp = (char *)userbuf;
    while (nleft > 0) {
        // std::cout<<"nleft:"<<nleft<<std::endl;
        nread = read(fd, bufp, nleft);

//        if(!recvfrom(fd, &u_info, sizeof(user_msg_info), 0, (struct sockaddr *)&m_daddr, &m_len)) {
        if (!recvfrom(fd, userbuf, sizeof(user_msg_info), 0, (struct sockaddr *)&m_daddr, &m_len)) {
            perror("recv form kernel error\n");
            close(fd);
            exit(-1);
        }

        if (nread < 0) {
            qDebug() << "read error: " << strerror(errno);
            if (errno == EINTR) {
                nread = 0;
            } else {
                return -1;
            }
        } else if (nread == 0) {
            break; // EOF
        }
        // success
        nleft -= nread;
        bufp += nread;
    }
    return n - nleft;
}

void NetlinkSocket::run()
{
    fd_set  m_readfds;
    while (!isInterruptionRequested()) {
//        FD_ZERO(&m_readfds);
//        FD_SET(0, &m_readfds);
        FD_SET(m_sockfd, &m_readfds);

        FD_ZERO(&m_readfds); //每次循环都要清空集合，否则不能检测描述符变化
                FD_SET(m_sockfd,&m_readfds); //添加描述符
//                FD_SET(fp,&m_readfds); //同上

        if (m_sockfd == -1) {
            perror("create socket error\n");
            return;
        }

//        if (select(m_sockfd + 1, &m_readfds, nullptr, nullptr, &m_timeout) <= 0) {
//            qDebug() << "select erro.";
//            continue;
//        } else {
//            if (FD_ISSET(m_sockfd, &m_readfds)) {
//                int len = 0;
//                memset(gbuffer, 0, sizeof(gbuffer));
//                uint32_t msg_length = 0;
//                len = RioReadn(m_sockfd, &msg_length, sizeof(uint32_t));

//                if (len > 0) {
//                    int nread = RioReadn(m_sockfd, u_info.msg, msg_length);
//                    qDebug() << "------------ u_info.msg";

                //Receive netlink message from kernel.
                memset(&u_info, 0, sizeof(u_info));
                m_len = sizeof(struct sockaddr_nl);

                if (!recvfrom(m_sockfd, &u_info, sizeof(user_msg_info), 0, (struct sockaddr *)&m_daddr, &m_len)) {
                    perror("recv form kernel error\n");
                    close(m_sockfd);
                    exit(-1);
                }

//                printf("from kernel:%s\n", u_info.msg);
                if (u_info.msg)
                    qDebug() << "--- from kernel: " << u_info.msg;

//            }/* else if (len == 0) {
//                    LOGE<<"server socket close"<<std::endl;
//                    monitor_off("server socket close");
                    continue;
//                }*/
//            }
        }
//    }
}
