/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     wubowen <wubowen_cm@deepin.com>
 *
 * Maintainer: wubowen <wubowen_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "operatedbusdata.h"
#include "csingleteon.h"

#include <QDebug>

const QString service = "com.deepin.defender.analysisapp";
const QString path = "/com/deepin/defender/analysisapp";

OperateDBusData::OperateDBusData(QObject *parent) : QObject(parent)
{
    registerDefenderProcessListMetaType();
    registerDefenderProcessInfoMetaType();

    qRegisterMetaType<QList<QString>>("QList<QString>");
    qDBusRegisterMetaType<QList<QString>>();

    m_adaptor = new AnalysisappAdaptor(this);

    if (!QDBusConnection::sessionBus().registerService(service) ||
            !QDBusConnection::sessionBus().registerObject(path, this)) {
        qDebug() << "dbus service already registered! " << service;

        exit(0);
    }

    m_data = new GetPidDetailName;
    connect(m_data, &GetPidDetailName::notifyUpdateGuiProcList, [this] {
        qDebug() << "[wubw] notifyUpdateGuiProcList ";
        DEFENDER_MUTEX_LOCKER(&SMutex::getInstance().m_mutex);
        Q_EMIT m_adaptor->processInfoList(m_data->getProcInfoList());
        Q_EMIT m_adaptor->allProcPidList(m_data->getProcList());
        Q_EMIT m_adaptor->guiProcDesktopList(m_data->getGuiProcDesktopList());
        Q_EMIT m_adaptor->singleProcDesktopList(m_data->getSingleProcDesktopList());
    });

    m_launcherInter = new Launcherd("com.deepin.dde.daemon.Launcher", "/com/deepin/dde/daemon/Launcher",
                                    QDBusConnection::sessionBus(), this);

    QTimer *timer = new QTimer;
    connect(timer, &QTimer::timeout, [this] {
        //每5秒更新一次Launcher dbus的应用列表
        updateLauncherList();
    });
    timer->start(1000 * 5);

    //第一次获取数据
    updateLauncherList();
    qDebug() << "[NetworkProtectionWorker] Start. First get launcher app list ";
}

void OperateDBusData::updateLauncherList()
{
    //先获取新应用列表
    QDBusPendingReply<LauncherItemInfoList> reply = getLauncherList();

    reply.waitForFinished();
    if (reply.isError()) {
        qWarning() << "data is empty, quit!!";
        qWarning() << reply.error();
        qApp->quit();
    }

    LauncherItemInfoList list = reply.value();

    bool bIsChanged = false;
    int count = list.count();
    int useCount = m_allApplist.count();
    //①若两个列表数量不一样，必然列表有更新
    //②若数量一样，则会将列表排序，逐一按位比较,若有不一样则说明列表有更新(反之没有更新)
    if (count != useCount) {
        bIsChanged = true;
    } else {
        //使用path进行排序
        qSort(list.begin(), list.end(), [](const LauncherItemInfo & a, const LauncherItemInfo & b) {
            return a.Path > b.Path;
        });

        //比较新收到的list和m_allApplist，有不一样的则说明列表更新了(排序过)
        for (int i = 0; i < count; i++) {
            //找到不一样的就退出循环，说明两个列表不一致
            if (list.at(i).Path != m_allApplist.at(i).Path) {
                bIsChanged = true;
                break;
            }
        }
    }

    //列表有改变才会更新列表
    //没有改变则不通知
    if (bIsChanged) {
        DEFENDER_MUTEX_LOCKER(&SMutex::getInstance().m_mutex);
        m_allApplist = list;
        Q_EMIT m_adaptor->notifyUpdateLauncherList();
        qDebug() << "[NetworkProtectionWorker] Update Launcher app list. count : " << m_allApplist.count();
    }
}

QDBusPendingReply<LauncherItemInfoList> OperateDBusData::getLauncherList()
{
    return m_launcherInter->GetAllItemInfos();
}

void OperateDBusData::StartApp()
{
    qDebug() << Q_FUNC_INFO;
}

void OperateDBusData::ExitApp()
{
    qDebug() << Q_FUNC_INFO;
    qApp->quit();
}

QString OperateDBusData::getProcessEnvironmentVariable(int pid, QString environmentName)
{
    QString desktop = m_data->getProcessEnvironmentVariable(pid, environmentName);
    qDebug() << Q_FUNC_INFO << pid << environmentName << desktop;
    return desktop;
}
