/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     wubowen <wubowen_cm@deepin.com>
 *
 * Maintainer: wubowen <wubowen_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "operatedbusdata.h"
#include "common.h"

#include <DPinyin>
#include <QDebug>

#define Action_Flag_Disable     0                   // 状态标志 - 不允许
#define Action_Flag_Enable      1                   // 状态标志 - 允许

OperateDBusData::OperateDBusData(QObject *parent) :
    QObject(parent)
    , m_launcherInter(new Launcherd("com.deepin.dde.daemon.Launcher", "/com/deepin/dde/daemon/Launcher", QDBusConnection::sessionBus(), this))
    , m_startManagerInterface(new DBusStartManager)
{
    m_adaptpr = new AutostartAdaptor(this);
    if (!QDBusConnection::sessionBus().registerService(service) ||
            !QDBusConnection::sessionBus().registerObject(path, this)) {
        qDebug() << "dbus service already registered! " << service;

        exit(0);
    }

    initialize();
}

void OperateDBusData::StartApp()
{
    qDebug() << Q_FUNC_INFO << " Start AutoStart.";
}

void OperateDBusData::ExitApp()
{
    qApp->quit();
}

void OperateDBusData::initialize()
{
    m_monitor = new StartupMonitor;
    m_monitor->addWatchPath("~/.config/autostart");

    StartupMonitor *hand = StartupMonitor::getInstance();
    connect(hand, SIGNAL(doDataChage(bool, QString)), this, SLOT(refreshData(bool, QString)));
}

void OperateDBusData::preInitialize()
{
}

bool OperateDBusData::startLauncherManage()
{
    m_EnableAppsInfo.clear();
    m_DisableAppsInfo.clear();
    m_EnableData.clear();
    m_DisableData.clear();

    m_reply = m_launcherInter->GetAllItemInfos();
    m_reply.waitForFinished();
    if (m_reply.isError()) {
        qWarning() << "data is empty, quit!!";
        qWarning() << m_reply.error();
    }

    // 添加数据到QMap容器
    m_mapEnable.clear();
    m_mapDisable.clear();
    const LauncherItemInfoList &datas = m_reply.value();
    for (const auto &it : datas) {
        QString sInfo = it.Path
                        + "," + it.Icon
                        + "," + it.ID
                        + "," + it.Name;

        // 判断该应用状态（是否是自启动）
        bool bStatus = isAutostart(it.Path);

        QString sName = DTK_CORE_NAMESPACE::Chinese2Pinyin(it.Name);
        for (auto iterEnable = m_mapEnable.begin(); iterEnable != m_mapEnable.end(); ++iterEnable) {
            if (sName == iterEnable.key())
                sName = sName + "_deepin";
        }

        for (auto iterDisable = m_mapDisable.begin(); iterDisable != m_mapDisable.end(); ++iterDisable) {
            if (sName == iterDisable.key())
                sName = sName + "_deepin";
        }
        sInfo = sInfo + "," + sName;

        if (bStatus) {
            m_EnableData.append(sInfo);
            m_mapEnable[sName] = sInfo;
        } else {
            m_DisableData.append(sInfo);
            m_mapDisable[sName] = sInfo;
        }
    }

    for (int i = 0; i < m_EnableData.count(); ++i) {
        if (i == 0)
            m_EnableAppsInfo = m_EnableData.at(0);
        else
            m_EnableAppsInfo = m_EnableAppsInfo + "|" + m_EnableData.at(i);
    }

    for (int i = 0; i < m_DisableData.count(); ++i) {
        if (i == 0)
            m_DisableAppsInfo = m_DisableData.at(0);
        else
            m_DisableAppsInfo = m_DisableAppsInfo + "|" + m_DisableData.at(i);
    }

    return true;
}
QString OperateDBusData::getAppsInfoEnable()
{
    return m_EnableAppsInfo;
}
QString OperateDBusData::getAppsInfoDisable()
{
    return m_DisableAppsInfo;
}

// 判断是否是开机自启
bool OperateDBusData::isAutostart(QString sPath)
{
    return m_startManagerInterface->IsAutostart(sPath);
}

// 执行开机自启动管理 开启/禁止操作
bool OperateDBusData::exeAutostart(int nStatus, QString sPath)
{
    bool flag = false;
    // 执行自启动关闭操作
    if (Action_Flag_Disable == nStatus) {
        QDBusPendingReply<bool> reply = m_startManagerInterface->RemoveAutostart(sPath);
        reply.waitForFinished();
        if (!reply.isError()) {
            flag = true;
        }
    } else {
        // 执行自启动开启操作
        QDBusPendingReply<bool> reply =  m_startManagerInterface->AddAutostart(sPath);
        reply.waitForFinished();
        if (!reply.isError()) {
            flag = true;
        }
    }

    return flag;
}
// 外部调用刷新表格数据
void OperateDBusData::refreshData(bool bAdd, QString sID)
{
    Q_EMIT AccessRefreshData(bAdd, sID);
}
