/*
 * Copyright (C) 2017 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     wubowen <wubowen_cm@deepin.com>
 *
 * Maintainer: wubowen <wubowen_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "servicedbus.h"
#include "common/common.h"

const int timeInter = 3;

const QString service = "com.deepin.defender.daemonservice";
const QString path = "/com/deepin/defender/daemonservice";

ServiceDBus::ServiceDBus(QObject *parent)
    : QObject(parent)
    , m_analysisapp(new Analysisapp("com.deepin.defender.analysisapp", "/com/deepin/defender/analysisapp",
                                    QDBusConnection::sessionBus(), this))
    , m_monitornetflow(new MonitorNetFlow("com.deepin.defender.MonitorNetFlow", "/com/deepin/defender/MonitorNetFlow",
                                          QDBusConnection::systemBus(), this))
    , m_trafficDetails(new TrafficDetails("com.deepin.defender.trafficdetails", "/com/deepin/defender/trafficdetails",
                                          QDBusConnection::sessionBus(), this))//注意大小写，格式
    , m_autostart(new Autostart("com.deepin.defender.autostart", "/com/deepin/defender/autostart",
                                QDBusConnection::sessionBus(), this))
    , m_netcontrol(new Netcontrol("com.deepin.defender.netcontrol", "/com/deepin/defender/netcontrol",
                                  QDBusConnection::sessionBus(), this))
{
    m_gset = new QGSettings("com.deepin.defender.deamonservice", QByteArray(), this);
    m_adaptor = new DaemonserviceAdaptor(this);
    if (!QDBusConnection::sessionBus().registerService(service) ||
            !QDBusConnection::sessionBus().registerObject(path, this)) {
        qDebug() << "dbus service already registered! " << service;

        exit(0);
    }
    QTimer *timer = new QTimer;
    //每隔3秒调启动一次各应用
    connect(timer, &QTimer::timeout, [this] {
        //应用启动需要注意时序，服务直接彼此也会调用
        m_trafficDetails->StartApp();

        m_autostart->StartApp();

        m_netcontrol->StartApp();

        m_monitornetflow->StartApp();

        m_analysisapp->StartApp();

        if (m_trafficDetails->lastError().type() != 0 ||
                m_autostart->lastError().type() != 0 ||
                m_netcontrol->lastError().type() != 0 ||
                m_monitornetflow->lastError().type() != 0 ||
                m_analysisapp->lastError().type() != 0)
        {
            qDebug() << "dbus error. trafficDetails : " <<  m_trafficDetails->lastError().name() << m_trafficDetails->lastError().type()
                     << " autostart error : " << m_autostart->lastError().name() << m_autostart->lastError().type()
                     << " netcontrol error : " << m_netcontrol->lastError().name() << m_netcontrol->lastError().type()
                     << " monitornetflow error : " << m_monitornetflow->lastError().name() << m_monitornetflow->lastError().type()
                     << " analysisapp error : " << m_analysisapp->lastError().name() << m_analysisapp->lastError().type();
        }
    });
    timer->start(1000 * timeInter);
    int loginsafety = m_gset->get(loginSafety).toInt();
    m_adaptor->setSafetygrade(loginsafety);
}

int ServiceDBus::Charactertypes(QString password)
{
    int i = 0;
    int Number_flag = 0;
    int Capital_flag = 0;
    int Small_flag = 0;
    int Symbol_flag = 0;
    QByteArray ba = password.toLatin1();//QString 转换为 char*
    const char *s = ba.data();

    while (*s) {
        qDebug() << "s = " << *s;
        qDebug() << int(ba.at(i));//输出
        if ('0' <= *s && '9' >= *s) {
            Number_flag = 1 ;
        } else if ('A' <= *s && 'Z' >= *s) {
            Capital_flag = 1;
        } else if ('a' <= *s && 'z' >= *s) {
            Small_flag = 1;
        } else {
            Symbol_flag = 1;
        }
        s++;
    }
    i = Number_flag + Capital_flag + Small_flag + Symbol_flag;
    return i;
}

void ServiceDBus::ExitApp()
{
    qDebug() << Q_FUNC_INFO;
    qApp->quit();
}

void ServiceDBus::StartApp()
{
    qDebug() << Q_FUNC_INFO;
}

//退出任何一个服务进程，都需要将daemon退出，否则3秒后该服务进程又会重启
void ServiceDBus::ExitAllService()
{
    qDebug() << Q_FUNC_INFO;
    m_analysisapp->ExitApp();
    m_autostart->ExitApp();
    m_netcontrol->ExitApp();
    m_trafficDetails->ExitApp();
    m_monitornetflow->ExitApp();
    ExitApp();
}

void ServiceDBus::ExitAnalysisService()
{
    qDebug() << Q_FUNC_INFO;
    m_analysisapp->ExitApp();
    ExitApp();
}

void ServiceDBus::ExitAutoStartService()
{
    qDebug() << Q_FUNC_INFO;
    m_autostart->ExitApp();
    ExitApp();
}

void ServiceDBus::ExitNetControlService()
{
    qDebug() << Q_FUNC_INFO;
    m_netcontrol->ExitApp();
    ExitApp();
}

void ServiceDBus::ExitTrafficDetailService()
{
    qDebug() << Q_FUNC_INFO;
    m_trafficDetails->ExitApp();
    ExitApp();
}

void ServiceDBus::ExitMonitorNetFloService()
{
    qDebug() << Q_FUNC_INFO;
    m_monitornetflow->ExitApp();
    ExitApp();
}

void ServiceDBus::SetGsetting(int value)
{
    m_adaptor->setSafetygrade(value);
    m_gset->set(loginSafety, value);
}

QString ServiceDBus::notifySendPassword(const QString &password)
{
    Q_UNUSED(password);
#if 0
    int loginsafety = m_gset->get(loginSafety).toInt();
    QString ret ="";
    switch(static_cast<LoginSecurity>(loginsafety))
    {
        case high :
        {
            if (password.size() >= 8 && Charactertypes(password) >= 3)
                ret = "true";
            else
                ret = getError(high);//QObject::tr("The password must have at least 8 characters, and contain at least 3 of the four available character types: lowercase letters, uppercase letters, numbers, and symbols");
            break;
        }
        case middle :
        {
            if (password.size() >= 6 && Charactertypes(password) >=2)
                ret = "true";
            else
                ret = ret = getError(middle);//QObject::tr("The password must have at least 6 characters, and contain at least 2 of the four available character types: lowercase letters, uppercase letters, numbers, and symbols");
            break;
        }
        case low :
            ret = "true";
        break;
        }

    return ret;
#endif
    return "";
}

QString ServiceDBus::getError(LoginSecurity type)
{
    QString retsult = "false";
    QString language = QLocale::system().name();
    switch(type)
    {
    case high:
        if (language == "zh_CN") {
            retsult = "密码最少8位，至少同时包含小写字母、大写字母、数字、符号中的3种";
        } else {
            retsult = tr("The password must have at least 8 characters, and contain at least 3 of the four available character types: lowercase letters, uppercase letters, numbers, and symbols");
        }
        break;
    case middle:
        if (language == "zh_CN") {
            retsult = "密码最少6位，至少同时包含小写字母、大写字母、数字、符号中的2种";
        } else {
            retsult = tr("The password must have at least 6 characters, and contain at least 2 of the four available character types: lowercase letters, uppercase letters, numbers, and symbols");
        }
        break;
    default:
        break;
    }

    return retsult;
}

QString ServiceDBus::GetPwdError()
{
    int loginsafety = m_gset->get(loginSafety).toInt();
    QString retsult = "false";
    QString language = QLocale::system().name();
    switch(static_cast<LoginSecurity>(loginsafety))
    {
    case high:
        if (language == "zh_CN") {
            retsult = "密码最少8位，至少同时包含小写字母、大写字母、数字、符号中的3种";
        } else {
            retsult = tr("The password must have at least 8 characters, and contain at least 3 of the four available character types: lowercase letters, uppercase letters, numbers, and symbols");
        }
        break;
    case middle:
        if (language == "zh_CN") {
            retsult = "密码最少6位，至少同时包含小写字母、大写字母、数字、符号中的2种";
        } else {
            retsult = tr("The password must have at least 6 characters, and contain at least 2 of the four available character types: lowercase letters, uppercase letters, numbers, and symbols");
        }
        break;
    default:
        break;
    }

    return retsult;
}

int ServiceDBus::GetPwdLen()
{
    int loginsafety = m_gset->get(loginSafety).toInt();
    int ret =0;
    switch(static_cast<LoginSecurity>(loginsafety))
    {
        case high :
        {
            ret = highlen_return;
            break;
        }
        case middle :
        {
            ret = middlelen_return;
            break;
        }
        default:
            break;
    }
    Q_EMIT UpdateUI();
    return ret;
}

int ServiceDBus::GetPwdTypeLen()
{
    int loginsafety = m_gset->get(loginSafety).toInt();
    int ret =0;
    switch(static_cast<LoginSecurity>(loginsafety))
    {
        case high :
        {
            ret = hightypelen_return;
            break;
        }
        case middle :
        {
            ret = middletypelen_return;
            break;
        }
        default:
            break;
    }
    return ret;
}
