/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     wubowen <wubowen_cm@deepin.com>
 *
 * Maintainer: wubowen <wubowen_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "operatedbusdata.h"
#include "common.h"

#include <QDebug>

OperateDBusData::OperateDBusData(QObject *parent)
    : QObject(parent)
    , m_AnalysisAppInter(new AnalysisApp("com.deepin.defender.analysisapp", "/com/deepin/defender/analysisapp", QDBusConnection::sessionBus(), this))
{
    m_adaptpr = new NetcontrolAdaptor(this);
    if (!QDBusConnection::sessionBus().registerService(service) ||
            !QDBusConnection::sessionBus().registerObject(path, this)) {
        qDebug() << "dbus service already registered! " << service;

        exit(0);
    }

    // 初始化联网管控
    initialize();
}

void OperateDBusData::StartApp()
{
    qDebug() << Q_FUNC_INFO << " Start NetControl.";
}

void OperateDBusData::ExitApp()
{
    qApp->quit();
}

void OperateDBusData::preInitialize()
{
}

void OperateDBusData::initialize()
{
    m_monitor = new NetworkConnectMonitor(this);
    m_model = new NetworkProtectionModel;
    m_worker = new NetworkProtectionWorker(m_model);

    connect(m_worker, &NetworkProtectionWorker::requestShowNetworkSettings, this, &OperateDBusData::showFuncConnectNetControl);

    connect(m_monitor, SIGNAL(requestNetworkConnet(ACLNetwork *, QString, QStringList)),
            m_worker, SLOT(onAppNetworkAccessed(ACLNetwork *, QString, QStringList)), Qt::QueuedConnection);

    connect(m_AnalysisAppInter, &AnalysisApp::notifyUpdateLauncherList, m_worker, &NetworkProtectionWorker::updateLauncherList);
}

void OperateDBusData::showFuncConnectNetControlInter(QString appNamePath)
{
    QStringList sNamePathList = appNamePath.split("|");

    showFuncConnectNetControl(sNamePathList);
}

void OperateDBusData::showFuncConnectNetControl(const QStringList &appNamePath)
{
    m_worker->updateLauncherList();
    if (m_NetDialog == nullptr) {
        m_NetDialog = new FuncConnNetControl(m_model, m_worker);
        connect(m_NetDialog, &FuncConnNetControl::onDialogClose, this, &OperateDBusData::doDialogClose);
        m_NetDialog->setAttribute(Qt::WA_DeleteOnClose);
        m_NetDialog->setWindowFlags(m_NetDialog->windowFlags() | Qt::WindowStaysOnTopHint);
        m_NetDialog->setModal(true);
        m_NetDialog->setDialogColor(m_sColor);
        m_NetDialog->setVisible(true);
        m_NetDialog->setEnabled(true);
    }
    if (!appNamePath.isEmpty())
        m_NetDialog->setCurrentApp(appNamePath);
}

void OperateDBusData::doDialogClose()
{
    m_NetDialog = nullptr;
}

void OperateDBusData::setConnCunrrentIndex(int index)
{
    qDebug() << " funConn index : " << index;
    m_model->setNetworkAccessOption(static_cast<NetworkProtectionModel::NetworkAccessOption>(index));

}
int OperateDBusData::getConnCunrrentIndex()
{
    return m_model->getNetworkAccessOption();
}

void OperateDBusData::setQuitDefender(bool state)
{
    qDebug() << Q_FUNC_INFO << state;

    m_adaptpr->setQuitDefender(state);
}

void OperateDBusData::setColor(QString sColor)
{
    m_sColor = sColor;

    if (m_NetDialog != nullptr){
        m_NetDialog->setDialogColor(m_sColor);
    }
}
