/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     wubowen <wubowen_cm@deepin.com>
 *
 * Maintainer: wubowen <wubowen_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "funcconnnetcontrol.h"
#include "networkaccesscontroldelegate.h"
#include "window/modules/networkprotection/networkprotectionmodel.h"
#include "window/modules/networkprotection/networkprotectionworker.h"

#include <DPinyin>
#include <DStandardItem>
#include <DStyle>
#include <DPalette>
#include <DFontSizeManager>
#include <DApplicationHelper>

#include <QVBoxLayout>
#include <QStandardItemModel>
#include <QComboBox>
#include <QTableView>
#include <QHeaderView>
#include <QLabel>
#include <QColor>
#include <QVariant>
#include <QApplication>
#include <QDebug>

DWIDGET_USE_NAMESPACE
DEF_USING_NAMESPACE
DEF_USING_ANTIVIRUS_NAMESPACE

FuncConnNetControl::FuncConnNetControl(NetworkProtectionModel *model,
                                       NetworkProtectionWorker *worker,
                                       QWidget *parent)
    : DDialog(parent)
    , m_model(new QStandardItemModel(this))
    , m_delegate(new TableViewItemDelegate(this))
    , m_view(new QTableView)
    , m_npModel(model)
    , m_worker(worker)
{
    setFixedWidth(587);
    setFixedHeight(458);
    setOnButtonClickedClose(true);
    setModal(true);
    this->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

    // 添加信息栏
    QLabel *title = new QLabel(tr("Internet Access"));
    QFont ft = title->font();
    ft.setBold(true);
    title->setFont(ft);
    DFontSizeManager::instance()->bind(title, DFontSizeManager::T6);

    m_view->setModel(m_model);
    m_view->setItemDelegate(m_delegate);
    m_view->horizontalHeader()->setVisible(true);
    //m_view->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);
    m_view->horizontalHeader()->setSortIndicator(0, Qt::AscendingOrder);
    m_view->setShowGrid(false);
    m_view->setSortingEnabled(true);
    m_view->setFrameShape(QTableView::NoFrame);
    m_view->setEditTriggers(QAbstractItemView::NoEditTriggers);
    m_view->setSelectionMode(QAbstractItemView::NoSelection);
    m_view->verticalHeader()->hide();
    m_view->verticalHeader()->setDefaultSectionSize(48);
    m_view->setSelectionMode(QAbstractItemView::NoSelection);

    onAppsAccessOptionsChanged(m_worker->getRunningProcessOption());
    connect(m_npModel, &NetworkProtectionModel::appAccessOptionsChanged, this,
            &FuncConnNetControl::onAppsAccessOptionsChanged);

    DFrame *tableFrame = new DFrame;
    QVBoxLayout *widgetLayout = new QVBoxLayout;
    widgetLayout->setContentsMargins(4, 4, 4, 4);
    widgetLayout->addWidget(m_view);
    tableFrame->setLayout(widgetLayout);

    addContent(title);
    addContent(tableFrame);
    setContentLayoutContentsMargins(QMargins(38, 0, 39, 38));
    setSpacing(10);
}

void FuncConnNetControl::setDialogColor(QString sColor)
{
    DGuiApplicationHelper::instance()->setPaletteType(DGuiApplicationHelper::ColorType(sColor.toInt()));
}

FuncConnNetControl::~ FuncConnNetControl()
{
    Q_EMIT onDialogClose();
}

void FuncConnNetControl::onAppsAccessOptionsChanged(QList<NetworkProtectionModel::AppNetworkAccessStruct> list)
{
    int i = 0;
    int count = m_model->rowCount();
    int column = m_model->columnCount();
    m_model->removeRows(0, count);
    m_model->removeColumns(0, column);

    m_comboToAppName.clear();
    // 添加表格
    QStandardItem *item_0 = new QStandardItem(tr("Name"));
    item_0->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QStandardItem *item_1 = new QStandardItem(tr("Status"));
    item_1->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    m_model->setHorizontalHeaderItem(0, item_0);
    m_model->setHorizontalHeaderItem(1, item_1);
    m_model->setHorizontalHeaderItem(2, new QStandardItem(QString("Icon")));
    m_model->setHorizontalHeaderItem(3, new QStandardItem(QString("TypeName")));
    m_model->setHorizontalHeaderItem(4, new QStandardItem(QString("RealNmae")));


    for (NetworkProtectionModel::AppNetworkAccessStruct appOpt : list) {
        QString sName = DTK_CORE_NAMESPACE::Chinese2Pinyin(appOpt.appName);
        m_model->setItem(i, 0, new QStandardItem(sName));
        m_model->setItem(i, 1, new QStandardItem(QString::number(appOpt.option)));
        m_model->setItem(i, 2, new QStandardItem(appOpt.icon));
        m_model->setItem(i, 3, new QStandardItem(m_worker->getAppTypeName(appOpt.type)));
        m_model->setItem(i, 4, new QStandardItem(appOpt.appName));

        QComboBox *box = new QComboBox;

        m_npModel->getNetworkAccessOption();
        box->addItem(tr("Ask"));
        box->addItem(tr("Allow"));
        box->addItem(tr("Disable"));
        box->addItem(QString(tr("Default (%1)")).arg(m_npModel->getNetworkAccessOptionName()));
        box->setCurrentIndex(appOpt.option);
        connect(box, static_cast<void(QComboBox::*)(int)>(&QComboBox::currentIndexChanged), this, &FuncConnNetControl::onAppOptChanged);

        QWidget *boxw = new QWidget;
        QHBoxLayout *boxLayout = new QHBoxLayout;
        boxLayout->setContentsMargins(6, 6, 6, 6);
        boxLayout->addWidget(box);
        boxw->setLayout(boxLayout);
        m_comboToAppName.insert(box, appOpt.appName);
        m_view->setIndexWidget(m_model->index(i, 1), boxw);
        ++i;
    }

    m_view->horizontalHeader()->setHighlightSections(false);
    m_view->horizontalHeader()->setSortIndicator(0, Qt::DescendingOrder);
    m_view->horizontalHeader()->setSortIndicator(0, Qt::AscendingOrder);

    // 设置前两列列的宽度
    m_view->setColumnWidth(0, 300);
    m_view->setColumnWidth(1, 180);

    // 隐藏多余的列 及设置初始化 排序
    m_view->setColumnHidden(2, true);
    m_view->setColumnHidden(3, true);
    m_view->setColumnHidden(4, true);
    if (!m_currentAppName.isEmpty()) {
        QStringList list;
        list << m_currentAppName;
        setCurrentApp(list);
        m_currentAppName.clear();
    }
}

void FuncConnNetControl::onAppOptChanged(int index)
{
    NetworkProtectionModel::AppsAccessOption accessOpt = static_cast<NetworkProtectionModel::AppsAccessOption>(index);
    QComboBox *box = qobject_cast<QComboBox *>(sender());

    QStringList list;
    list << m_comboToAppName.value(box);
    int i = m_npModel->getAppNetworkIndexbyName(list);
    m_currentAppName = m_comboToAppName.value(box);
    qDebug() << "m_comboToAppName.value(box) = " << i << " index = " << index;
    m_worker->setRunningProcessOption(i, accessOpt);
}

QPixmap FuncConnNetControl::getPixmap(const QString &appIcon, const QSize &size)
{
    QIcon icon = QIcon::fromTheme(appIcon, QIcon::fromTheme("application-x-desktop"));
    const qreal ratio = devicePixelRatioF();
    QPixmap pixmap = icon.pixmap(size * ratio).scaled(size * ratio, Qt::KeepAspectRatio, Qt::SmoothTransformation);
    pixmap.setDevicePixelRatio(ratio);
    return pixmap;
}

void FuncConnNetControl::setCurrentApp(const QStringList &appNamePath)
{
    for (int i = 0; i < m_model->rowCount(); ++i) {
        QModelIndex index = m_model->index(i,  1);
        QStandardItem *item = m_model->item(i, 4);

        if (item == nullptr)
            continue;

        if ((appNamePath.size() > 0) && (item->data(Qt::DisplayRole).toString() == appNamePath[0])) {
            m_view->setCurrentIndex(index);
            break;
        } else if ((appNamePath.size() == 2) && (item->data(Qt::DisplayRole).toString() == appNamePath[1])) {
            m_view->setCurrentIndex(index);
            break;
        }
    }
}

TableViewItemDelegate::TableViewItemDelegate(FuncConnNetControl *widget, QObject *parent)
    : QItemDelegate(parent)
    , m_widget(widget)
{
}

void TableViewItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    if (index.row() % 2  == 0) {
        QBrush brush(QColor(0, 0, 0, 8));
        painter->fillRect(option.rect, brush);
    }

    int Pix_Inteval = 2;
    QRect rect = option.rect;
    // 第一列
    if (index.column() == 0) {
        // 设置图片
        int pix_x = rect.x() + 20;
        int pix_y = rect.y() + rect.height() / 2 - 14;
        int height = rect.height() - Pix_Inteval * 2;

        QString iconName = index.model()->data(index.model()->index(index.row(), 2), Qt::DisplayRole).toString();
        QPixmap pixmap = m_widget->getPixmap(iconName, QSize(28, 28));

        painter->drawPixmap(pix_x, pix_y, pixmap);

        // 绘画文字的坐标
        painter->save();
        QFont ft = painter->font();
        int textWidth = rect.width() - height - Pix_Inteval * 2;
        int x = pix_x + 38;
        int y = pix_y + height / 4;
        QString sName = index.model()->data(index.model()->index(index.row(), 4), Qt::DisplayRole).toString();
        QString sElidedName = painter->fontMetrics().elidedText(sName, Qt::ElideRight, textWidth, Qt::TextShowMnemonic);
        painter->drawText(x, y, sElidedName);
        y += rect.height() / 2;
        ft.setPixelSize(12);
        painter->setFont(ft);
        painter->setPen("#526A7F");
        painter->drawText(x, y, index.model()->data(index.model()->index(index.row(), 3), Qt::DisplayRole).toString());
        painter->restore();
    } else {
    }
}
