/*
 * Copyright (C) 2017 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caimengci <caimengci_cm@deepin.com>
 *
 * Maintainer: caimengci <caimengci_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "networkaccesscontroldelegate.h"

#include <DStyle>

#include <QPainter>
#include <QApplication>
#include <QMouseEvent>
#include <QDebug>

DWIDGET_USE_NAMESPACE

static inline QFlags<Qt::AlignmentFlag> alignmentFlag(int col)
{
    auto emCol = static_cast<NetworkAccessControlDelegate::AppAccessControlColumn>(col);
    switch (emCol) {
    case NetworkAccessControlDelegate::ItemAppName:
    case NetworkAccessControlDelegate::ItemAsk:
    case NetworkAccessControlDelegate::ItemAllow:
    case NetworkAccessControlDelegate::ItemDisallow:
    case NetworkAccessControlDelegate::ItemDefault:
        return (Qt::AlignLeft | Qt::AlignVCenter);
    case NetworkAccessControlDelegate::ColumnButt:
        break;
    }
    return (Qt::AlignLeft | Qt::AlignVCenter);
}

static inline QRect colRect(int col, const QStyleOptionViewItem &option)
{
    static auto tailwidth  = 10;
    auto w = option.rect.width() - tailwidth;

    auto emCol = static_cast<NetworkAccessControlDelegate::AppAccessControlColumn>(col);
    switch (emCol) {
    case NetworkAccessControlDelegate::ItemAppName:
        return QRect(0, option.rect.y(), w * 3 / 8, option.rect.height());
    case NetworkAccessControlDelegate::ItemAsk:
        return QRect(w * 3 / 8, option.rect.y(), w / 8, option.rect.height());
    case NetworkAccessControlDelegate::ItemAllow:
        return QRect(w / 2, option.rect.y(), w / 8, option.rect.height());
    case NetworkAccessControlDelegate::ItemDisallow:
        return QRect(w * 5 / 8, option.rect.y(), w / 8, option.rect.height());
    case NetworkAccessControlDelegate::ItemDefault:
        return QRect(w * 3 / 4, option.rect.y(), w / 4, option.rect.height());
    case NetworkAccessControlDelegate::ColumnButt:
        break;
    }
    return option.rect.marginsRemoved(QMargins(0, 0, 0, 0));
}

NetworkAccessControlDelegate::NetworkAccessControlDelegate(QAbstractItemView *parent)
    : DStyledItemDelegate(parent)
{

}

void NetworkAccessControlDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    painter->save();
    painter->setRenderHint(QPainter::Antialiasing);

    DStyle *sty = qobject_cast<DStyle *>(QApplication::style());
    auto ico = sty->standardIcon(DStyle::SP_IndicatorSearch);
    auto title = index.data(ItemAppNameRole).toString();
    QStringList optionList;
    optionList << tr("Ask") << tr("Allow") << tr("Disable") << tr("Default (%1)").arg("Ask");

    for (int col = 0; col < ColumnButt; ++col) {
        auto flag = alignmentFlag(col);
        auto rect = colRect(col, option);
        switch (col) {
        case ItemAppName: {
            auto sz = QSizeF();
            if (!title.isEmpty())
                sz = QSizeF(15, 15);
            auto icon = ico.pixmap(sz.toSize());
            auto centerF = QRectF(rect).center();
            auto iconRect = QRectF(rect.x(), centerF.y() - sz.height() / 2, sz.width(), sz.height());
            auto txtRect = QRectF(10 + sz.width(), rect.y(), rect.width() - 10 - sz.width(), rect.height());
            if (!title.isEmpty())
                painter->drawPixmap(iconRect, icon, icon.rect());
            QFont font(painter->font());
            QFontMetrics fm(font);
            auto text = fm.elidedText(index.data(ItemAppNameRole).toString(), Qt::ElideMiddle, rect.width() - 10 - sz.width());
//            painter->drawText(txtRect, flag, text);
            break;
        }
        case ItemAsk:
        case ItemAllow:
        case ItemDisallow:
        case ItemDefault: {
            QFont font(painter->font());
            QFontMetrics fm(font);
            auto text = fm.elidedText(optionList[col - 1], Qt::ElideMiddle, rect.width());
            if (title.isEmpty()) {
                painter->drawText(rect, flag, text);
                break;
            }
            bool checked = false;
            int type = index.data(ItemAccessOptionRole).toInt();
            if (col == type + 1)
                checked = true;
            QStyleOptionButton check_box_style_option;
            check_box_style_option.text = text;
            check_box_style_option.state |= QStyle::State_Enabled;
            if (checked) {
                check_box_style_option.state |= QStyle::State_On;
            } else {
                check_box_style_option.state |= QStyle::State_Off;
            }
            check_box_style_option.rect = rect;
            QApplication::style()->drawControl(QStyle::CE_RadioButton, &check_box_style_option, painter);
            break;
        }
        default:
            break;
        }
    }
    painter->restore();
    DStyledItemDelegate::paint(painter, option, index);
}

bool NetworkAccessControlDelegate::editorEvent(QEvent *event, QAbstractItemModel *model,
                                               const QStyleOptionViewItem &option, const QModelIndex &index)
{
    QRect decorationRect = option.rect;
    QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
    if (event->type() == QEvent::MouseButtonPress && decorationRect.contains(mouseEvent->pos())) {
        int col = colSelectedd(option, mouseEvent->pos());
        if (col > 0) {
            model->setData(index, col, ItemAccessOptionRole);
        }
    }
    return QStyledItemDelegate::editorEvent(event, model, option, index);
}

int NetworkAccessControlDelegate::colSelectedd(const QStyleOptionViewItem &option, const QPoint &pos)
{
    for (int i = ItemAsk; i < ColumnButt; ++i) {
        QRect r = colRect(i, option);
        if (pos.x() > r.x() && pos.x() <= r.x() + r.width()
                && pos.y() > r.y() && pos.y() <= r.y() + r.height()) {
            return (i - ItemAsk);
        }
    }
    return -1;
}
