/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caimengci <caimengci_cm@deepin.com>
 *
 * Maintainer: caimengci <caimengci_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "networkconnectmonitor.h"
#include "csingleteon.h"
#include "operatedbusdata.h"

#include <QDebug>

NetworkConnectMonitor::NetworkConnectMonitor(OperateDBusData *dbus, QObject *parent)
    : QObject(parent)
    , m_aclInter(new ACLNetwork("com.deepin.daemon.ACL", "/com/deepin/daemon/ACL/Network", QDBusConnection::systemBus(), this))
    , m_AnalysisAppInter(new AnalysisApp("com.deepin.defender.analysisapp", "/com/deepin/defender/analysisapp", QDBusConnection::sessionBus(), this))
{
    QThread *thread = new QThread;
    moveToThread(thread);
    thread->start();

    // ACL信号接收
    connect(m_aclInter, &ACLNetwork::AccessRequest, this, &NetworkConnectMonitor::onAppNetworkConnect);

    connect(this, &NetworkConnectMonitor::registerReply, this, [ = ]() {
        QDBusPendingReply<bool> reply = m_aclInter->Register(QCoreApplication::applicationFilePath(), QCoreApplication::applicationPid(), 20);
        reply.waitForFinished();
        qDebug() << "NetworkConnectMonitor register result : " << reply.value() << QCoreApplication::applicationFilePath() << QCoreApplication::applicationPid();
        if (!reply) {
            qDebug() << "Close Defender" ;
            qApp->quit();
        }

        //reply : false -> close deepin-defender
        //reply : true  -> normal
        dbus->setQuitDefender(!reply);
    });
    Q_EMIT registerReply();

    connect(this, &NetworkConnectMonitor::reqStartTimer, this, &NetworkConnectMonitor::onReqStartTimer);
    Q_EMIT reqStartTimer();
}

void NetworkConnectMonitor::deleteProcessDesktop()
{
    QList<QString> list;
    QMap<QString, QStringList>::iterator it;
    for (it = m_saveProcess_Desktop.begin(); it != m_saveProcess_Desktop.end(); ++it) {
        QStringList saveList = it.key().split("&");
        if (saveList.count() >= 0) {
            qint64 timer = QDateTime::currentSecsSinceEpoch();
            if (timer - saveList.at(1).toInt() > 60)
                list.append(it.key());
        }
    }

    {
        DEFENDER_MUTEX_LOCKER(&SMutex::getInstance().m_mutex);
        for (int i = 0; i < list.count(); ++i) {
            m_saveProcess_Desktop.remove(list.at(i));
        }
    }

    qDebug() << "NetworkConnectMonitor SaveProcessDesktop Count : " << m_saveProcess_Desktop.count();
}

//不能在主线程new对象，给子线程使用
//否则会报警告：QObject: Cannot create children for a parent that is in a different thread.
void NetworkConnectMonitor::onReqStartTimer()
{
    // 安装定时器
    m_showTimer = new QTimer(this);
    connect(m_showTimer, SIGNAL(timeout()), this, SLOT(deleteProcessDesktop()));
    m_showTimer->setInterval(1000 * 30);
    // 启动定时器
    m_showTimer->start();
}

void NetworkConnectMonitor::onAppNetworkConnect(const QString &id)
{
    if (id == "") {
        m_aclInter->Enable(id, true);
        return;
    }

    qDebug() << "NetworkConnectMonitor ACl Signal Value : " << id;

    // 取pid 进程号
    QStringList sProList = id.split("-");
    QString sPid;
    if (sProList.count() >= 3) sPid = sProList.at(2);

    QStringList sListDesktop;

    // 取进程--包名缓存
    QMap<QString, QStringList>::iterator it;
    for (it = m_saveProcess_Desktop.begin(); it != m_saveProcess_Desktop.end(); ++it) {
        QStringList saveList = it.key().split("&");
        if (saveList.count() >= 0) {
            if (saveList.at(0) == sPid) {
                sListDesktop = it.value();
            }
        }
    }

    if (sListDesktop.count() <= 0)
        getDesktopName(id);
    else
        Q_EMIT requestNetworkConnet(m_aclInter, id, sListDesktop);
}

void NetworkConnectMonitor::getDesktopName(const QString &id)
{
    // 取pid 进程号
    QStringList sProList = id.split("-");
    QString sPid;
    if (sProList.count() >= 3) sPid = sProList.at(2);

    // 取进程执行命令行路径
    QFile file(QString("/proc/%1/exe").arg(sPid));
    QString sCmdLinePath;
    if (file.open(QFile::ReadOnly)) {
        sCmdLinePath = file.readLink();
        sCmdLinePath = sCmdLinePath.split(" ").at(0);
        file.close();
    }
    qDebug() << "caimengci: sCmdLinePath: " << sCmdLinePath ;

    QStringList sListDesktop;
    // 判断第三方应用
    QStringList list = sCmdLinePath.split("/");
    if (list.at(list.count() - 1) == "wineserver.real") {
        QString sDeskTop = m_AnalysisAppInter->getProcessEnvironmentVariable(sPid.toInt(), "GIO_LAUNCHED_DESKTOP_FILE");
        qDebug() << "caimengci: sProcL: " << sDeskTop;
        QStringList s = sDeskTop.split("/");
        sListDesktop.append(s.at(s.count() - 1));
        Q_EMIT requestNetworkConnet(m_aclInter, id, sListDesktop);
        return;
    }

    // 查找包名
    QProcess *processS = new QProcess(this);
    processS->start("dpkg", QStringList() << "-S" << sCmdLinePath);
    processS->waitForStarted();
    processS->waitForFinished();
    QString sProcessResultS = QString::fromLocal8Bit(processS->readAllStandardOutput()).trimmed();

    QStringList sListS = sProcessResultS.split(":");
    if (sListS.count() > 0) {
        QProcess *processL = new QProcess(this);
        processL->start("dpkg", QStringList() << "-L" << sListS.at(0));
        processL->waitForStarted();
        processL->waitForFinished();
        QString sProcessResultL = QString::fromLocal8Bit(processL->readAllStandardOutput()).trimmed();
        QStringList sList = sProcessResultL.split("\n");
        for (int i = 0; i < sList.count(); ++i) {
            if (sList.at(i).contains("desktop")) {
                QStringList sListProcL = sList.at(i).split("/");
                qDebug() << "caimengci: sProcLList: " << sListProcL.at(sListProcL.count() - 1);
                sListDesktop.append(sListProcL.at(sListProcL.count() - 1));
            }
        }
    }
    Q_EMIT requestNetworkConnet(m_aclInter, id, sListDesktop);

    qint64 timer = QDateTime::currentSecsSinceEpoch();
    QString sPisAddTimer = sPid + "&" + QString::number(timer);

    {
        DEFENDER_MUTEX_LOCKER(&SMutex::getInstance().m_mutex);
        m_saveProcess_Desktop[sPisAddTimer] = sListDesktop;
    }
}
