/*
 * Copyright (C) 2017 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caimengci <caimengci_cm@deepin.com>
 *
 * Maintainer: caimengci <caimengci_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "networkprotectionworker.h"
#include "networkprotectionmodel.h"
#include "reminderdialog.h"

#include <QFile>
#include <QProcess>
#include <QLabel>
#include <QMessageBox>
#include <QDateTime>
#include <QCoreApplication>
#include <QDebug>

NetworkProtectionWorker::NetworkProtectionWorker(NetworkProtectionModel *model, QObject *parent)
    : QObject(parent)
    , m_model(model)
    , m_launcherInter(new Launcherd("com.deepin.dde.daemon.Launcher", "/com/deepin/dde/daemon/Launcher", QDBusConnection::sessionBus(), this))
    , m_ask(false)
{
    connect(this, &NetworkProtectionWorker::notifyUpdateLauncherList, this, &NetworkProtectionWorker::getRunningProcessOption);
    //获取本机当前启动的全部pid
    Q_EMIT notifyUpdateAllProcessPidName();

    //第一次获取数据
    updateLauncherList();
}

void NetworkProtectionWorker::reset()
{
    m_model->reset();
}

QList<NetworkProtectionModel::AppNetworkAccessStruct> NetworkProtectionWorker::getRunningProcessOption()
{
    QList<NetworkProtectionWorker::AppIcon> launcherAppList = getAllApps();

    QList<NetworkProtectionModel::AppNetworkAccessStruct> modelAppList = m_model->getRunningProcessOption();

    QList<NetworkProtectionModel::AppNetworkAccessStruct> list;

    for (int i = 0; i < launcherAppList.size(); ++i) {
        NetworkProtectionModel::AppNetworkAccessStruct item = {
            launcherAppList[i].appName,
            launcherAppList[i].icon,
            launcherAppList[i].path,
            launcherAppList[i].type,
            NetworkProtectionModel::ItemDefault
        };

        if (item.type == NetworkProtectionModel::SystemApp)
            item.option = NetworkProtectionModel::ItemAllow;

        for (int j = 0; j < modelAppList.size(); ++j) {
            if (item.path == modelAppList[j].path) {
                item.option = modelAppList[j].option;
                break;
            }
        }
        list << item;
    }

    m_model->setRunningProcessOption(list);
    return list;
}

void NetworkProtectionWorker::setRunningProcessOption(QList<NetworkProtectionModel::AppNetworkAccessStruct> list)
{
    m_model->setRunningProcessOption(list);
}

QStringList NetworkProtectionWorker::getAllProcess()
{
    QStringList strList;

    QProcess p;
    p.start("ls /proc");
    p.waitForFinished();
    QString allPids = p.readAllStandardOutput();
    QStringList pidlist = allPids.split("\n");
    for (QString pidStr : pidlist) {
        bool ret = false;
        pid_t pid = pidStr.toInt(&ret);
        QString appname;
        if (!ret)
            continue;
        appname = getProcNameByPid(pid);
        strList << appname;
    }

    return strList;
}

QList<NetworkProtectionWorker::AppIcon> NetworkProtectionWorker::getAllApps()
{
    QDBusPendingReply<LauncherItemInfoList> reply = getLauncherList();
    reply.waitForFinished();
    if (reply.isError()) {
        qWarning() << "data is empty, quit!!";
        qWarning() << reply.error();
        qApp->quit();
    }

    const LauncherItemInfoList &datas = reply.value();
    QList<AppIcon> apps;

    for (const auto &it : datas) {
        AppIcon app;
        app.appName = it.Name;
        app.icon = it.Icon;
        app.path = it.Path;
        app.type = getAppType(it.Path);
        apps << app;
    }

    //add ping for test
//    AppIcon app;
//    app.appName = "ping";
//    app.type = NetworkProtectionModel::SystemApp;
//    app.path = "ping";
//    apps << app;

    return apps;
}

QString NetworkProtectionWorker::getProcNameByPid(pid_t pid)
{
    QString name = "";
    QFile file(QString("/proc/%1/status").arg(pid));
    if (file.open(QFile::ReadOnly)) {
        QString firstline = file.readLine();
        QStringList list = QString(firstline).split(":");
        name = list.count() > 1 ? list.at(1).trimmed() : "";
        file.close();
    }
    return name;
}

int NetworkProtectionWorker::getParentPidByPid(pid_t pid)
{
    pid_t ppid = 0;
    QFile file(QString("/proc/%1/status").arg(pid));
    if (file.open(QFile::ReadOnly)) {
        QString firstline = "";
        while (!(firstline = file.readLine()).startsWith("PPid") && (!firstline.isEmpty()));
        if (firstline.startsWith("PPid")) {
            QStringList list = QString(firstline).split(":");
            ppid = list.count() > 1 ? list.at(1).trimmed().toInt() : 0;
        }
        file.close();
    }
    return ppid;
}

int NetworkProtectionWorker::getPidById(const QString &id)
{
    if (id.isEmpty())
        return -1;

    QStringList list = id.split("-");

    if (list.size() < 3)
        return -1;

    return list[2].toInt();
}

void NetworkProtectionWorker::setRunningProcessOption(int index, NetworkProtectionModel::AppsAccessOption option)
{
    m_model->setAppNetworkOption(index, option);
}

NetworkProtectionModel::AppType NetworkProtectionWorker::getAppType(const QString &path)
{
    NetworkProtectionModel::AppType type = NetworkProtectionModel::SystemApp;
    QFile fileInfo(path);
    if (QFileInfo(path).isSymLink()) {
        type = NetworkProtectionModel::ThirdPartyApp;
    } else if (fileInfo.open(QIODevice::ReadOnly | QIODevice::Text)) {
        QTextStream in(&fileInfo);
        in.setCodec("UTF-8");
        QString sAllInfo = in.readAll();
        int nIndex = sAllInfo.indexOf("Exec");
        QString sOptInfo = sAllInfo.mid(nIndex, 10);
        if (sOptInfo.contains("opt"))
            type = NetworkProtectionModel::ThirdPartyApp;
    }

    return type;
}

QString NetworkProtectionWorker::validateString(const QString &str)
{
    QString validateStr = str.simplified();
    int space = validateStr.indexOf(' ');

    if (space >= 0)
        validateStr = validateStr.left(space);

    if (validateStr.contains("\""))
        validateStr = validateStr.remove("\"");

    if (validateStr.contains("\'"))
        validateStr = validateStr.remove("\'");

    space = validateStr.indexOf("--");
    if (space >= 0)
        validateStr = validateStr.left(space);

    return validateStr;
}

QString NetworkProtectionWorker::getDesktopExec(const QString &path)
{
    QString exec;
    QFile fileInfo(path);

    if (fileInfo.open(QIODevice::ReadOnly | QIODevice::Text)) {
        QString txtline;
        while (!(txtline = fileInfo.readLine()).isEmpty()) {
            if (txtline.startsWith("Exec")) {
                QStringList execlist = txtline.split("=");
                if (execlist.size() == 2) {
                    exec = validateString(execlist[1]);
                    QFileInfo fi(exec);
                    if (fi.isSymLink())
                        exec = fi.symLinkTarget();
                    QString filename = fileInfo.fileName();
                    if (filename.endsWith("-stable") || filename.endsWith("-en") || filename.endsWith("-cn")) {
                        exec = exec.remove("-");
                        exec = exec.remove("_");
                    }
                    break;
                }
            }
        }

        fileInfo.close();
    }

    return exec;
}

QString NetworkProtectionWorker::getCmdline(pid_t pid)
{
    QFile file(QString("/proc/%1/cmdline").arg(pid));
    QString cmdline;
    if (file.open(QFile::ReadOnly)) {
        cmdline = file.readLine();

        file.close();
    }
    return validateString(cmdline);
}

QString NetworkProtectionWorker::getAppTypeName(NetworkProtectionModel::AppType type)
{
    switch (type) {
    case NetworkProtectionModel::SystemApp:
        return (tr("System"));
    case NetworkProtectionModel::ThirdPartyApp:
        return (tr("Third-party"));
    }
    return "";
}

void NetworkProtectionWorker::showDialog(bool ask, const QString &id, const QString &appName)
{
    if (m_processing)
        return;

    m_processing = true;
    m_currentId = id;

    ReminderDialog *reminderDialog = nullptr;
    if (ask) {
        reminderDialog = new ReminderDialog(id, appName);
    } else {
        reminderDialog = new ReminderDialog(id, appName, false, true);
    }

    reminderDialog->setAttribute(Qt::WA_DeleteOnClose);
    //设置点击按钮立即关闭当前页面
    reminderDialog->setOnButtonClickedClose(true);
    connect(reminderDialog, &ReminderDialog::notifyButtonClicked, this, &NetworkProtectionWorker::onAppNetworkAccessResult);
    connect(reminderDialog, &ReminderDialog::notifyTimedout, this, &NetworkProtectionWorker::onAppNetworkAccessTimeout);

    reminderDialog->show();
    QTimer::singleShot(100, this, [ = ]() {
        reminderDialog->grabMouseAndKeyboardFocus();
    });
}

void NetworkProtectionWorker::showNetworkAccessSettings(const QStringList &appNamePath)
{
    Q_EMIT requestShowNetworkSettings(appNamePath);
}

QStringList NetworkProtectionWorker::getNamePathById(const QString &id)
{
    return  m_savaAppPath.value(id);
}

//获取当前目录全部文件(不包括文件夹)
QVector<int> NetworkProtectionWorker::GetFileList(QString dir)
{
    QVector<int> filePath;
    QDir dirPath(dir);
    QList<QFileInfo> file(dirPath.entryInfoList());
    QString name = "";
    int processaId = -1;

    for (auto it = file.begin(); it != file.end(); it++) {
        name = it->fileName();
        processaId = name.toInt();

        //(*it).isFile()可以获取全部文件名称，(*it).isDir()获取全部文件夹名称
        if ((*it).isDir() && processaId) {
            filePath.push_back(processaId);
        }
    }

    return filePath;
}

QDBusPendingReply<LauncherItemInfoList> NetworkProtectionWorker::getLauncherList()
{
    return m_launcherInter->GetAllItemInfos();
}

void NetworkProtectionWorker::onAppNetworkAccessed(ACLNetwork *aclInter, QString sId, QStringList sAppName)
{
    m_aclInter = aclInter;
    if (sAppName.count() <= 0)
        m_aclInter->Enable(sId, true);

    QStringList appNamePath;
    for (NetworkProtectionModel::AppNetworkAccessStruct appAccess : m_model->getRunningProcessOption()) {
        QStringList sAppPath = appAccess.path.split("/");
        QString sAppPathName = sAppPath.at(sAppPath.count() - 1);

        for (int i = 0; i < sAppName.count(); ++i) {
            if (sAppPathName == sAppName.at(i)) {
                appNamePath.append(appAccess.appName);
                appNamePath.append(appAccess.path);
                m_savaAppPath[sId] = appNamePath;
                break;
            }
        }
    }

    if (appNamePath.isEmpty() || (appNamePath.size() != 2)) {
        QString procName = getProcNameByPid(getPidById(sId));
        m_aclInter->Enable(sId, true);
        return;
    }
    pid_t pid = getPidById(sId);
    pid_t ppid = getParentPidByPid(pid);
    QString ppName = getProcNameByPid(ppid);

    NetworkProtectionModel::AppsAccessOption acOpt = m_model->getAppNetworkAccessOption(appNamePath[1]);

    if (acOpt == NetworkProtectionModel::ItemAllow) {
        m_aclInter->Enable(sId, true);
        return;
    }

    if (m_processing) {
        return;
    }

    if (acOpt == NetworkProtectionModel::ItemAsk) {
        if (m_appAskTime.contains(appNamePath[0])) {
            qint64 ago = m_appAskTime.value(appNamePath[0]);
            //超过时间范围继续弹对话框
            if (QDateTime::currentSecsSinceEpoch() - ago > 15) {
                m_appAskTime.remove(appNamePath[0]);
                m_ask = true;
                showDialog(m_ask, sId, appNamePath[0]);
            } else {
                m_ask = true;
                m_aclInter->Enable(sId, true);
            }
        } else if (m_appDisableTime.contains(appNamePath[0])) {
            qint64 ago = m_appDisableTime.value(appNamePath[0]);
            //超过时间范围继续弹对话框
            if (QDateTime::currentSecsSinceEpoch() - ago > 15) {
                m_appDisableTime.remove(appNamePath[0]);
                m_ask = true;
                showDialog(m_ask, sId, appNamePath[0]);
            } else {
                m_ask = true;
                m_aclInter->Enable(sId, false);
            }
        } else {
            m_ask = true;
            showDialog(m_ask, sId, appNamePath[0]);
        }
    } else {
        m_aclInter->Enable(sId, false);
        if (m_appDisableTime.contains(appNamePath[0])) {
            qint64 ago = m_appDisableTime.value(appNamePath[0]);
            //超过时间范围继续弹对话框
            if (QDateTime::currentSecsSinceEpoch() - ago > 15) {
                m_appDisableTime.remove(appNamePath[0]);
                m_ask = false;
                showDialog(m_ask, sId, appNamePath[0]);
            } else {
                m_ask = false;
                m_aclInter->Enable(sId, false);
            }
        } else {
            m_ask = false;
            m_aclInter->Enable(sId, false);
            showDialog(m_ask, sId, appNamePath[0]);
        }
    }
}

void NetworkProtectionWorker::onAppNetworkAccessResult(const QString &id, int index)
{
    QStringList appNamePath = getNamePathById(id);
    pid_t pid = getPidById(id);

    if (m_ask) {
        if (index == 2) {
            m_aclInter->Enable(id, false);
            if (!m_appDisableTime.contains(appNamePath[0])) {
                m_appDisableTime.insert(appNamePath[0], QDateTime::currentSecsSinceEpoch());
            }
        } else if (index == 0) {
            m_aclInter->Enable(id, true);
            m_model->setAppAskOption(appNamePath, NetworkProtectionModel::AllowOnce);
            if (!m_appAskTime.contains(appNamePath[0])) {
                m_appAskTime.insert(appNamePath[0], QDateTime::currentSecsSinceEpoch());
            }
        } else if (index == 1) {
            m_aclInter->Enable(id, true);
            m_model->setAppAskOption(appNamePath, NetworkProtectionModel::AllowAlways);
        }
    } else {
        if (!m_appDisableTime.contains(appNamePath[0])) {
            m_appDisableTime.insert(appNamePath[0], QDateTime::currentSecsSinceEpoch());
        }
        if (index == 0) {
            showNetworkAccessSettings(appNamePath);
        }
    }

    m_savaAppPath.clear();
    m_currentId = "";
    m_processing = false;
}

void NetworkProtectionWorker::onAppNetworkAccessTimeout(const QString &id, bool ask)
{
    QStringList appNamePath = getNamePathById(id);
    if (ask) {
        m_aclInter->Enable(id, false);
    }

    m_currentId = "";
    m_processing = false;
}

void NetworkProtectionWorker::updateLauncherList()
{
    //先获取新应用列表
    QDBusPendingReply<LauncherItemInfoList> reply = getLauncherList();

    reply.waitForFinished();
    if (reply.isError()) {
        qWarning() << "data is empty, quit!!";
        qWarning() << reply.error();
        qApp->quit();
    }

    LauncherItemInfoList list = reply.value();

    bool bIsChanged = false;
    int count = list.count();
    int useCount = m_allApplist.count();
    //①若两个列表数量不一样，必然列表有更新
    //②若数量一样，则会将列表排序，逐一按位比较,若有不一样则说明列表有更新(反之没有更新)
    if (count != useCount) {
        bIsChanged = true;
    } else {
        //使用path进行排序
        qSort(list.begin(), list.end(), [](const LauncherItemInfo & a, const LauncherItemInfo & b) {
            return a.Path > b.Path;
        });

        //比较新收到的list和m_allApplist，有不一样的则说明列表更新了(排序过)
        for (int i = 0; i < count; i++) {
            //找到不一样的就退出循环，说明两个列表不一致
            if (list.at(i).Path != m_allApplist.at(i).Path) {
                bIsChanged = true;
                break;
            }
        }
    }

    //列表有改变才会更新列表
    //没有改变则不通知
    if (bIsChanged) {
        DEFENDER_MUTEX_LOCKER(&SMutex::getInstance().m_mutex);
        m_allApplist = list;
        Q_EMIT notifyUpdateLauncherList();
    }
}

