/*
 * Copyright (C) 2017 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caimengci <caimengci_cm@deepin.com>
 *
 * Maintainer: caimengci <caimengci_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#pragma once

#include "window/namespace.h"
#include "networkprotectionmodel.h"
#include "com_deepin_dde_daemon_launcherd.h"
#include "com_deepin_daemon_aclnetwork.h"
#include "csingleteon.h"

#include <QObject>
#include <QMap>

#define Work_Debug_Flag 1

struct AppParameter {
    QString name;
    double recv;
    double send;
    int pid;
    QString device;
    QString type;
    QString pidname;//desktop
    double apptotal = 0;//单个APP总流量
    QString sName;//显示的名字
};

enum FlowRangeStyle {today, yesterday, thisMonth, lastMonth};

Q_DECLARE_METATYPE(AppParameter);

using Launcherd = com::deepin::dde::daemon::Launcher;
using ACLNetwork = com::deepin::daemon::acl::Network;

DEF_NAMESPACE_BEGIN
class ReminderDialog;
class NetworkProtectionWorker : public QObject
{
    Q_OBJECT
public:
    explicit NetworkProtectionWorker(NetworkProtectionModel *model, QObject *parent = nullptr);
    void reset();
    QList<NetworkProtectionModel::AppNetworkAccessStruct> getRunningProcessOption();
    void setRunningProcessOption(QList<NetworkProtectionModel::AppNetworkAccessStruct> list);
    inline QString nameIsMatch(QString name);

    inline Launcherd *getlauncherd()
    {
        return m_launcherInter;
    }

    static QString getProcNameByPid(pid_t pid);
    static int getParentPidByPid(pid_t pid);
    static int getPidById(const QString &id);
    void setRunningProcessOption(int index, NetworkProtectionModel::AppsAccessOption option);
    static NetworkProtectionModel::AppType getAppType(const QString &path);
    static QString validateString(const QString &str);
    static QString getDesktopExec(const QString &path);
    static QString getCmdline(pid_t pid);
    static QString getAppTypeName(NetworkProtectionModel::AppType type);

    void showDialog(bool ask, const QString &id, const QString &appName);
    void showNetworkAccessSettings(const QStringList &appNamePath);
    void updateLauncherList();

    struct AppIcon {
        QString appName;
        QString icon;
        QString path;
        NetworkProtectionModel::AppType type;
    };

private:
    QStringList getAllProcess();
    QList<AppIcon> getAllApps();
    QStringList getNamePathById(const QString &id);
    QVector<int> GetFileList(QString dir);
    QDBusPendingReply<LauncherItemInfoList> getLauncherList();

Q_SIGNALS:
    void requestShowNetworkSettings(const QStringList &appName);
    void notifyUpdateAllProcessPidName();

    void notifyUpdateLauncherList();

private Q_SLOTS:
    void onAppNetworkAccessed(ACLNetwork *aclInter, QString sId, QStringList sAppName);
    void onAppNetworkAccessResult(const QString &id, int index);
    void onAppNetworkAccessTimeout(const QString &id, bool ask);

private:
    NetworkProtectionModel *m_model;
    Launcherd *m_launcherInter;
    ACLNetwork *m_aclInter;

    QString m_currentId;
    volatile bool m_processing;
    bool m_ask;
    QMap<QString, qint64> m_appAskTime;
    QMap<QString, qint64> m_appDisableTime;

    LauncherItemInfoList m_allApplist;//用于获取Launcher里面取出来的全部应用list
    QMap<QString, QStringList> m_savaAppPath;
};
DEF_NAMESPACE_END
