/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     wubowen <wubowen_cm@deepin.com>
 *
 * Maintainer: wubowen <wubowen_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "operatedbusdata.h"
#include "common.h"

#include <QDebug>

OperateDBusData::OperateDBusData(QObject *parent)
    : QObject(parent)
    , m_adaptor(nullptr)
    , m_worker(nullptr)
    , m_useData(nullptr)
{
    registerDefenderDisplayMetaType();
    // 注册dbus信号类型
    registerDefenderAppFlowMetaType();

    m_worker = new TrafficDetailsWorker;
    //获取使用数据库实例
    m_useData = m_worker->getUseData();

    m_adaptor = new TrafficdetailsAdaptor(this);

    if (!QDBusConnection::sessionBus().registerService(service) ||
            !QDBusConnection::sessionBus().registerObject(path, this)) {
        qDebug() << "dbus service already registered! " << service;

        exit(0);
    }

    m_handlethread = new DefenderHandleThread(m_worker);
    m_handlethread->start();

    qRegisterMetaType<QList<allApp>>("QList<allApp>");
    qRegisterMetaType<QList<AppParameter>>("QList<AppParameter");
    connect(m_worker, &TrafficDetailsWorker::DesktopUpdate, m_handlethread, &DefenderHandleThread::UpdateAllProcPid);
    connect(m_handlethread, &DefenderHandleThread::setSaveFlowList, m_worker, &TrafficDetailsWorker::onSetSvaeFLowList);
    connect(m_handlethread, &DefenderHandleThread::SendSpeedData, this, &OperateDBusData::SendDisplaySpeedlist);
    connect(m_handlethread, &DefenderHandleThread::SendFlowData, this, &OperateDBusData::SendDisplayFlowlist);
    connect(m_worker->GetAnalysisapp(), &Analysisapp::notifyUpdateLauncherList, [this] {
        qDebug()<<"notifyUpdateLauncherList";
        Q_EMIT m_adaptor->updatedisplay();
    });

    //链接UseData信号
    connect(m_useData, &UseData::sendAppYesterdayFlowData, this, &OperateDBusData::recAppFlowList);
    connect(m_useData, &UseData::sendAppTodayFlowData, this, &OperateDBusData::recAppFlowList);
    connect(m_useData, &UseData::sendAppLastMonthFlowData, this, &OperateDBusData::recAppFlowList);
    connect(m_useData, &UseData::sendAppThisMonthFlowData, this, &OperateDBusData::recAppFlowList);
}

void OperateDBusData::StartApp()
{
    qDebug() << Q_FUNC_INFO << " Start TrafficDetailis.";
}

void OperateDBusData::ExitApp()
{
    qApp->quit();
}

void OperateDBusData::SendDisplayFlowlist(DefenderDisplayList displayflowlist)
{
    Q_EMIT m_adaptor->displayflow(displayflowlist);
}

void OperateDBusData::SendDisplaySpeedlist(DefenderDisplayList displayspeedlist)
{
    Q_EMIT m_adaptor->displayspped(displayspeedlist);
}

void OperateDBusData::notifyGetYesterdayFlowData()
{
    m_worker->requestGetYesterdayFlowData();
}

void OperateDBusData::notifyGetTodayFlowData()
{
    m_worker->requestGetTodayFlowData();
}

void OperateDBusData::notifyGetLastMonthFlowData()
{
    m_worker->requestGetLastMonthFlowData();
}

void OperateDBusData::notifyGetThisMonthFlowData()
{
    m_worker->requestGetThisMonthFlowData();
}

void OperateDBusData::notifyGetAppYesterdayFlowData(QString appname)
{
    m_useData->notifyGetAppYesterdayFlowData(appname);
}

void OperateDBusData::notifyGetAppTodayFlowData(QString appname)
{
    m_useData->notifyGetAppTodayFlowData(appname);
}

void OperateDBusData::notifyGetAppLastMonthFlowData(QString appname)
{
    m_useData->notifyGetAppLastMonthFlowData(appname);
}

void OperateDBusData::notifyGetAppThisMonthFlowData(QString appname)
{
    m_useData->notifyGetAppThisMonthFlowData(appname);
}

void OperateDBusData::recAppFlowList(QList<AppFlowDateParameter> appFlowLst)
{
    //应用流量详情弹窗接收
    DefenderAppFlowList defenderAppFlowList;
    int id = 0;
    for (AppFlowDateParameter app : appFlowLst) {
        DefenderAppFlow defenderAppFlowTmp;
        defenderAppFlowTmp.id = id;
        defenderAppFlowTmp.name = app.name;
        defenderAppFlowTmp.sname = app.sname;
        defenderAppFlowTmp.timeymdh = app.timeymdh;
        defenderAppFlowTmp.downloaded = app.downloaded;
        defenderAppFlowTmp.uploaded = app.uploaded;
        defenderAppFlowTmp.apptotal = app.apptotal;

        defenderAppFlowList.append(defenderAppFlowTmp);
        id++;
    }
    Q_EMIT m_adaptor->sendAppFlowData(defenderAppFlowList);
}
