/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     luokai <luokai_cm@deepin.com>
 *
 * Maintainer: luokai <luokai_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "trafficdetailsworker.h"

//流量数据缓冲存储间隔，单位s，最大设置为60s，间隔影响流量跨时统计误差
const int SaveFlowDateInterval = 5;

TrafficDetailsWorker::TrafficDetailsWorker(QObject *parent)
    : QObject(parent)
    , m_inter(new Analysisapp("com.deepin.defender.analysisapp", "/com/deepin/defender/analysisapp", QDBusConnection::sessionBus(), this))
    , m_monitornetflow(new MonitorNetFlow("com.deepin.defender.MonitorNetFlow", "/com/deepin/defender/MonitorNetFlow", QDBusConnection::systemBus(), this))
    , m_useData(new UseData)
    , m_saveFlowDateTimer(new QTimer)
{
    //获取数据
    connect(m_monitornetflow, &MonitorNetFlow::NotifyDetailData, this, &TrafficDetailsWorker::onAppMonitorNetFlowDetailData);
//    m_monitornetflow->ExitApp();
//    QThread::currentThread()->msleep(500);
//    for (int i = 0; i < 10; i++)
//        m_monitornetflow->StartApp();

    //初始化默认显示的时间跨度
    m_flowRangeStyle = FlowRangeStyle::today;

    //save flow date
    QThread *m_useDateThread = new QThread;
    m_useData->moveToThread(m_useDateThread);
    m_useDateThread->start();

    m_saveFlowDateTimer->setInterval(1000 * SaveFlowDateInterval);
    m_saveFlowDateTimer->start();
    connect(m_saveFlowDateTimer, &QTimer::timeout, this, &TrafficDetailsWorker::saveFlowData);

    connect(m_useData, &UseData::sendTodayFlowData, this, &TrafficDetailsWorker::onReceiveFlowDate);
    connect(m_useData, &UseData::sendYesterdayFlowData, this, &TrafficDetailsWorker::onReceiveFlowDate);
    connect(m_useData, &UseData::sendThisMonthFlowData, this, &TrafficDetailsWorker::onReceiveFlowDate);
    connect(m_useData, &UseData::sendLastMonthFlowData, this, &TrafficDetailsWorker::onReceiveFlowDate);
    connect(m_monitornetflow, &MonitorNetFlow::NotifyDetailData, this, &TrafficDetailsWorker::onAppMonitorNetFlowDetailData);
    connect(m_inter, &Analysisapp::processInfoList, this, &TrafficDetailsWorker::Getallproclist);
    connect(m_inter, SIGNAL(allProcPidList(DefenderProcessList)), this, SLOT(Getallprocpid(DefenderProcessList)));
    connect(m_inter, &Analysisapp::singleProcDesktopList, this, &TrafficDetailsWorker::Getdesktoplist);
    connect(m_inter, &Analysisapp::guiProcDesktopList,this,&TrafficDetailsWorker::GetGuidesktop);
}

void TrafficDetailsWorker::requestGetYesterdayFlowData()
{
    m_flowRangeStyle = FlowRangeStyle::yesterday;
    Q_EMIT m_useData->notifyGetYesterdayFlowData();
}

void TrafficDetailsWorker::requestGetTodayFlowData()
{
    m_flowRangeStyle = FlowRangeStyle::today;
    Q_EMIT m_useData->notifyGetTodayFlowData();
}

void TrafficDetailsWorker::requestGetLastMonthFlowData()
{
    m_flowRangeStyle = FlowRangeStyle::lastMonth;
    Q_EMIT m_useData->notifyGetLastMonthFlowData();
}

void TrafficDetailsWorker::requestGetThisMonthFlowData()
{
    m_flowRangeStyle = FlowRangeStyle::thisMonth;
    Q_EMIT m_useData->notifyGetThisMonthFlowData();
}

//获取网速/流量并写入对应list
void TrafficDetailsWorker::onAppMonitorNetFlowDetailData(const QString &name, double recv, double send, int pid, const QString &device, const QString &type)
{
    m_parameter.recv = recv;
    m_parameter.send = send;
    m_parameter.name = name;
    m_parameter.pid = pid;
    m_parameter.device = device;
    m_parameter.type = type;
    m_parameter.pidname = "";

    {
        DEFENDER_MUTEX_LOCKER(&SMutex::getInstance().m_mutex);
        if (!m_procpid.contains(pid))//没有打开的应用   //吴博文配合
            return;
    }

    if (pid == 0)
        return;

    m_parameter.pidname = getDetailNameByPid(m_parameter.pid).pidname;//根据pid获取desktop
    m_parameter.sName = getDetailNameByPid(m_parameter.pid).name;//根据pid获取Sname
    m_parameter.pidname = m_parameter.pidname.split("/").last();
    m_parameter.pidname = getSystemDesktop(m_parameter.pidname);
#if Work_Debug_Flag
    qDebug() << "m_parameter.pidname = " << m_parameter.pidname << m_parameter.sName << m_parameter.pid << m_parameter.type
             << m_parameter.recv << m_parameter.send;
#endif
    if ((m_parameter.pid && m_parameter.pidname == "") || !m_parameter.pid) {//pid为0或pid>0能取到pidname
        return;
    }

    {
        DEFENDER_MUTEX_LOCKER(&SMutex::getInstance().m_mutex);
        QMap<QString, QList<AppParameter>>::Iterator it = map.begin();
        for (; it != map.end(); it++) {
            if ((it.key() == "kb/s") && (m_parameter.type == "kb/s")) { //判断是流量还是网速
                if (it.value().size() == 0) { //对应key的value为0 添加value
                    insertflag = 2;
                }
                for (int i = 0; i < it.value().size(); i++) {
                    //pid在map中且名字解析相同,只修改流量/速度
                    if ((it.value()[i].pid == m_parameter.pid) && (m_parameter.pidname == (it.value()[i].pidname))) {
                        it.value()[i].send = m_parameter.send;
                        it.value()[i].recv = m_parameter.recv;
                        insertflag = 0;
                        break;
                    } else if ((it.value()[i].pid == m_parameter.pid) && (m_parameter.pidname != (it.value()[i].pidname))) { //pid在map中且名字解析不相同则丢弃) { //pid在map中且名字解析不相同则丢弃
                        it.value()[i].pidname = m_parameter.pidname;
                        it.value()[i].recv = m_parameter.recv;
                        it.value()[i].send = m_parameter.send;
                        insertflag = 0;
                        break;
                    } else if ((it.value()[i].pid != m_parameter.pid) && (m_parameter.pidname == (it.value()[i].pidname))) { //pid不在map中且名字解析相同则更新pid和数据;
                        it.value()[i].pid = m_parameter.pid;
                        it.value()[i].recv += m_parameter.recv;
                        it.value()[i].send += m_parameter.send;
                        insertflag = 0;
                        break;
                    } else if ((it.value()[i].pid != m_parameter.pid) && (m_parameter.pidname == (it.value()[i].pidname))
                               && (it.value()[i].sName == m_parameter.sName)) { //pid不在map中,pidname不相同,sname相同则替换pidname和pid
                        it.value()[i].pid = m_parameter.pid;
                        it.value()[i].recv += m_parameter.recv;
                        it.value()[i].send += m_parameter.send;
                        insertflag = 0;
                    } else if ((it.value()[i].pid != m_parameter.pid) && (m_parameter.pidname != (it.value()[i].pidname))
                               && (it.value()[i].sName == m_parameter.sName)) { //pid不在map中,pidname不相同,sname相同则替换pidname和pid
                        it.value()[i].pid = m_parameter.pid;
                        it.value()[i].pidname = m_parameter.pidname;
                        it.value()[i].send = m_parameter.send;
                        it.value()[i].recv = m_parameter.recv;
                        insertflag = 0;
                    } else if ((it.value()[i].pid != m_parameter.pid) && (!(m_parameter.pidname == (it.value()[i].pidname)))) { //数据插入map;
                        insertflag = 2;
                    }
                }
            } else if ((it.key() == "total_kb") && (m_parameter.type == "total_kb")) { //判断是流量还是网速
                if (it.value().size() == 0) {
                    insertflag = 2;
                }
                for (int i = 0; i < it.value().size(); i++) {
                    //pid在map中且名字解析相同,只修改流量/速度
                    if ((it.value()[i].pid == m_parameter.pid) && (m_parameter.pidname == (it.value()[i].pidname))) {
                        it.value()[i].send = m_parameter.send;
                        it.value()[i].recv = m_parameter.recv;
                        insertflag = 0;
                        break;
                    } else if ((it.value()[i].pid == m_parameter.pid) && (m_parameter.pidname != (it.value()[i].pidname))) { //pid在map中且名字解析不相同则丢弃) { //pid在map中且名字解析不相同则丢弃
                        it.value()[i].pidname = m_parameter.pidname;
                        it.value()[i].recv = m_parameter.send;
                        it.value()[i].send = m_parameter.recv;
                        insertflag = 0;
                        break;
                    } else if ((it.value()[i].pid != m_parameter.pid) && (m_parameter.pidname == (it.value()[i].pidname))) { //pid不在map中且名字解析相同则更新pid和数据;
                        it.value()[i].pid = m_parameter.pid;
                        it.value()[i].recv += m_parameter.recv;
                        it.value()[i].send += m_parameter.send;
                        insertflag = 0;
                        break;
                    } else if ((it.value()[i].pid != m_parameter.pid) && (m_parameter.pidname == (it.value()[i].pidname))
                               && (it.value()[i].sName == m_parameter.sName)) { //pid不在map中,pidname不相同,sname相同则替换pidname和pid
                        it.value()[i].pid = m_parameter.pid;
                        it.value()[i].send += m_parameter.send;
                        it.value()[i].recv += m_parameter.recv;
                        insertflag = 0;
                    } else if ((it.value()[i].pid != m_parameter.pid) && (m_parameter.pidname != (it.value()[i].pidname))
                               && (it.value()[i].sName == m_parameter.sName)) { //pid不在map中,pidname不相同,sname相同则替换pidname和pid
                        it.value()[i].pid = m_parameter.pid;
                        it.value()[i].pidname = m_parameter.pidname;
                        it.value()[i].send = m_parameter.send;
                        it.value()[i].recv = m_parameter.recv;
                        insertflag = 0;
                    } else if ((it.value()[i].pid != m_parameter.pid) && (m_parameter.pidname != (it.value()[i].pidname))
                               && (it.value()[i].sName == m_parameter.sName)) { //pid不在map中,pidname不相同,sname相同则替换pidname和pid
                        it.value()[i].pid = m_parameter.pid;
                        it.value()[i].pidname = m_parameter.pidname;
                        it.value()[i].send = m_parameter.send;
                        it.value()[i].recv = m_parameter.recv;
                        insertflag = 0;
                    } else {//数据插入map;
                        insertflag = 2;
                    }
                }
            }
        }
        if (insertflag == 2) {
            insertflag = 0;
            if (m_parameter.type == "kb/s") {
                qDebug() << "kb/s insert" << m_parameter.pid << m_parameter.sName << m_parameter.pidname << m_parameter.recv << m_parameter.send;
                m_mapspeedlist << m_parameter;//数据插入list
                map[m_parameter.type] = m_mapspeedlist;//数据替换
            } else if (m_parameter.type == "total_kb") {
                qDebug() << "total_kb insert" << m_parameter.pid << m_parameter.sName << m_parameter.pidname << m_parameter.recv << m_parameter.send;
                m_mapflowlist << m_parameter;//数据插入list
                map[m_parameter.type] = m_mapflowlist;//数据替换
            }
        }
        m_speedlist = map["kb/s"];
        m_flowlist = map["total_kb"];
    }
}
//网速清零
void TrafficDetailsWorker::resetspeedlist()
{
    DEFENDER_MUTEX_LOCKER(&SMutex::getInstance().m_mutex);
    if (m_mapspeedlist.size() == 0)
        return;
    for (int i = 0; i < m_mapspeedlist.size(); i++) {
        m_mapspeedlist[i].recv = 0;
        m_mapspeedlist[i].send = 0;
    }
    map["kb/s"] = m_mapspeedlist;//数据替换
    m_speedlist = map["kb/s"];
}
//根据pid获取desktop
appname TrafficDetailsWorker::getDetailNameByPid(int pid)
{
    DEFENDER_MUTEX_LOCKER(&SMutex::getInstance().m_mutex);

    appname app;
    app.pidname = "";
    app.name = "";

    for (auto at : m_procinfolist) {
        if (at.proc == pid) {
            app.pidname = at.desktop;
            app.name = at.title;
            break;
        }
    }
    return app;
}
//获取porclist中的信息
void TrafficDetailsWorker::Getallproclist(DefenderProcessInfoList proclist)
{
    m_procinfolist = proclist;
    qDebug() << proclist.count();
    for (auto at : m_procinfolist) {
        qDebug() << at.proc << at.title << at.desktop;
    }
}
//获取proc目录下的pid
void TrafficDetailsWorker::Getallprocpid(DefenderProcessList pid)
{
    m_procpid = pid;
}
//获取打开的应用的desktop
void TrafficDetailsWorker::Getdesktoplist(QStringList desktoplist)
{
    m_desktop = desktoplist;
    for (auto at : m_desktop)
        qDebug() << "desktop = " << at;
    Q_EMIT DesktopUpdate();
}

void TrafficDetailsWorker::GetGuidesktop(QStringList guidesktop)
{
    m_guidesktop = guidesktop;
    for (auto at : m_guidesktop)
        qDebug() << "m_guidesktop = " << at;
//    Q_EMIT DesktopUpdate();
}

void TrafficDetailsWorker::onSetSvaeFLowList(QList<AppParameter> saveflowlist)
{
    DEFENDER_MUTEX_LOCKER(&SMutex::getInstance().m_mutex);
    m_saveflowlist = saveflowlist;

#if Work_Debug_Flag
    int i = 0;
    for (auto at : m_saveflowlist) {
        qDebug() << "m_flownamelist" << m_saveflowlist.size() << at.sName  <<  at.pidname  << i  << at.pid;
        i++;
    }
#endif
}

void TrafficDetailsWorker::saveFlowData()
{
    DEFENDER_MUTEX_LOCKER(&SMutex::getInstance().m_mutex);
    //如果修改了系统时间
    if (QDateTime::currentDateTime() < m_lastSavedTime) {
        m_monitornetflow->ExitApp();
        QThread::currentThread()->msleep(500);
        for (int i = 0; i < 5; i++)
            m_monitornetflow->StartApp();
    }

    m_useData->apendDeleteFlowCacheDataSignal();
    for (AppParameter app : m_saveflowlist) {
        m_useData->apendSaveFlowCacheDataToLstSignal(app.pidname, app.sName, app.recv, app.send);
    }
    m_useData->apendSaveFlowDataSignal();

    //根据此时刻更新详情列表显示的数据
    switch (m_flowRangeStyle) {
    case FlowRangeStyle::today:
        requestGetTodayFlowData();
        break;
    case FlowRangeStyle::yesterday:
        requestGetYesterdayFlowData();
        break;
    case FlowRangeStyle::thisMonth:
        requestGetThisMonthFlowData();
        break;
    case FlowRangeStyle::lastMonth:
        requestGetLastMonthFlowData();
        break;
    }

    m_lastSavedTime = QDateTime::currentDateTime();

}

void TrafficDetailsWorker::onReceiveFlowDate(QList<AppFlowDateParameter> appLst)
{
    DEFENDER_MUTEX_LOCKER(&SMutex::getInstance().m_mutex);

    m_onShowFlowList.clear();
    for (AppFlowDateParameter app : appLst) {
        AppParameter appParTmp;
        appParTmp.pidname = app.name;
        appParTmp.recv = app.downloaded;
        appParTmp.send = app.uploaded;
        appParTmp.apptotal = app.apptotal;
        appParTmp.sName = app.sname;

        m_onShowFlowList.append(appParTmp);
    }
#if Work_Debug_Flag
    qDebug() << m_onShowFlowList.size();
#endif
}
