/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     luokai <luokai_cm@deepin.com>
 *
 * Maintainer: luokai <luokai_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#pragma once

#include "com_deepin_dde_daemon_launcherd.h"
#include "com_deepin_defender_monitornetflow.h"
#include "com_deepin_defender_trafficdetails.h"
#include "com_deepin_defender_analysisapp.h"
#include "csingleteon.h"
#include "usedata.h"
#include "common.h"

#include <QThread>
#include <QObject>

#define Work_Debug_Flag 0

struct AppParameter {
    QString name;
    double recv;
    double send;
    int pid;
    QString device;
    QString type;
    QString pidname;//desktop
    double apptotal = 0;//单个APP总流量
    QString sName;//显示的名字
};
struct appname {
    QString pidname;
    QString name;
};

//时间跨度
enum FlowRangeStyle {today, yesterday, thisMonth, lastMonth};

Q_DECLARE_METATYPE(AppParameter);

using Launcherd = com::deepin::dde::daemon::Launcher;
using MonitorNetFlow = com::deepin::defender::MonitorNetFlow;
using Analysisapp = com::deepin::defender::analysisapp;

class TrafficDetailsWorker : public QObject
{
    Q_OBJECT
public:
    TrafficDetailsWorker(QObject *parent = nullptr);
    void requestGetYesterdayFlowData();
    void requestGetTodayFlowData();
    void requestGetLastMonthFlowData();
    void requestGetThisMonthFlowData();
    void resetspeedlist();
    appname getDetailNameByPid(int pid);

    inline QList<AppParameter> GetSpeedList()
    {
        return m_speedlist;
    }
    inline QList<AppParameter> GetFlowFlit()
    {
        return m_flowlist;
    }
    inline QStringList GetDesktop()
    {
        return m_desktop;
    }
    inline QList<AppParameter> Getonshowlist()
    {
        return m_onShowFlowList;
    }
    inline Analysisapp *GetAnalysisapp()
    {
        return m_inter;
    }
    //获取worker中的使用数据库类实例
    inline UseData *getUseData() {return m_useData;}

Q_SIGNALS:
    void DesktopUpdate();

public Q_SLOTS:
    void onAppMonitorNetFlowDetailData(const QString &name, double recv, double send, int pid, const QString &device, const QString &type);
    void onSetSvaeFLowList(QList<AppParameter> saveflowlist);
    //存储应用流量数据
    void saveFlowData();
    //当从UseData中接收到流量数据
    void onReceiveFlowDate(QList<AppFlowDateParameter> appLst);
    void Getallproclist(DefenderProcessInfoList proclist);
    void Getallprocpid(DefenderProcessList pid);
    void Getdesktoplist(QStringList desktoplist);
    void GetGuidesktop(QStringList guidesktop);

private:
    Analysisapp *m_inter;
    MonitorNetFlow *m_monitornetflow;
    QMap<QString, QList<AppParameter>> map;
    //网速list
    QList<AppParameter> m_speedlist;
    //流量list
    QList<AppParameter> m_flowlist;
    //给数据库的流量list
    QList<AppParameter> m_saveflowlist;
    QList<AppParameter> m_mapspeedlist;
    QList<AppParameter> m_mapflowlist;
    DefenderProcessInfoList m_procinfolist;
    DefenderProcessList m_procpid;
    QStringList m_desktop;
    QStringList m_guidesktop;
    int insertflag = 1;
    AppParameter m_parameter;
    //当前显示的时间段
    FlowRangeStyle m_flowRangeStyle;
    //当前显示时间段下的应用流量统计列表
    QList<AppParameter> m_onShowFlowList;
    //使用数据库
    UseData *m_useData;
    //应用数据存储定时
    QTimer *m_saveFlowDateTimer;
    //上次存储流量数据的时间
    QDateTime m_lastSavedTime;
};
