/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#pragma once

#include <QSqlDatabase>
#include <QSqlError>
#include <QSqlQuery>
#include <QWidget>
#include <QDateTime>
#include <QDebug>
#include <QQueue>

QT_BEGIN_NAMESPACE
class QVBoxLayout;
class QHBoxLayout;
QT_END_NAMESPACE

struct AppFlowDateParameter {
    QString name;
    QString sname;
    QString timeymdh;
    double downloaded = 0;
    double uploaded = 0;
    double apptotal = 0;//单个APP总流量
};

//消息队列
enum SignalQueueType {
    DeleteFlowCacheData = 0,
    SaveFlowCacheDataToLst,
    SaveFlowData,
    GetYesterdayFlowData,
    GetTodayFlowData,
    GetLastMonthFlowData,
    GetThisMonthFlowData,
    GetAppYesterdayFlowData,
    GetAppTodayFlowData,
    GetAppLastMonthFlowData,
    GetAppThisMonthFlowData,
    SignalCount
};

//消息队列流量入参
struct SignalQueFlow {
    QString appname;
    QString sname;
    double downlod;
    double upload;
};

Q_DECLARE_METATYPE(AppFlowDateParameter);

#define UseDataDebugFlag 0

class UseData : public QObject
{
    Q_OBJECT

public:
    explicit UseData(QObject *parent = nullptr);
    //清空数据缓冲列表
    bool deleteFlowCacheData();
    //向数据缓冲列表中添加数据
    bool saveFlowCacheDataToLst(QString appname, QString sname, double downloaded, double uploaded);
    //储存缓冲列表中的数据
    bool saveFlowData();
    //获取所有应用某天的流量数据
    bool getDayFlowData(QList<AppFlowDateParameter> &appLst, QDateTime time);
    bool getYesterdayFlowData();
    bool getTodayFlowData();
    //获取所有应用某个月的数据
    bool getMonthFlowData(QList<AppFlowDateParameter> &appLst, QDateTime time);
    bool getLastMonthFlowData();
    bool getThisMonthFlowData();
    //获取某个应用某天的流量数据
    QList<AppFlowDateParameter> getAppDayFlowData(QString appname, QDateTime time);
    bool getAppYesterdayFlowData(QString appname);
    bool getAppTodayFlowData(QString appname);
    //获取某个应用某月的流量数据
    QList<AppFlowDateParameter> getAppMonthFlowData(QString appname, QDateTime time);
    bool getAppLastMonthFlowData(QString appname);
    bool getAppThisMonthFlowData(QString appname);
    //向操作信号序列中添加相应操作信号
    void apendDeleteFlowCacheDataSignal();
    void apendSaveFlowCacheDataToLstSignal(QString appname, QString sname, double downloaded, double uploaded);
    void apendSaveFlowDataSignal();
    //删除旧的数据
    void deleteOldData();

Q_SIGNALS:
    void notifyDeleteFlowCacheData();
    void notifySaveFlowCacheDataToLst(QString appname, QString sname, double downloaded, double uploaded);
    void notifySaveFlowData();
    //获取所有app流量数据
    void notifyGetYesterdayFlowData();
    void sendYesterdayFlowData(QList<AppFlowDateParameter> appLst);
    void notifyGetTodayFlowData();
    void sendTodayFlowData(QList<AppFlowDateParameter> appLst);
    void notifyGetLastMonthFlowData();
    void sendLastMonthFlowData(QList<AppFlowDateParameter> appLst);
    void notifyGetThisMonthFlowData();
    void sendThisMonthFlowData(QList<AppFlowDateParameter> appLst);
    //获取某个app流量数据
    void notifyGetAppYesterdayFlowData(QString appname);
    void sendAppYesterdayFlowData(QList<AppFlowDateParameter> appLst);
    void notifyGetAppTodayFlowData(QString appname);
    void sendAppTodayFlowData(QList<AppFlowDateParameter> appLst);
    void notifyGetAppLastMonthFlowData(QString appname);
    void sendAppLastMonthFlowData(QList<AppFlowDateParameter> appLst);
    void notifyGetAppThisMonthFlowData(QString appname);
    void sendAppThisMonthFlowData(QList<AppFlowDateParameter> appLst);
    //请求相应消息信号
    void reqSendSignal();

public Q_SLOTS:
    //响应消息信号，操作信号处理中心
    void onReqSendSignal();

private:
    void initDataBase();

private:
    //数据库对象
    QSqlDatabase m_sqlDb;
    //数据库文件路径
    QString m_dbPath;
    //查询数据库工具对象
    QSqlQuery m_sqlQuery;
    //待储存的数据列表
    QList<AppFlowDateParameter> m_appFlowCacheLst;
    //上次储存的数据列表
    QList<AppFlowDateParameter> m_lastAppFlowCacheLst;
    //操作信号队列
    QQueue<QPair<SignalQueueType, SignalQueFlow>> m_signalList;
};
