/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     wubowen <wubowen_cm@deepin.com>
 *
 * Maintainer: wubowen <wubowen_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "writedbusdata.h"

#include <QDebug>

WriteDBusData::WriteDBusData(QObject *parent)
    : QObject(parent)
    , m_recv_value(0.0)
    , m_send_value(0.0)
    , global_recv_value(0.0)
    , global_send_value(0.0)
    , m_pid(0)
    , m_device("")
{
    m_adaptor = new MonitorNetFlowAdaptor(this);

    if (!QDBusConnection::systemBus().registerService(service) ||
            !QDBusConnection::systemBus().registerObject(path, this)) {
        qDebug() << "dbus service already registered! " << service;

        exit(0);
    }

    connect(this, &WriteDBusData::notifyDBusKbData, this, &WriteDBusData::onNotifyDBusData);
    connect(this, &WriteDBusData::notifyGlobalData, this, &WriteDBusData::onNotifyGlobalData);
    connect(this, &WriteDBusData::notifyWorking, [this] {
        qDebug() << " catch libpcap is Working ";
    });

    QTimer *timer = new QTimer;
    connect(timer, &QTimer::timeout, this, &WriteDBusData::onNotifyHeartbeat);
    timer->start(1000 * timeout);
}

WriteDBusData::~WriteDBusData()
{

}

/*!
 * \brief WriteDBusData::onNotifyDBusData
 * \param name 应用名称
 * \param recv_value　收到的数据流量
 * \param send_value　发送的数据流量
 * \param pid   应用进程号
 * \param device　使用设备名称
 * \param type　流量单位 kb(每秒截取的流量)，total_kb(name应用全部消耗的流量)
 */
void WriteDBusData::onNotifyDBusData(QString name, double recv_value, double send_value, int pid, QString device, QString type)
{
    //只判断了recv,send都不同的情况
    //还有可能recv或send不变，另一个值变化了
    if (qFuzzyCompare(m_recv_value, recv_value) && qFuzzyCompare(m_send_value, send_value))
        return;

    //下面为每秒钟取到的流量
    if (recv_value > 0 || send_value > 0) {
        m_recv_value = recv_value;
        m_send_value = send_value;

        qDebug() << Q_FUNC_INFO << " do_refresh , name : " << name << " , send_value : " << send_value << " , recv_value : " << recv_value << " , pid : " << pid << " , device : " << device << " , type : " << type;
#if USE_DEBUG
        Q_EMIT m_adaptor->NotifyDetailData(name, recv_value, send_value, pid, device, type);
#endif
    }
}

//抓包到的整个系统所有应用消耗的总流量
void WriteDBusData::onNotifyGlobalData(double recv_value, double send_value)
{
    if (qFuzzyCompare(global_recv_value, recv_value) && qFuzzyCompare(global_send_value, send_value))
        return;

    global_recv_value = recv_value;
    global_send_value = send_value;

    qDebug() << Q_FUNC_INFO << " Global data , send_value , recv_value : " << send_value << recv_value;
#if USE_DEBUG
    Q_EMIT m_adaptor->NotifyGlobalData(recv_value, send_value);
#endif
}

void WriteDBusData::StartApp()
{
    qDebug() << Q_FUNC_INFO << " Start the App";
}

void WriteDBusData::ExitApp()
{
    qDebug() << Q_FUNC_INFO << " EXit the App";
    qApp->quit();
}

void WriteDBusData::onNotifyHeartbeat()
{
    qDebug() << Q_FUNC_INFO << " Monitor's heartbeat is working.";
    Q_EMIT m_adaptor->NotifyHeartbeat();
}
