/*
 * Copyright (C) 2017 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     zccrs <zccrs@live.com>
 *
 * Maintainer: zccrs <zhangjide@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "mainwindow.h"
#include "utils.h"
#include "window/modules/antivirus/antivirusmodule.h"
#include "window/modules/homepage/homepagemodule.h"
#include "window/modules/safetylevel/safetylevelmodule.h"
#include "window/modules/networkprotection/networkprotectionmodule.h"
#include "window/modules/startup/startupmodule.h"
#include "com_deepin_defender_daemonservice.h"
#include "com_deepin_defender_netcontrol.h"
#include "window/modules/systemsafety/systemsafetymodule.h"
#include <DBackgroundGroup>
#include <DGuiApplicationHelper>
#include <DListView>
#include <DStandardItem>
#include <DApplicationHelper>
#include <DTitlebar>
#include <DDialog>
#include <DRadioButton>
#include <DCheckBox>
#include <DSettingsDialog>

#include <QHBoxLayout>
#include <QMetaEnum>
#include <QDebug>
#include <QStandardItemModel>
#include <QPushButton>
#include <QLocale>
#include <QLinearGradient>
#include <QWidget>
#include <QResizeEvent>
#include <qsettingbackend.h>
#include <QGSettings>
#include <QDBusInterface>

DEF_USING_NAMESPACE
DTK_USE_NAMESPACE

const int WidgetMinimumWidget = 950;
const int WidgetMinimumHeight = 640;

//此处为带边距的宽度
const int first_widget_min_width = 200;

const QMargins navItemMargin(5, 3, 5, 3);
const QVariant NavItemMargin = QVariant::fromValue(navItemMargin);
const QString GSettingsCloseReminder = "close-type";
const QString GSettingsAskType = "ask-type";
const QString dmanInterface = "com.deepin.Manual.Open";

MainWindow::MainWindow(QWidget *parent)
    : DMainWindow(parent)
    , m_contentLayout(nullptr)
    , m_rightContentLayout(nullptr)
    , m_navView(nullptr)
    , m_rightView(nullptr)
    , m_navModel(nullptr)
    , m_pSystemTray(nullptr)
    , m_trayMenu(nullptr)
    , m_trayDialag(nullptr)
    , m_gsetting(nullptr)
    , m_NetControlServerInter(new NetControlServer("com.deepin.defender.netcontrol", "/com/deepin/defender/netcontrol", QDBusConnection::sessionBus(), this))
{
    //启动守护进程，确保守护进程每次都能正常工作
    initDaemonService();

    //Initialize view and layout structure
    QWidget *content = new QWidget(this);

    setWindowFlags(Qt::FramelessWindowHint | Qt::WindowMinimizeButtonHint | Qt::WindowCloseButtonHint);

    m_contentLayout = new QHBoxLayout(content);
    m_contentLayout->setContentsMargins(0, 0, 0, 0);
    m_contentLayout->setSpacing(0);
    m_rightContentLayout = new QHBoxLayout();

    m_rightView = new DBackgroundGroup(m_rightContentLayout);
    m_rightView->setItemSpacing(2);
    m_rightView->setItemMargins(QMargins(10, 10, 10, 10));
    m_rightView->setContentsMargins(10, 10, 10, 10);

    m_navView = new DListView(this);
    m_navView->setItemSize(QSize(180, 48));
    m_navView->setItemSpacing(0);
    m_navView->setSpacing(20);
    m_navView->setIconSize(QSize(32, 32));
    m_navView->setFrameShape(QFrame::Shape::NoFrame);
    m_navView->setEditTriggers(QListView::NoEditTriggers);
    m_navView->setAutoScroll(true);//滚动条
    m_navView->setTabKeyNavigation(true);//tab按键是否有效
    m_navView->setViewportMargins(QMargins(10, 0, 10, 0));//滚动区域边框大小
    m_navView->setFixedWidth(first_widget_min_width);//固定窗口值
    m_navView->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding);
    m_navView->setSpacing(0);
    m_navView->setSelectionMode(QAbstractItemView::SingleSelection);
    m_navView->setAutoFillBackground(true);
    m_navView->setBackgroundType(DStyledItemDelegate::BackgroundType(DStyledItemDelegate::RoundedBackground | DStyledItemDelegate::NoNormalState));

    m_contentLayout->addWidget(m_navView);
    m_contentLayout->addWidget(m_rightView);

    m_rightView->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

    layout()->setMargin(0);
    setCentralWidget(content);

    m_gsetting = new QGSettings("com.deepin.dde.deepin-defender", QByteArray(), this);

    DTitlebar *titlebar = this->titlebar();
    titlebar->setIcon(QIcon::fromTheme("deepin-defender"));

    auto menu = titlebar->menu();
    if (!menu) {
        qDebug() << "menu is nullptr, create menu!";
        menu = new QMenu;
    } else {
        qDebug() << "get title bar menu :" << menu;
    }
    titlebar->setMenu(menu);

    auto action = new QAction(tr("Settings"));
    menu->addAction(action);
    connect(action, &QAction::triggered, this, [this] {
        QTemporaryFile tmpFile;
        tmpFile.open();
        auto backend = new Dtk::Core::QSettingBackend(tmpFile.fileName());

        auto settings = Dtk::Core::DSettings::fromJsonFile(":/dt-settings.json");
        settings->setBackend(backend);

        //设置settings
        connect(settings, &Dtk::Core::DSettings::valueChanged, this, [ = ](const QString & key, const QVariant & value) {
            qDebug() << "[QSettings] key : " << key << " , value : " << value << value.toInt();
            if (key.contains("radiogroup")) {
                int type = value.toInt();
                if (type >= Tray && type <= Exit) {
                    this->setCloseType(type);
                    this->setAskType(0);
                } else if (type == Ask) {
                    this->setAskType(1);
                }
            }
        });

        //根据关闭窗口属性gsetting配置，更新设置界面选项
        if (0 == this->getAskType()) //优先显示每次询问按钮是否选中
            settings->setOption("base.close_window.radiogroup", getCloseType());
        else if (1 == this->getAskType())
            settings->setOption("base.close_window.radiogroup", 2);

        DSettingsDialog dsd(this);
        dsd.updateSettings(settings);
        dsd.exec();
    });

    //add help button
    auto helpAction = new QAction(tr("Help"));
//    menu->addAction(helpAction);
    connect(helpAction, &QAction::triggered, this, [] {
        QDBusInterface *inter = new QDBusInterface(dmanInterface,
                                                   "/com/deepin/Manual/Open",
                                                   dmanInterface,
                                                   QDBusConnection::sessionBus());
        inter->call("ShowManual", "dde");
        inter->deleteLater();
    });

    m_backwardBtn = new DIconButton(this);
    m_backwardBtn->setEnabled(false);
    m_backwardBtn->setIcon(QStyle::SP_ArrowBack);
    titlebar->addWidget(m_backwardBtn, Qt::AlignLeft | Qt::AlignVCenter);
    connect(m_backwardBtn, &DIconButton::clicked, [this] {
        popWidget();
    });

    //Initialize top page view and model
    m_navModel = new QStandardItemModel(m_navView);
    m_navView->setModel(m_navModel);
    connect(m_navView, &DListView::activated, this, &MainWindow::onFirstItemClick);
    connect(m_navView, &DListView::clicked, m_navView, &DListView::activated);

    QTimer::singleShot(0, this, &MainWindow::initAllModule);
    QTimer::singleShot(0, this, &MainWindow::modulePreInitialize);

    setFixedSize(QSize(WidgetMinimumWidget, WidgetMinimumHeight));

    initTray();

    //list里面存入模块的name：只有两级页面的模块才需要存储
    m_mulPageList.append("systemsafety");
}

MainWindow::~MainWindow()
{
}

void MainWindow::initAllModule()
{
    DEF_USING_ANTIVIRUS_NAMESPACE

    m_modules = {
//        { new HomePageModule(this), tr("Home")},
//        { new SafetyLevelModule(this), tr("Security Level")},
//        { new AntiVirusModule(this), tr("AntiVirus")},
        { new NetworkProtectionModule(this), tr("Firewall")},
        { new StartupModule(this), tr("Startup Programs")},
        { new SystemSafetyModule(this), tr("System Security")},
    };

    QStringList modulesIconLst = {
//        "dcc_home",
//        "dcc_security_level",
//        "dcc_virus_scan",
        "dcc_network_protection",
        "dcc_application_security",
        "dcc_system_security"
    };

    for (auto it = m_modules.cbegin(); it != m_modules.cend(); ++it) {
        DStandardItem *item = new DStandardItem;
        item->setText(it->second);

        QString iconPath = modulesIconLst.at(it - m_modules.cbegin());
        QIcon icon = QIcon::fromTheme(iconPath);
        item->setIcon(icon);

        item->setData(NavItemMargin, Dtk::MarginsRole);

        m_navModel->appendRow(item);
    }
}

void MainWindow::showModulePage(const QString &module, const QString &page)
{
    qDebug() << Q_FUNC_INFO << module << page;

    ModuleInterface *inter = nullptr;
    for (int i = 0; i < m_modules.count(); i++) {
        inter = m_modules[i].first;
        qDebug() << inter->name() << module;
        if (module == inter->name()) {
            setCurrentModule(i);
            break;
        }
    }
}

int MainWindow::getModuleCount()
{
    return m_modules.count();
}

void MainWindow::initTray()
{
    m_pSystemTray = new QSystemTrayIcon(this);
    m_trayMenu = new QMenu(this);

    QAction *show = new QAction;
    show->setText(tr("Security Center"));
    m_trayMenu->addAction(show);
    connect(show, &QAction::triggered, [this] {
        this->show();
        this->setWindowState(Qt::WindowState::WindowActive);
    });

    QAction *exit = new QAction;
    exit->setText(tr("Exit"));
    m_trayMenu->addAction(exit);
    connect(exit, &QAction::triggered, [ = ] {
        m_NetControlServerInter->ExitApp();
        qApp->quit();
    });

    // 设置系统托盘的上下文菜单
    m_pSystemTray->setContextMenu(m_trayMenu);
    // 设置系统托盘提示信息、托盘图标
    QString name = tr("Security Center");
    m_pSystemTray->setToolTip(name);
    m_pSystemTray->setIcon(QIcon::fromTheme("deepin-defender"));
    connect(m_pSystemTray, &QSystemTrayIcon::activated, this, &MainWindow::onTrayActivated);

    m_pSystemTray->show();
}

void MainWindow::initCloseDialog()
{
    m_trayDialag = new DDialog(this);
    m_trayDialag->setModal(true);
    m_trayDialag->setIcon(QIcon::fromTheme("deepin-defender"));
    DRadioButton *radioExit = new DRadioButton(tr("Exit"));
    DRadioButton *radioMin = new DRadioButton(tr("Minimize to system tray"));
    DCheckBox *radioAsk = new DCheckBox(tr("Do not ask again"));
    closeType nChoose = static_cast<closeType>(getCloseType());
    //根据GSettings设置settings页面的默认值
    switch (nChoose) {
    case Exit:
        radioExit->setChecked(true);
        break;
    case Tray:
    default:
        radioMin->setChecked(true);
        break;
    }
    radioAsk->setCheckState(Qt::CheckState::Unchecked);

    m_trayDialag->setMessage(tr("Please choose your action"));
    m_trayDialag->addContent(radioMin);
    m_trayDialag->addContent(radioExit);
    m_trayDialag->addContent(radioAsk);
    m_trayDialag->addButton(tr("Cancel"));
    m_trayDialag->addButton(tr("Confirm"), true, DDialog::ButtonRecommend);

    m_trayDialag->show();

    connect(m_trayDialag, &DDialog::buttonClicked, this, [ = ](int index, const QString & text) {
        Q_UNUSED(text);
        if (!m_trayDialag)
            return ;
        //没有点击确定，就不存储信息
        if (index != 1) {
            return;
        }

        if (radioMin->isChecked()) {
            this->hide();
        } else if (radioExit->isChecked()) {
            qApp->quit();
        }

        if (radioMin->isChecked()) {
            setCloseType(Tray);
        } else if (radioExit->isChecked()) {
            setCloseType(Exit);
        }

        //只有当选择radioAsk后才会设置不再询问
        if (radioAsk->isChecked()) {
            setAskType(0);
        }

        m_trayDialag->deleteLater();
        m_trayDialag = nullptr;
    });
}

void MainWindow::modulePreInitialize()
{
    for (auto it = m_modules.cbegin(); it != m_modules.cend(); ++it) {
        QElapsedTimer et;
        et.start();
        it->first->preInitialize();
        qDebug() << QString("initalize %1 module using time: %2ms")
                 .arg(it->first->name())
                 .arg(et.elapsed());
    }

    QModelIndex index0 = m_navModel->index(0, 0);
    if (index0.isValid())
        onFirstItemClick(index0);
}

void MainWindow::popWidget()
{
    if (!m_contentStack.size())
        return;

    QWidget *w = m_contentStack.pop().second;

    m_rightContentLayout->removeWidget(w);
    w->setParent(nullptr);
    w->deleteLater();

    if (m_contentStack.isEmpty())
        return;
    QWidget *top = m_contentStack.top().second;
    m_rightContentLayout->addWidget(top);

    if (m_backwardBtn) {
        m_backwardBtn->setEnabled(m_contentStack.count() > 1);
    }
}

//确保“安全中心”启动后，所有的服务都能正常工作
void MainWindow::initDaemonService()
{
    using DaemonService = com::deepin::defender::daemonservice;
    DaemonService *daemon = new DaemonService("com.deepin.defender.daemonservice", "/com/deepin/defender/daemonservice",
                                              QDBusConnection::sessionBus(), this);
    daemon->StartApp();
    daemon->deleteLater();

//    using NetControl = com::deepin::defender::netcontrol;
//    NetControl *net = new NetControl("com.deepin.defender.netcontrol", "/com/deepin/defender/netcontrol",
//                                     QDBusConnection::sessionBus(), this);

//    bool isValid = net->quitDefender();
//    qDebug() << " net->quitDefender() : " << net->quitDefender();
//    if (isValid) {
//        qDebug() << " Can't start on terminel.";
//        exit(0);
//    }
//    net->deleteLater();
}

void MainWindow::closeEvent(QCloseEvent *event)
{
    m_NetControlServerInter->ExitApp();
    //仅缩小到托盘
    if (!getAskType()) {
        //缩小到托盘且退出
        if (getCloseType()) {
            qApp->quit();
        }
        return;
    }

    event->ignore();
    initCloseDialog();
}

void MainWindow::hideEvent(QHideEvent *event)
{
    Q_UNUSED(event);
    m_NetControlServerInter->ExitApp();
}

//Only used to from third page to top page can use it
void MainWindow::popAllWidgets(int place)
{
    for (int pageCount = m_contentStack.count(); pageCount > place; pageCount--) {
        popWidget();
    }
}

void MainWindow::popWidget(ModuleInterface *const inter)
{
    Q_UNUSED(inter)

    popWidget();
}

void MainWindow::toggle()
{
    raise();
    if (isMinimized() || !isVisible())
        show();

    activateWindow();
}

void MainWindow::setModuleVisible(ModuleInterface *const inter, const bool visible)
{
    auto find_it = std::find_if(m_modules.cbegin(),
                                m_modules.cend(),
    [ = ](const QPair<ModuleInterface *, QString> &pair) {
        return pair.first == inter;
    });

    if (find_it != m_modules.cend()) {
        m_navView->setRowHidden(find_it - m_modules.cbegin(), !visible);
        Q_EMIT moduleVisibleChanged(find_it->first->name(), visible);
    }
}

void MainWindow::pushWidget(ModuleInterface *const inter, QWidget *const w)
{
    if (!inter)  {
        qWarning() << Q_FUNC_INFO << " inter is nullptr";
        return;
    }

    if (!w)  {
        qWarning() << Q_FUNC_INFO << " widget is nullptr";
        return;
    }

    pushNormalWidget(inter, w, m_mulPageList.contains(inter->name()));

    if (m_mulPageList.contains(inter->name())) {
        if (m_backwardBtn) {
            m_backwardBtn->setEnabled(m_contentStack.count() > 2);
        }
    } else {
        if (m_backwardBtn) {
            m_backwardBtn->setEnabled(m_contentStack.count() > 1);
        }
    }
}

void MainWindow::pushNormalWidget(ModuleInterface *const inter, QWidget *const w, bool bThreePage)
{
    m_rightView->setItemMargins(QMargins(0, 0, 0, 0));
    m_rightView->setContentsMargins(0, 0, 0, 0);

    //Set the newly added page to fill the blank area
    w->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

    qDebug() << " m_contentStack.count() : " << m_contentStack.count();
    //bThreePage : true 表示支持插入3级页面 , false 表示不可以插入3级页面（只能在第二级页面上切换）
    //m_contentStack.count() = 0,表示正常替换页面栈的页面
    //m_contentStack.count() >= 1 说明会有第3级页面加进了
    if (bThreePage && m_contentStack.count() >= 1) {
        if (m_contentStack.count() >= 2)
            popWidget();
        m_contentStack.push({inter, w});
        m_rightContentLayout->addWidget(w, m_contentStack.count() == 1 ? 3 : 7);
    } else {
        QWidget *top = nullptr;
        if (!m_contentStack.isEmpty())
            top = m_contentStack.top().second;

        if (top == w)
            return;

        m_contentStack.push({inter, w});
        //replace current widget
        if (top != nullptr) {
            m_rightContentLayout->removeWidget(top);
            top->setParent(nullptr);
        }

        qDebug() << " top widget = " << top << ", current widget = " << w;
        m_rightContentLayout->addWidget(w);
    }
}

int MainWindow::getCloseType()
{
    return m_gsetting ? m_gsetting->get(GSettingsCloseReminder).toInt() : 0;
}

//0 : Not Ask  1 : Ask
void MainWindow::setCloseType(int value)
{
    if (m_gsetting) {
        m_gsetting->set(GSettingsCloseReminder, value);
    }
}

int MainWindow::getAskType()
{
    return m_gsetting ? m_gsetting->get(GSettingsAskType).toInt() : 0;
}

void MainWindow::setAskType(int value)
{
    if (m_gsetting) {
        m_gsetting->set(GSettingsAskType, value);
    }
}

void MainWindow::onFirstItemClick(const QModelIndex &index)
{
    ModuleInterface *inter = m_modules[index.row()].first;

    if (!m_contentStack.isEmpty() && m_contentStack.last().first == inter) {
        return;
    }

    popAllWidgets();

    inter->active();
}

void MainWindow::onTrayActivated(QSystemTrayIcon::ActivationReason state)
{
    qDebug() << Q_FUNC_INFO << state;

    if (state == QSystemTrayIcon::ActivationReason::Trigger) {
        if (!this->isVisible()) {
            //页面隐藏，点击托盘重新show
            this->show();
        } else {
            //页面能显示的时候，点击托盘会＂正常/最小化＂切换窗口
            if (this->isMinimized()) {
                this->setWindowState(Qt::WindowState::WindowActive);
            } else {
                this->setWindowState(Qt::WindowState::WindowMinimized);
            }
        }
    }
}

void MainWindow::setCurrentModule(int index)
{
    if (index < 0)
        return;
    m_navView->setCurrentIndex(m_navModel->index(index, 0));
    Q_EMIT m_navView->clicked(m_navModel->index(index, 0));
}

int MainWindow::getModuleIndex(const QString &name)
{
    int index = 0;
    for (QPair<ModuleInterface *, QString> p : m_modules) {
        if (p.first->name() == name)
            return index;
        index++;
    }
    return -1;
}
