/*
 * Copyright (C) 2017 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     zccrs <zccrs@live.com>
 *
 * Maintainer: zccrs <zhangjide@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#pragma once

#include "window/interface/frameproxyinterface.h"
#include "window/interface/moduleinterface.h"
#include "com_deepin_defender_netcontrol.h"

#include <DMainWindow>
#include <DIconButton>
#include <QCloseEvent>

#include <QStack>
#include <QPair>
#include <QDBusContext>
#include <QSystemTrayIcon>

DWIDGET_BEGIN_NAMESPACE
class DListView;
class DBackgroundGroup;
class DDialog;
DWIDGET_END_NAMESPACE

QT_BEGIN_NAMESPACE
class QHBoxLayout;
class QStandardItemModel;
class QGSettings;
QT_END_NAMESPACE

DWIDGET_USE_NAMESPACE

enum closeType {
    Tray,
    Exit,
    Ask,
    Count
};

using NetControlServer = com::deepin::defender::netcontrol;
DEF_NAMESPACE_BEGIN
class MainWindow : public DTK_WIDGET_NAMESPACE::DMainWindow, public FrameProxyInterface, protected QDBusContext
{
    Q_OBJECT
public:
    explicit MainWindow(QWidget *parent = nullptr);
    ~MainWindow() override;

    void popWidget(ModuleInterface *const inter) override;
    void pushWidget(ModuleInterface *const inter, QWidget *const w) override;
    void setModuleVisible(ModuleInterface *const inter, const bool visible) override;
    void setCurrentModule(int index) override;
    int getModuleIndex(const QString &name) override;

public:
    void toggle();
    void popWidget();
    void initDaemonService();
    void initAllModule();
    void showModulePage(const QString &module, const QString &page);
    int getModuleCount();

protected:
    void closeEvent(QCloseEvent *event) override;
    void hideEvent(QHideEvent *event) override;

private:
    QHBoxLayout *m_contentLayout;
    QHBoxLayout *m_rightContentLayout;
    DTK_WIDGET_NAMESPACE::DListView *m_navView;
    DTK_WIDGET_NAMESPACE::DBackgroundGroup *m_rightView;
    QStandardItemModel *m_navModel;
    QStack<QPair<ModuleInterface *, QWidget *>> m_contentStack;
    QList<QPair<ModuleInterface *, QString>> m_modules;
    DIconButton *m_backwardBtn;
    QSystemTrayIcon *m_pSystemTray;
    QMenu *m_trayMenu;
    DDialog *m_trayDialag;
    QGSettings *m_gsetting;
    NetControlServer  *m_NetControlServerInter = nullptr; // 联网管控接口
    QList<QString> m_mulPageList;//存储需要从第二级页面切换到第三级页面的模块name （正常最多只能显示两级页面）

Q_SIGNALS:
    void moduleVisibleChanged(const QString &module, bool visible);

private Q_SLOTS:
    void onFirstItemClick(const QModelIndex &index);
    void onTrayActivated(QSystemTrayIcon::ActivationReason state);

private:
    void initTray();
    void initCloseDialog();
    void modulePreInitialize();
    void popAllWidgets(int place = 0);//place is Remain count
    void pushNormalWidget(ModuleInterface *const inter, QWidget *const w, bool bThreePage = false);  //exchange third widget : push new widget
    int getCloseType();
    void setCloseType(int value);
    int getAskType();
    void setAskType(int value);
};
DEF_NAMESPACE_END
