/*
 * Copyright (C) 2017 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     LiLinling <lilinling_cm@deepin.com>
 *
 * Maintainer: LiLinling <lilinling_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "antivirusmainwidget.h"
#include "circlelabel.h"
#include "virusmodel.h"

#include <DSuggestButton>
#include <DTipLabel>
#include <DFontSizeManager>

#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QPixmap>
#include <QCheckBox>
#include <QMenu>
#include <QMenuBar>
#include <QDate>

DEF_USING_ANTIVIRUS_NAMESPACE
DWIDGET_USE_NAMESPACE

AntiVirusMainWidget::AntiVirusMainWidget(QWidget *parent)
    : QWidget(parent)
    , m_centralLayout(new QHBoxLayout)
    , m_daysPassed(new QLabel("6 days passed since last virus library update"))
    , m_cb1(new QCheckBox("###############"))
    , m_cb2(new QCheckBox("###############"))
    , m_scanBtns(new DSuggestButton("Quick Scan"))
    , m_engineBtn(new CircleLabel(QPixmap("/home/li/.cache/deepin/dde-control-center/sync/default.png"), false))
    , m_libVersionTime(new QLabel("anti virus library version is : V1.0 2019-11-23"))
    , m_libstatus(new DTipLabel("anti virus is up to date"))
{
    QVBoxLayout *leftLayout = new QVBoxLayout;
    leftLayout->setSpacing(10);
    leftLayout->setMargin(10);
    QLabel *title = new QLabel("Anti virus, defending system security.");
    DFontSizeManager::instance()->bind(title, DFontSizeManager::T3);
    leftLayout->addWidget(title);

    leftLayout->addSpacing(58);
    leftLayout->addWidget(m_daysPassed);
    leftLayout->addWidget(m_cb1);
    leftLayout->addWidget(m_cb2);

    leftLayout->addSpacing(48);
    m_scanBtns->setFixedSize(QSize(200, 56));
    leftLayout->addWidget(m_scanBtns);

    QMenu *menu = new QMenu("Quick Scan");
    m_scanBtns->setMenu(menu);
    QAction *tqs = new QAction(this);
    tqs->setText("Quick Scan");
    menu->addAction(tqs);
    QAction *qs = new QAction(this);
    qs->setText("Total Scan");
    menu->addAction(qs);

    QAction *ps = new QAction(this);
    ps->setText("Partial Scan");
    QMenu *submenu = new QMenu("Partial Scan");
    submenu->addAction(ps);
    menu->addMenu(submenu);

    connect(tqs, &QAction::triggered, this, &AntiVirusMainWidget::onScanBtnClicked);
    connect(qs, &QAction::triggered, this, &AntiVirusMainWidget::onScanBtnClicked);
    connect(ps, &QAction::triggered, this, &AntiVirusMainWidget::onScanBtnClicked);

    leftLayout->addSpacing(48);
    m_engineBtn->setFixedSize(QSize(68, 68));
    leftLayout->addWidget(m_engineBtn);
    leftLayout->addSpacing(10);
    QHBoxLayout *libLayout = new QHBoxLayout;
    libLayout->addWidget(m_libVersionTime);
    libLayout->addWidget(m_libstatus);
    leftLayout->addLayout(libLayout);

    QVBoxLayout *rightLayout = new QVBoxLayout;
    rightLayout->addWidget(new QLabel("ICON"));

    m_centralLayout->setSpacing(20);
    m_centralLayout->addLayout(leftLayout);
    m_centralLayout->addLayout(rightLayout, Qt::AlignVCenter);
    setLayout(m_centralLayout);
}

void AntiVirusMainWidget::setModel(VirusModel *model)
{
    m_model = model;

    QString str = QString("anti virus library version is : %1 %2").arg(m_model->virusVersion()).arg(m_model->virusDate());
    m_libVersionTime->setText(str);
    m_daysPassed->setText(QString("%1 days passed since last virus library update").arg(m_model->lastAntiVirusDays()));
    if (m_model->latestVersion() != m_model->virusVersion()) {
        m_libstatus->setText("anti virus is up to date");
    } else {
        m_libstatus->setText(QString("latest anti virus library version is %1").arg(m_model->latestVersion()));
    }

    connect(m_model, &VirusModel::currentVirusVersionChanged, this, [this](const QString & verion) {
        m_virusVersion = verion;
        QString str = QString("anti virus library version is : %1 %2").arg(m_virusVersion).arg(m_virusDate);
        m_libVersionTime->setText(str);
    });

    connect(m_model, &VirusModel::currentVirusDateChanged, this, [this](const QString & date) {
        m_virusDate = date;
        QString str = QString("anti virus library version is : %1 %2").arg(m_virusVersion).arg(m_virusDate);
        m_libVersionTime->setText(str);
    });

    connect(m_model, &VirusModel::latestVersionChanged, this, [this](const QString & version) {
       if (version == m_virusVersion) {
           m_libstatus->setText("anti virus is up to date");
       } else {
           m_libstatus->setText(QString("latest anti virus library version is %1").arg(version));
       }
    });
}

void AntiVirusMainWidget::onScanBtnClicked()
{
    //start dialog
    m_model->setLastAntiVirusDays(QDate::currentDate());
    Q_EMIT requestStartScan(0);
}
