/*
 * Copyright (C) 2017 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     LiLinling <lilinling_cm@deepin.com>
 *
 * Maintainer: LiLinling <lilinling_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#pragma once

#include "window/namespace.h"
#include "window/interface/moduleinterface.h"

#include <QWidget>

DEF_NAMESPACE_BEGIN
class FrameProxyInterface;
class ModuleInterface;
class ScanWorker;
class VirusModel;
DEF_ANTIVIRUS_NAMESPACE_BEGIN
class AntiVirusModule : public QObject, public ModuleInterface
{
    Q_OBJECT
public:
    explicit AntiVirusModule(FrameProxyInterface *frameProxy, QObject *parent = nullptr);

    // preInitialize会在模块初始化时被调用，用于模块在准备阶段进行资源的初始化；
    // preInitialize不允许进行高资源的操作；
    virtual void preInitialize() override;

    // initialize初始化相应的模块，参数proxy用于Moudle向Frame信息查询和主动调用；
    // 返回Module的id；
    // initialize的时候不能做资源占用较高的操作；
    virtual void initialize() override;

    ///
    /// \brief name
    /// your module name
    /// \return
    ///
    virtual const QString name() const override;

    ///
    /// \brief active
    /// 当模块第一次被点击进入时，active会被调用
    virtual void active() override;

    ///
    /// \brief active
    /// 当模块被销毁时，deactive会被调用
    virtual void deactive() override;
private:
    VirusModel *m_model;
    ScanWorker *m_worker;
};
DEF_ANTIVIRUS_NAMESPACE_END
DEF_NAMESPACE_END
