/*
 * Copyright (C) 2017 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     LiLinling <lilinling_cm@deepin.com>
 *
 * Maintainer: LiLinling <lilinling_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "circlelabel.h"

#include <DFontSizeManager>

#include <QPainter>

DWIDGET_USE_NAMESPACE

CircleLabel::CircleLabel(QWidget *parent)
    : CircleLabel("", Qt::blue, parent)
{

}

CircleLabel::CircleLabel(const QString &text, QColor color, QWidget *parent)
    : QWidget (parent)
    , m_color(color)
    , m_progressColor("#3399ff")
    , m_txt(text)
    , m_value(80)
    , m_isTxt(true)
    , m_hasLine(true)
    , m_lineWidth(4)
{

}

CircleLabel::CircleLabel(QPixmap pixmap, bool hasLine, QWidget *parent)
    : QWidget(parent)
    , m_pixmap(pixmap)
    , m_isTxt(false)
    , m_hasLine(hasLine)
{
    if (m_hasLine) {
        m_lineWidth = 4;
    } else {
        m_lineWidth = 0;
    }
}

void CircleLabel::setText(const QString &text, QColor color)
{
    m_txt = text;
    m_color = color;
    m_isTxt = true;
    update();
}

void CircleLabel::setPixmap(QPixmap pixmap, QColor color)
{
    m_pixmap = pixmap;
    m_isTxt = false;
    m_hasLine = true;
    m_color = color;
    m_lineWidth = 4;
    m_value = 100;
    update();
}

void CircleLabel::setSimplePixmap(QPixmap pixmap)
{
    m_pixmap = pixmap;
    m_isTxt = false;
    m_hasLine = false;
}

void CircleLabel::setValue(int value)
{
    m_value = value;
    update();
}

void CircleLabel::paintEvent(QPaintEvent *event)
{
    Q_UNUSED(event);
    int width = this->width();
    int height = this->height();
    int side = qMin(width, height);
    int radius = side / 2 - 2; //avoid line cut

    //绘制准备
    QPainter painter(this);
    painter.setRenderHints(QPainter::Antialiasing); //设置渲染特性：反锯齿
    painter.translate(width / 2, height / 2);

    //绘制中心圆
    drawCircle(&painter, radius);

    if (m_isTxt) {
        painter.save();
        QFont ft = font();
        ft.setBold(true);
        setFont(ft);
//        DFontSizeManager::instance()->bind(this, DFontSizeManager::T6);
        int wide = fontMetrics().width(m_txt);
        int high = fontMetrics().height();
        QPen pen = painter.pen();
        pen.setColor(m_color);
        painter.setPen(pen);
        painter.drawText(QPoint(-wide / 2, high / 4), m_txt);
        painter.restore();
    } else {
        radius = radius - m_lineWidth;
        QRect r = QRect(-radius, -radius, radius * 2, radius * 2);
        QPainterPath path;
        path.addEllipse(r);

        painter.setClipPath(path);
        painter.drawPixmap(r, m_pixmap);
    }

    //绘制圆弧边框
    drawArc(&painter, radius);

}

void CircleLabel::drawCircle(QPainter *painter, int radius)
{
    if (!m_hasLine)
        return;

    radius = radius - m_lineWidth;

    painter->save();
    QPen pen = painter->pen();
    pen.setColor(m_color);
    pen.setWidth(m_lineWidth);
    painter->setPen(pen);
    painter->setBrush(Qt::NoBrush);
    painter->drawEllipse(-radius, -radius, radius * 2, radius * 2); //原点为左上角
    painter->restore();
}

void CircleLabel::drawArc(QPainter *painter, int radius)
{
    if (!m_hasLine)
        return;

    painter->save();
    painter->setBrush(Qt::green);

    QPen pen = painter->pen();
    pen.setWidthF(m_lineWidth);

    //设置线条风格
//    pen.setCapStyle(Qt::RoundCap);

    double arcLength = 360.0 / 100 * m_value; //°
    QRect rect(-radius, -radius, radius * 2, radius * 2);

    //绘制当前进度
    pen.setColor(Qt::blue);
    painter->setPen(pen);
    painter->drawArc(rect, 90 * 16, static_cast<int>(-arcLength * 16));

    painter->restore();
}
