/*
 * Copyright (C) 2017 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     LiLinling <lilinling_cm@deepin.com>
 *
 * Maintainer: LiLinling <lilinling_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#pragma once

#include "window/namespace.h"
#include <com_deepin_defender_hmiscreen.h>

#include <QObject>

using defenderInter = com::deepin::defender::hmiscreen;

DEF_NAMESPACE_BEGIN
class VirusModel;
class ScanWorker : public QObject
{
    Q_OBJECT
public:
    explicit ScanWorker(VirusModel *model, const QString &type, QWidget *parent = nullptr);
    void pause();
    void resume();

    enum Type {
    // ⽂件类型
        Normal = 0, //正常,
        Suspicious, //可疑,
        Danger, //危险,
        Malicious,//恶意程序,
        ScanTaskError,// 扫描任务错误
        NetAnomaly,//⽹络异常,
        ServerAnomaly,//服务器异常,
        TypeMaxCnt
    };

    QStringList TypesNames {
        "Normal",
        "Suspicious",
        "Danger",
        "Malicious",
        "ScanTaskError",
        "NetAnomaly",
        "ServerAnomaly"
    };

Q_SIGNALS:
    void over();       //we have to inform the GUI thread that we are over
    void progress(int value, int type, const QString &itemInfo);
    void totalItem(int total);

public Q_SLOTS:
    void scan(const QString &);    //the only slot here that does it all
    void scanOne(int current, int total, const QString &filePath);
    void scanFile(const QString &path);
    void scanProcess(const int processId);
    void onVersionInfoChanged(DefenderVersionInfo info);
    void onDissectionInfoChanged(DefenderDissection info);
    void onUpdateStateChanged(DefenderUpdateState info);

    void onNotifyDissedtionStateChanged(int taskId, QString state);
    void onNotifyVersionInfoChanged(QString state);
    void onNotifyUpdateInfoChanged(QString state);

private:
    QFileInfoList getAllFiles(const QString &path);

private:
    QWidget *m_parent;
    VirusModel *m_model;
    volatile bool m_isScanning;    //true: scan   false: paused
    volatile bool m_selfPause;
    defenderInter *m_defenderInter;
    QStringList m_fileList;
    volatile int m_totalFiles;
    volatile int m_currentFileCnt;
};
DEF_NAMESPACE_END
