/*
 * Copyright (C) 2017 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     LiLinling <lilinling_cm@deepin.com>
 *
 * Maintainer: LiLinling <lilinling_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "virusmodel.h"

#include <QGSettings>
#include <QDate>
#include <QVariant>

static const QString LAST_ANTI_VIRUS_DATE = "last-anti-virus-date";
static const QString CURRENT_VIRUS_VERSION = "current-virus-version";
static const QString VIRUS_LIB_DATE = "virus-lib-date";
static const QString LATEST_VIRUS_VERSION = "latest-virus-version";

VirusModel::VirusModel(QObject *parent)
    : QObject(parent)
    , m_lastAntiVirusDate("")
    , m_currentVirusVersion("")
    , m_currentVirusDate("")
    , m_latestVersion("")
{
    m_gs = new QGSettings("com.deepin.dde.deepin-defender", QByteArray(), this);
}

qint64 VirusModel::lastAntiVirusDays()
{
    QString dateStr = m_gs->get(LAST_ANTI_VIRUS_DATE).toString();
    QDate lastDate = QDate::fromString(dateStr);
    return (QDate::currentDate().daysTo(lastDate));
}

void VirusModel::setLastAntiVirusDays(const QDate &date)
{
    if (date.toString() != m_lastAntiVirusDate) {
        m_lastAntiVirusDate = date.toString();
        m_gs->set(LAST_ANTI_VIRUS_DATE, date.toString());
        Q_EMIT lastAntiVirusDaysChanged(date.daysTo(QDate::currentDate()));
    }
}

QString VirusModel::virusVersion()
{
    return m_gs->get(CURRENT_VIRUS_VERSION).toString();
}

void VirusModel::setVirusVersion(const QString &version)
{
    if (m_currentVirusVersion != version) {
        m_currentVirusVersion = version;
        m_gs->set(CURRENT_VIRUS_VERSION, version);
        Q_EMIT currentVirusVersionChanged(version);
    }
}

QString VirusModel::virusDate()
{
    return (m_gs->get(VIRUS_LIB_DATE).toString());
}

void VirusModel::setVirusDate(const QString &date)
{
    if (m_currentVirusDate != date) {
        m_currentVirusDate = date;
        m_gs->set(VIRUS_LIB_DATE, date);
        Q_EMIT currentVirusDateChanged(date);
    }
}

QString VirusModel::latestVersion()
{
    return m_gs->get(LATEST_VIRUS_VERSION).toString();
}

void VirusModel::setLatestVersion(const QString &version)
{
    if (m_latestVersion != version) {
        m_latestVersion = version;
        m_gs->set(LATEST_VIRUS_VERSION, version);
        Q_EMIT latestVersionChanged(version);
    }
}
