/*
 * Copyright (C) 2017 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     LiLinling <lilinling_cm@deepin.com>
 *
 * Maintainer: LiLinling <lilinling_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "virusscanneditemdelegate.h"

#include <DStyle>

#include <QApplication>
#include <QFont>
#include <QPainter>
#include <QDebug>

DWIDGET_USE_NAMESPACE

static inline QFlags<Qt::AlignmentFlag> alignmentFlag(int col)
{
    auto emCol = static_cast<VirusScannedItemDelegate::ScannedColumn>(col);
    switch (emCol) {
    case VirusScannedItemDelegate::ItemName:
    case VirusScannedItemDelegate::ItemStatics:
    case VirusScannedItemDelegate::ItemState:
        return (Qt::AlignLeft | Qt::AlignVCenter);
    case VirusScannedItemDelegate::ColumnButt:
        break;
    }
    return (Qt::AlignLeft | Qt::AlignVCenter);
}

static inline QRect colRect(int col, const QStyleOptionViewItem &option)
{
    static auto tailwidth  = 120;
    auto w = option.rect.width() - tailwidth;

    auto emCol = static_cast<VirusScannedItemDelegate::ScannedColumn>(col);
    switch (emCol) {
    case VirusScannedItemDelegate::ItemName:
        return QRect(0, option.rect.y(), w / 2, option.rect.height());
    case VirusScannedItemDelegate::ItemStatics:
        return QRect(w/2, option.rect.y(), w / 2, option.rect.height());
    case VirusScannedItemDelegate::ItemState:
        return QRect(w, option.rect.y(), tailwidth, option.rect.height());
    case VirusScannedItemDelegate::ColumnButt:
        break;
    }
    return option.rect.marginsRemoved(QMargins(0, 0, 0, 0));
}

VirusScannedItemDelegate::VirusScannedItemDelegate(QWidget *parent)
    : QStyledItemDelegate(parent)
{

}

void VirusScannedItemDelegate::initStyleOption(QStyleOptionViewItem *option, const QModelIndex &index) const
{
    QStyledItemDelegate::initStyleOption(option, index);
    option->state = option->state & ~QStyle::State_HasFocus;
}

void VirusScannedItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    painter->save();
    painter->setRenderHint(QPainter::Antialiasing);

    DStyle* sty = qobject_cast<DStyle *>(QApplication::style());
    auto ico = sty->standardIcon(DStyle::SP_IndicatorSearch);

    for (int col = 0; col < ColumnButt; ++col) {
        auto flag = alignmentFlag(col);
        auto rect = colRect(col, option);
        switch (col) {
        case ItemName: {
            auto type = index.data(ItemLevelRole).toInt();
            auto sz = QSizeF();
            if (type != -1)
                sz = QSizeF(15, 15);
            auto icon = ico.pixmap(sz.toSize());
            auto centerF = QRectF(rect).center();
            auto iconRect = QRectF(rect.x(), centerF.y() - sz.height() / 2, sz.width(), sz.height());
            auto txtRect = QRectF(10 + sz.width(), rect.y(), rect.width() - 10 - sz.width(), rect.height());
            if (type != -1)
                painter->drawPixmap(iconRect, icon, icon.rect());
            QFont font(painter->font());
            QFontMetrics fm(font);
            auto text = fm.elidedText(index.data(ItemNameRole).toString(), Qt::ElideMiddle, rect.width() - 10 - sz.width());
            painter->drawText(txtRect, flag, text);
            break;
        }
        case ItemStatics: {
            QFont font(painter->font());
            QFontMetrics fm(font);
            auto text = fm.elidedText(index.data(ItemStaticsRole).toString(), Qt::ElideMiddle, rect.width());
            painter->drawText(rect, flag, text);
            break;
        }
        case ItemState: {
            QFont font(painter->font());
            QFontMetrics fm(font);
            QString text;
            auto type = index.data(ItemLevelRole).toInt();
            QPen pen = painter->pen();
            QString severityStr;
            if ((type >= NORMAL) && (type < TypeMaxCnt)) {
                severityStr = TypesNames[type];
            } else {
                severityStr = index.data(ItemStateRole).toString();
            }

            if (type == NORMAL) {
                pen.setColor(Qt::green);
            } else if (type == -1) {
            } else if (type == ABNORMAL) {
                pen.setColor(Qt::red);
            } else {
                pen.setColor(Qt::red);
            }
            text = fm.elidedText(severityStr, Qt::ElideMiddle, rect.width());
            painter->setPen(pen);
            painter->drawText(rect, flag, text);
            break;
        }
        default:
            break;
        }
    }
    painter->restore();
}
