/*
 * Copyright (C) 2017 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     LiLinling <lilinling_cm@deepin.com>
 *
 * Maintainer: LiLinling <lilinling_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "virusscanningwidget.h"
#include "circlelabel.h"
#include "virusscanneditemdelegate.h"
#include "scanworker.h"

#include <DTipLabel>
#include <DListView>
#include <DStandardItem>

#include <QLabel>
#include <QProgressBar>
#include <QPushButton>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QTableView>
#include <QStandardItemModel>
#include <QDateTime>
#include <QTimer>
#include <QDir>
#include <QThread>
#include <QDebug>

DWIDGET_USE_NAMESPACE

VirusScanningWidget::VirusScanningWidget(ScanWorker *worker, int scanType, QWidget *parent)
    : QWidget(parent)
    , m_centralLayout(new QVBoxLayout)
    , m_usedTime(new CircleLabel("15:53", Qt::green))
    , m_scanning(new QLabel("quick scanning on going..."))
    , m_abnormalTxt(new DTipLabel(""))
    , m_progressBar(new QProgressBar)
    , m_scanDetailTxt(new QLabel("Scanning system related abnormals"))
    , m_stopBtn(new QPushButton("Pause..."))
    , m_predictedTime(new QLabel())
    , m_line(new QLabel("---------------------------------------------------------------------"))
    , m_tb(new DListView)
    , m_scanedModel(new QStandardItemModel)
    , m_scanThread(new QThread())
    , m_worker(worker)
    , m_startTime(QDateTime::currentSecsSinceEpoch())
    , m_isScanning(true)
    , m_abnormalCnt(0)
    , m_total(0)
    , m_progress(0)
    , m_lastUpdateTime(0)
{
    Q_UNUSED(scanType);
    QHBoxLayout *topLayout = new QHBoxLayout;
    m_usedTime->setFixedSize(80, 80);
    topLayout->addWidget(m_usedTime, 2);

    QVBoxLayout *toprightLayout = new QVBoxLayout;

    QHBoxLayout *scanInfoLayout = new QHBoxLayout;
    scanInfoLayout->addWidget(m_scanning);
    scanInfoLayout->addWidget(m_abnormalTxt);
    scanInfoLayout->addWidget(m_stopBtn, 0, Qt::AlignRight);
    toprightLayout->addLayout(scanInfoLayout);

    connect(m_stopBtn, &QPushButton::clicked, this, [=] {
        m_isScanning = !m_isScanning;
        if (m_isScanning) {
            m_stopBtn->setText("Pause");
            m_worker->resume();
        } else {
            m_stopBtn->setText("Resume");
            m_worker->pause();
        }
    });

    QHBoxLayout *btnLayout = new QHBoxLayout;
    btnLayout->setMargin(10);
    btnLayout->setSpacing(30);
    m_progressBar->setRange(0, 100);
    m_progressBar->setValue(0);
//    m_progressBar->setFormat("");
    m_progressBar->setFixedHeight(6);
    btnLayout->addWidget(m_progressBar);
    btnLayout->addWidget(m_predictedTime, 0, Qt::AlignRight);
    toprightLayout->addLayout(btnLayout);
    toprightLayout->addWidget(m_scanDetailTxt);

    topLayout->addLayout(toprightLayout, 8);

    m_centralLayout->addLayout(topLayout);
    m_centralLayout->addWidget(m_line);
    m_centralLayout->addWidget(m_tb);

    setLayout(m_centralLayout);

    m_tb->setFrameStyle(QFrame::NoFrame);
    m_tb->setEditTriggers(QAbstractItemView::NoEditTriggers);
    m_tb->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    m_tb->setModel(m_scanedModel);
    m_tb->setItemDelegate(new VirusScannedItemDelegate);
    //add header
    onAddItem("Scan Item", "Scan Statics", "State", -1);
    m_worker->moveToThread(m_scanThread);
    connect(this, &VirusScanningWidget::operate, m_worker, &ScanWorker::scan);
    connect(m_scanThread, &QThread::finished, m_scanThread, &QObject::deleteLater);
    connect(m_worker, &ScanWorker::over, this, &VirusScanningWidget::requestScanResultWidget);
    connect(m_worker, &ScanWorker::progress, this, &VirusScanningWidget::onProgress);
    connect(m_worker, &ScanWorker::totalItem, this, [=](int total) {
        m_total = total;
        int seconds = static_cast<int>(0.3 * total);
        m_predictedTime->setText(QString("Predict time : %1 min").arg(seconds / 60));
    });

    qDebug() << "current thread " << QThread::currentThreadId();
    m_scanThread->start();
    while (!m_scanThread->isRunning()) {
        qDebug() << "wait for thread to start";
        QThread::currentThread()->msleep(100);
    }
    Q_EMIT operate(QDir::homePath());
}

void VirusScanningWidget::onAddItem(const QString &itemName, const QString &itemStatics, const QString &itemState, int type)
{
    DStandardItem *item = new DStandardItem();
    item->setData(itemName, VirusScannedItemDelegate::ItemNameRole);
    item->setData(itemStatics, VirusScannedItemDelegate::ItemStaticsRole);
    if (type < VirusScannedItemDelegate::NORMAL)
        item->setData(itemState, VirusScannedItemDelegate::ItemStateRole);
    item->setData(type, VirusScannedItemDelegate::ItemLevelRole);

    m_scanedModel->appendRow(item);
    updateUsedTime();
}

void VirusScanningWidget::updateUsedTime(int progress)
{
    qint64 usedSecs = QDateTime::currentSecsSinceEpoch() - m_startTime;
    QColor color = Qt::green;
    if (m_abnormalCnt > 0)
        color = Qt::red;
    if (usedSecs - m_lastUpdateTime < 1 && (progress - m_progress < 5))
        return;
    m_lastUpdateTime = usedSecs;

    if (usedSecs > 3600) {
        m_usedTime->setText(QString("%1:%2").arg(usedSecs / 3600, 2, 10, QLatin1Char('0')).arg(usedSecs % 3600, 2, 10, QLatin1Char('0')),
                            color);
    } else {
        m_usedTime->setText(QString("%1:%2").arg(usedSecs / 60, 2, 10, QLatin1Char('0')).arg(usedSecs % 60, 2, 10, QLatin1Char('0')),
                            color);
    }
    m_usedTime->setValue(progress);
}

void VirusScanningWidget::onProgress(int value, int type, const QString &itemInfo)
{
    if (type == VirusScannedItemDelegate::ABNORMAL)
        m_abnormalCnt++;
    m_scanDetailTxt->setText(QString("Scanning %1...").arg(itemInfo));
    onAddItem("virus scan", itemInfo, "", type);
    m_progressBar->setValue(value);
    updateUsedTime(value);
    m_abnormalTxt->setText(QString("! %1 abnormal out of %2 items").arg(m_abnormalCnt).arg(m_total));
}
