/*
 * Copyright (C) 2017 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     LiLinling <lilinling_cm@deepin.com>
 *
 * Maintainer: LiLinling <lilinling_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "virusscanresultitemdelegate.h"

#include <DStyle>

#include <QFont>
#include <QPainter>
#include <QApplication>
#include <QMouseEvent>
#include <QDebug>

DWIDGET_USE_NAMESPACE

static inline QFlags<Qt::AlignmentFlag> alignmentFlag(int col)
{
    auto emCol = static_cast<VirusScanResultItemDelegate::ScannResultColumn>(col);
    switch (emCol) {
    case VirusScanResultItemDelegate::ItemName:
    case VirusScanResultItemDelegate::ItemDesc:
    case VirusScanResultItemDelegate::ItemState:
        return (Qt::AlignLeft | Qt::AlignVCenter);
    case VirusScanResultItemDelegate::ColumnButt:
        break;
    default:
        break;
    }
    return (Qt::AlignLeft | Qt::AlignVCenter);
}

static inline QRect colRect(int col, const QStyleOptionViewItem &option)
{
    static auto tailwidth  = 120;
    auto w = option.rect.width() - tailwidth;

    auto emCol = static_cast<VirusScanResultItemDelegate::ScannResultColumn>(col);
    switch (emCol) {
    case VirusScanResultItemDelegate::ItemName:
        return QRect(0, option.rect.y(), w / 2, option.rect.height());
    case VirusScanResultItemDelegate::ItemDesc:
        return QRect(w/2, option.rect.y(), w / 2, option.rect.height());
    case VirusScanResultItemDelegate::ItemState:
        return QRect(w, option.rect.y(), tailwidth, option.rect.height());
    case VirusScanResultItemDelegate::ColumnButt:
        break;
    default:
        break;
    }
    return option.rect.marginsRemoved(QMargins(0, 0, 0, 0));
}

static inline QRect CheckBoxRect(const QRect &rect, const QSize &sz)
{
    auto centerF = QRectF(rect).center();
    auto iconRect = QRect(rect.x(), static_cast<int>(centerF.y() - sz.height() / 2), sz.width(), sz.height());
    return iconRect;
}

VirusScanResultItemDelegate::VirusScanResultItemDelegate(QWidget *parent)
    : QStyledItemDelegate(parent)
{

}

void VirusScanResultItemDelegate::initStyleOption(QStyleOptionViewItem *option, const QModelIndex &index) const
{
    Q_UNUSED(option);
    Q_UNUSED(index);
}

void VirusScanResultItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    painter->save();
    painter->setRenderHint(QPainter::Antialiasing);

    DStyle* sty = qobject_cast<DStyle *>(QApplication::style());
    auto ico = sty->standardIcon(DStyle::SP_IndicatorSearch);
    bool hasCheck = index.data(Qt::DisplayRole).toBool();
    bool checked = index.data(ItemCheckRole).toBool();

    for (int col = 0; col < ColumnButt; ++col) {
        auto flag = alignmentFlag(col);
        auto rect = colRect(col, option);
        switch (col) {
        case ItemName: {
            auto type = index.data(ItemLevelRole).toInt();
            auto sz = QSize();
            if (type != -1)
                sz = QSize(15, 15);
            auto icon = ico.pixmap(sz);
            auto iconRect = CheckBoxRect(rect, sz);
            auto txtRect = QRectF(10 + sz.width(), rect.y(), rect.width() - 10 - sz.width(), rect.height());
            if (hasCheck) {
                QStyleOptionButton check_box_style_option;
                check_box_style_option.state |= QStyle::State_Enabled;
                if(checked)
                {
                    check_box_style_option.state |= QStyle::State_On;
                }
                else
                {
                    check_box_style_option.state |= QStyle::State_Off;
                }
                check_box_style_option.rect = iconRect;
                QApplication::style()->drawControl(QStyle::CE_CheckBox, &check_box_style_option, painter);
            } else if (type != -1)
                painter->drawPixmap(iconRect, icon, icon.rect());
            QFont font(painter->font());
            QFontMetrics fm(font);
            auto text = fm.elidedText(index.data(ItemNameRole).toString(), Qt::ElideMiddle, rect.width() - 10 - sz.width());
            auto color = QApplication::palette().text().color();
            if (type == SUMMARY_DANGEROUS) {
                color = Qt::darkRed;
                text = fm.elidedText(index.data(ItemSummaryRole).toString(), Qt::ElideMiddle, rect.width() - 10 - sz.width());
            } else if (type == SUMMARY_FATAL) {
                color = Qt::red;
                text = fm.elidedText(index.data(ItemSummaryRole).toString(), Qt::ElideMiddle, rect.width() - 10 - sz.width());
            }
            QPen pen = painter->pen();
            pen.setColor(color);
            painter->setPen(pen);
            painter->drawText(txtRect, static_cast<int>(flag), text);
            break;
        }
        case ItemDesc: {
            QFont font(painter->font());
            QFontMetrics fm(font);
            auto text = fm.elidedText(index.data(ItemDescRole).toString(), Qt::ElideMiddle, rect.width());
            auto color = QApplication::palette().text().color();
            QPen pen = painter->pen();
            pen.setColor(color);
            painter->setPen(pen);
            painter->drawText(rect, static_cast<int>(flag), text);
            break;
        }
        case ItemState: {
            QFont font(painter->font());
            QFontMetrics fm(font);
            auto text = fm.elidedText(index.data(ItemStateRole).toString(), Qt::ElideMiddle, rect.width());
            auto type = index.data(ItemLevelRole).toInt();
            if (type != 0) {
                QPen pen = painter->pen();
                pen.setColor(Qt::red);
                painter->setPen(pen);
            }
            painter->drawText(rect, static_cast<int>(flag), text);
            break;
        }
        default:
            break;
        }
    }
    painter->restore();
}

bool VirusScanResultItemDelegate::editorEvent(QEvent *event, QAbstractItemModel *model, const QStyleOptionViewItem &option, const QModelIndex &index)
{
    if((event->type() == QEvent::MouseButtonRelease) ||
        (event->type() == QEvent::MouseButtonDblClick)) {
        QMouseEvent *mouse_event = static_cast<QMouseEvent*>(event);
        auto rect = colRect(ItemName, option);
        auto checkBoxRect = CheckBoxRect(rect, QSize(15, 15));
        checkBoxRect = checkBoxRect.adjusted(-2, -2, 2, 2);

        if (mouse_event->button() != Qt::LeftButton ||
           !checkBoxRect.contains(mouse_event->pos())) {
            return false;
        }

        if(event->type() == QEvent::MouseButtonDblClick) {
            return true;
        }
    } else if(event->type() == QEvent::KeyPress) {
        if(static_cast<QKeyEvent*>(event)->key() != Qt::Key_Space &&
           static_cast<QKeyEvent*>(event)->key() != Qt::Key_Select)
        {
            return false;
        }
    } else {
        return false;
    }

    bool checked = index.model()->data(index, ItemCheckRole).toBool();
    return model->setData(index, !checked, ItemCheckRole);
}
