/*
 * Copyright (C) 2017 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     LiLinling <lilinling_cm@deepin.com>
 *
 * Maintainer: LiLinling <lilinling_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "virusscanresultwidget.h"
#include "circlelabel.h"
#include "virusscanresultitemdelegate.h"

#include <DSuggestButton>
#include <DListView>
#include <DStyle>

#include <QLabel>
#include <QPushButton>
#include <QPixmap>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QDebug>

DWIDGET_USE_NAMESPACE

VirusScanResultWidget::VirusScanResultWidget(QWidget *parent)
    : QWidget(parent)
    , m_resultIcon(new CircleLabel)
    , m_totalTxt(new QLabel("Found 4 items in this scan"))
    , m_backBtn(new QPushButton("back"))
    , m_handleBtn(new DSuggestButton("handle"))
    , m_detailTxt(new QLabel("took 5 minutes to found 4 dangerous items from 500 items"))
    , m_line(new QLabel("---------------------------------------------------------------------"))
    , m_tb(new DListView)
    , m_model(new QStandardItemModel)
    , m_fatalCnt(0)
    , m_dangerousCnt(0)
{
    QVBoxLayout *mainLayout = new QVBoxLayout;

    QHBoxLayout *topLayout = new QHBoxLayout;
    QHBoxLayout *topinfoLayout = new QHBoxLayout;
    topinfoLayout->addWidget(m_totalTxt);
    topinfoLayout->addWidget(m_backBtn);
    QVBoxLayout *topCLayout = new QVBoxLayout;
    topCLayout->addLayout(topinfoLayout);
    topCLayout->addWidget(m_detailTxt);

    connect(m_backBtn, &QPushButton::clicked, this, [=] {
        Q_EMIT requestBack();
    });
    DStyle *sty = qobject_cast<DStyle *>(style());
    QIcon ico = sty->standardIcon(DStyle::SP_ShowPassword);
    m_resultIcon->setFixedSize(48, 48);
    m_resultIcon->setPixmap(ico.pixmap(48, 48), Qt::red);
    topLayout->addWidget(m_resultIcon);
    topLayout->addLayout(topCLayout);
    topLayout->addWidget(m_handleBtn);

    mainLayout->addLayout(topLayout);
    mainLayout->addWidget(m_line);
    mainLayout->addWidget(m_tb);
    setLayout(mainLayout);
    m_tb->setModel(m_model);
    m_tb->setItemDelegate(new VirusScanResultItemDelegate);
    m_tb->setEditTriggers(QAbstractItemView::NoEditTriggers);
    //add header
    onAddItem(false, true, "Scan Item", "Description", "State", -1);
    //add test code
    for (int i = 0; i < 10; ++i) {
        if (i % 3 == 0) {
            onAddItem(true, (i % 2 == 0), "fatal virus detect", "scan 1 item in total", "FATAL", VirusScanResultItemDelegate::FATAL);
        } else {
            onAddItem(true, (i % 2 == 0), "dangerous virus detect", "scan 1 item in total", "Dangerous", VirusScanResultItemDelegate::DANGEROUS);
        }
    }
}

void VirusScanResultWidget::onAddItem(bool hasCheck, bool isChecked, const QString &name, const QString &desc, const QString &state, int type)
{
    QStandardItem *item = new QStandardItem;
    item->setData(hasCheck, Qt::DisplayRole);
    item->setData(isChecked, VirusScanResultItemDelegate::ItemCheckRole);
    item->setData(name, VirusScanResultItemDelegate::ItemNameRole);
    item->setData(desc, VirusScanResultItemDelegate::ItemDescRole);
    item->setData(state, VirusScanResultItemDelegate::ItemStateRole);
    item->setData(type, VirusScanResultItemDelegate::ItemLevelRole);

    int itemRow = 0;
    int summaryRow = 1;

    if (type == VirusScanResultItemDelegate::FATAL) {
        m_fatalCnt++;
        QStandardItem *sitem = nullptr;
        if (m_summaryMap.contains(type)) {
            sitem = m_summaryMap.value(type);
            sitem->setData(QString("total %1 fatal items").arg(m_fatalCnt), VirusScanResultItemDelegate::ItemSummaryRole);
        } else {
            sitem = new QStandardItem;
            sitem->setData(QString("total %1 fatal items").arg(m_fatalCnt), VirusScanResultItemDelegate::ItemSummaryRole);
            sitem->setData(VirusScanResultItemDelegate::SUMMARY_FATAL, VirusScanResultItemDelegate::ItemLevelRole);
            m_model->insertRow(summaryRow, sitem);
            m_summaryMap.insert(type, sitem);
        }
        itemRow = summaryRow + m_fatalCnt;
    } else if (type == VirusScanResultItemDelegate::DANGEROUS) {
        m_dangerousCnt++;
        QStandardItem *sitem = nullptr;
        if (m_fatalCnt > 0)
            summaryRow += m_fatalCnt + 1;

        if (m_summaryMap.contains(type)) {
            sitem = m_summaryMap.value(type);
            sitem->setData(QString("total %1 dangerous items").arg(m_dangerousCnt), VirusScanResultItemDelegate::ItemSummaryRole);
        } else {
            sitem = new QStandardItem;
            sitem->setData(QString("total %1 dangerous items").arg(m_dangerousCnt), VirusScanResultItemDelegate::ItemSummaryRole);
            sitem->setData(VirusScanResultItemDelegate::SUMMARY_DANGEROUS, VirusScanResultItemDelegate::ItemLevelRole);
            m_model->insertRow(summaryRow, sitem);
            m_summaryMap.insert(type, sitem);
        }
        itemRow = summaryRow + m_dangerousCnt;
    }
    m_model->insertRow(itemRow, item);
}

void VirusScanResultWidget::onAddSummary(const QString &summary)
{
    QStandardItem *item = new QStandardItem;
    item->setData(summary, VirusScanResultItemDelegate::ItemSummaryRole);
    m_model->appendRow(item);
}
