/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "examdetailpage.h"
#include "widgets/scoreprogressbar.h"
#include "widgets/fixitemwindow.h"

#include <DFontSizeManager>
#include <DApplicationHelper>
#include <DSuggestButton>
#include <DCommandLinkButton>
#include <DProgressBar>

#include <QDebug>
#include <QTime>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QLabel>
#include <QPushButton>
#include <QRadioButton>
#include <QButtonGroup>
#include <QProgressBar>
#include <QScrollArea>
#include <QGSettings>
#include <QThread>
#include <QStackedLayout>
#include <QGraphicsDropShadowEffect>
#include <QGraphicsOpacityEffect>

DWIDGET_USE_NAMESPACE

const QColor ScoreVerySafeColor = QColor(35, 196, 0, 255);
const QColor ScoreSafeColor = QColor(158, 202, 0, 255);
const QColor ScoreHaveDangersColor = QColor(255, 93, 0, 255);
const QColor ScoreUnderIllnessColor = QColor(216, 49, 108, 255);

//shadow color
const QColor ScoreVerySafeShadowColor = QColor(35, 196, 0, 128);
const QColor ScoreSafeShadowColor = QColor(158, 202, 0, 128);
const QColor ScoreHaveDangersShadowColor = QColor(255, 93, 0, 128);
const QColor ScoreUnderIllnessShadowColor = QColor(216, 49, 108, 128);

ExamDetailPage::ExamDetailPage(QWidget *parent)
    : QWidget(parent)
    , m_mode(nullptr)
    , m_work(nullptr)
    , m_leftLayout(nullptr)
    , m_scoreLabel(nullptr)
    , m_currentValue(0)
    , m_value(0)
    , m_moveDir(1)
    , m_scoreTimer(nullptr)
    , m_centLabel(nullptr)
    , m_scoreProg(nullptr)
    , m_progressBarShadow(nullptr)
    , m_securLeftLabel(nullptr)
    , m_securRightLabel(nullptr)
    , m_itemsArea(nullptr)
    , m_noNeedFixItemImg(nullptr)
{
    QHBoxLayout *mainLayout = new QHBoxLayout;
    mainLayout->setMargin(0);
    setLayout(mainLayout);

    m_leftLayout = new QVBoxLayout;
    m_leftLayout->setContentsMargins(20, 50, 0, 0);
    m_leftLayout->setAlignment(Qt::AlignLeft);
    mainLayout->addLayout(m_leftLayout);

    //safety score
    m_scoreTimer = new QTimer;
    m_scoreTimer->setInterval(10);
    m_moveDir = 1;
    m_currentValue = 0;

    QHBoxLayout *scoreLayout = new QHBoxLayout; //系统评分行，水平布局
    scoreLayout->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
    m_leftLayout->addLayout(scoreLayout);

    m_scoreLabel = new QLabel;
    m_scoreLabel->setFixedSize(80, 72);
    m_scoreLabel->setAlignment(Qt::AlignRight | Qt::AlignBottom);
    QFont scoreFont = m_scoreLabel->font();
    scoreFont.setBold(true);
    m_scoreLabel->setFont(scoreFont);
    m_scoreLabel->setContentsMargins(0, 0, 0, 0);
    DFontSizeManager::instance()->bind(m_scoreLabel, DFontSizeManager::T1);
    scoreLayout->addWidget(m_scoreLabel, 0, Qt::AlignRight | Qt::AlignBottom);

    m_centLabel = new QLabel("cent");
    QString systemLanguageStr = QLocale::system().name();
    if ('z' != systemLanguageStr[0] || 'h' != systemLanguageStr[1])
        m_centLabel->setText(""); //如果语言不是中文就隐藏
    m_centLabel->setContentsMargins(0, 0, 0, 10);
    DFontSizeManager::instance()->bind(m_centLabel, DFontSizeManager::T5);
    scoreLayout->addWidget(m_centLabel, 0, Qt::AlignLeft | Qt::AlignBottom);

    //safety score right part
    scoreLayout->addSpacing(13);
    QVBoxLayout *scoreRightLayout = new QVBoxLayout;
    scoreRightLayout->setContentsMargins(0, 0, 0, 0);
    scoreLayout->addLayout(scoreRightLayout);

    //system security
    QHBoxLayout *securityLayout = new QHBoxLayout;
    securityLayout->setContentsMargins(0, 0, 0, 0);
    securityLayout->setSpacing(0);
    securityLayout->setMargin(0);
    securityLayout->setAlignment(Qt::AlignLeft);
    scoreRightLayout->addLayout(securityLayout);

    m_securLeftLabel = new QLabel(tr("Overview:"));
    QFont securLabelFont = m_securLeftLabel->font();
    securLabelFont.setBold(true);
    m_securLeftLabel->setFont(securLabelFont);
    m_securLeftLabel->setContentsMargins(0, 0, 0, 0);
    DFontSizeManager::instance()->bind(m_securLeftLabel, DFontSizeManager::T5);
    securityLayout->addWidget(m_securLeftLabel, 0, Qt::AlignLeft | Qt::AlignBottom);

    m_securRightLabel = new QLabel;
    m_securRightLabel->setFont(securLabelFont);
    m_securRightLabel->setContentsMargins(0, 0, 0, 0);
    DFontSizeManager::instance()->bind(m_securRightLabel, DFontSizeManager::T5);
    securityLayout->addWidget(m_securRightLabel, 0, Qt::AlignLeft | Qt::AlignBottom);

    //score progress bar
    scoreRightLayout->addSpacing(9);
    QStackedLayout * scoreProgressLayout = new QStackedLayout;
    scoreProgressLayout->setStackingMode(QStackedLayout::StackAll);
    scoreProgressLayout->setContentsMargins(0, 0, 0, 0);
    scoreProgressLayout->setMargin(0);
    scoreProgressLayout->setSpacing(0);
    scoreRightLayout->addLayout(scoreProgressLayout);

    QSize scoreBarSize = QSize(220, 12);
    ScoreProgressBar *scoreBg = new ScoreProgressBar;
    scoreBg->setFixedSize(scoreBarSize);
    scoreBg->setCurrentValue(100);
    scoreProgressLayout->addWidget(scoreBg);
    scoreBg->lower();

    m_scoreProg = new ScoreProgressBar;
    m_scoreProg->setFixedSize(scoreBarSize);
    m_scoreProg->setColor(QColor(0, 255, 0, 255));
    m_scoreProg->setCurrentValue(0);
    m_scoreProg->setValue(0);
    scoreProgressLayout->addWidget(m_scoreProg);
    m_scoreProg->raise();

    m_progressBarShadow = new QGraphicsDropShadowEffect(this); //阴影
    m_progressBarShadow->setOffset(0, 3);
    m_progressBarShadow->setBlurRadius(20);
    m_progressBarShadow->setColor(QColor(0, 255, 0, 128));
    m_scoreProg->setGraphicsEffect(m_progressBarShadow);
    scoreRightLayout->addSpacing(14);

    //need fix items
    m_leftLayout->addSpacing(40);
    m_fixItemInfoLst = {
        {"1", safetylevel, "def_safetylevel_high", tr("Security level: Low"), "please use with caution", "Level Up"},
        {"2", safetylevel, "def_safetylevel_high", tr("Security level: Medium"), "please use with caution", "Level Up"},
        {"3", virus, "def_safetylevel_high", "Virus bank is not the latest", "please update it", "Update the virus bank"},
        {"4", virus, "def_safetylevel_high", "Virus killing recently is not in \"wholesale\" model", "remain some dangerous", "Wholesale kill"},
        {"5", virus, "def_safetylevel_high", "No antivirus for more than 60 days", "please antivirus in time", "Wholesale kill"},
        {"6", virus, "def_safetylevel_high", "No antivirus for more than 15 days", "please antivirus in time", "Wholesale kill"},

    };
    initFixItems(m_fixItemInfoLst);

    //禁用第一个修复项的按钮
    m_fixItemLst.at(0).btn->setEnabled(false);
    QGraphicsOpacityEffect *opacity = new QGraphicsOpacityEffect;
    opacity->setOpacity(0.8);
    m_fixItemLst.at(0).btn->setGraphicsEffect(opacity);

    //no need fix items background
    m_noNeedFixItemImg = new QLabel;
    m_noNeedFixItemImg->setFixedHeight(283);
    m_noNeedFixItemImg->setPixmap(QIcon::fromTheme("dcc_full_marks_empty").pixmap(145, 106));
    m_leftLayout->addWidget(m_noNeedFixItemImg, 0, Qt::AlignCenter);

    QPushButton *reExamineBtn = new QPushButton("Examine again");
    m_leftLayout->addWidget(reExamineBtn, 0,Qt::AlignLeft);

    m_leftLayout->addStretch(1);

    //right picture
    mainLayout->addStretch(1);
    static QIcon icon = QIcon::fromTheme(":/icons/deepin/builtin/icons/dcc_home_detection_image_300px.png");

    QLabel *rightPic = new QLabel;
    rightPic->setPixmap(icon.pixmap(300, 300));
    mainLayout->addWidget(rightPic, 0, Qt::AlignCenter);

    //connect
    connect(m_scoreTimer, &QTimer::timeout, this, &ExamDetailPage::updateScoreValue);

    //connect fix items' button
    int deltaId = 1;
    //connect(m_fixItemLst[1 - deltaId].btn, &DCommandLinkButton::clicked, this, &ExamDetailPage::requestShowSafetyLevelWidgt);
    connect(m_fixItemLst[2 - deltaId].btn, &DCommandLinkButton::clicked, this, &ExamDetailPage::requestShowSafetyLevelWidgt);
    connect(m_fixItemLst[4 - deltaId].btn, &DCommandLinkButton::clicked, this, &ExamDetailPage::requestShowAntivirusWidgt);
    connect(m_fixItemLst[5 - deltaId].btn, &DCommandLinkButton::clicked, this, &ExamDetailPage::requestShowAntivirusWidgt);
    connect(m_fixItemLst[6 - deltaId].btn, &DCommandLinkButton::clicked, this, &ExamDetailPage::requestShowAntivirusWidgt);

    //connect examine again button
    connect(reExamineBtn, &QPushButton::clicked, this, &ExamDetailPage::requestReEaxamine);
}

void ExamDetailPage::initFixItems(QList<FixItemInfo> InfoLst)
{
    QVBoxLayout *itemsLayout = new QVBoxLayout;
    itemsLayout->setAlignment(Qt::AlignTop);
    itemsLayout->setSizeConstraint(QLayout::SetMinimumSize);
    itemsLayout->setSpacing(10);
    itemsLayout->setMargin(0);
    itemsLayout->setContentsMargins(0, 0, 0, 0);

    QWidget *itemsWidgt = new QWidget(this);
    itemsWidgt->setLayout(itemsLayout);

    m_itemsArea = new QScrollArea;
    m_itemsArea->setFixedHeight(283);
    m_itemsArea->setAlignment(Qt::AlignTop);
    m_itemsArea->setWidgetResizable(true);
    m_itemsArea->setWidget(itemsWidgt);
    m_itemsArea->setFrameShape(QFrame::NoFrame);
    m_leftLayout->addWidget(m_itemsArea); //向左布局中加入滑动区域

    for (auto info : InfoLst) {
        FixItem fixItemTmp;
        fixItemTmp.id = info.id;

        FixItemWindow *itemWindowTmp = new FixItemWindow;
        itemWindowTmp->setContentsMargins(0, 0, 0, 0);
        itemWindowTmp->setFixedWidth(335);
        itemWindowTmp->setFixedHeight(40);
        itemWindowTmp->setIcon(QIcon::fromTheme(info.iconPath));
        itemWindowTmp->setText(info.title);
        itemWindowTmp->setButtonText(info.btnStr);
        itemsLayout->addWidget(itemWindowTmp);

        fixItemTmp.widget = itemWindowTmp;
        fixItemTmp.btn = itemWindowTmp->getButton();
        m_fixItemLst.append(fixItemTmp);
    }
}

void ExamDetailPage::setModelAndWork(HomePageModel *mode, HomePageWork *work)
{
     m_mode = mode;
     m_work = work;
}

void ExamDetailPage::setScoreValue(int value)
{
    m_value = value;

    if (m_value == m_currentValue) {
        m_scoreLabel->setNum(m_value);
        return;
    }
    else if (m_value > m_currentValue) m_moveDir = 1;
    else if(m_value < m_currentValue) m_moveDir = -1;

    m_scoreTimer->start();
}

void ExamDetailPage::updateScoreValue()
{
    m_currentValue += m_moveDir;
    m_scoreLabel->setNum(m_currentValue);
    if (m_value == m_currentValue) m_scoreTimer->stop();

    update();
}

void ExamDetailPage::showResult(QList<int> fixItemIdLst)
{
    int score = m_mode->getSafetyScore();
    setScoreValue(score);
    if (94 < score) {
        QPalette paTmp = m_scoreLabel->palette();
        paTmp.setColor(QPalette::Text, ScoreVerySafeColor);
        m_scoreLabel->setPalette(paTmp);

        m_securRightLabel->setText(tr("Very Safe"));
        m_scoreProg->setValue(score);
        m_scoreProg->setColor(ScoreVerySafeColor);
        m_progressBarShadow->setColor(ScoreVerySafeShadowColor);
    }else if (79 < score) {
        QPalette paTmp = m_scoreLabel->palette();
        paTmp.setColor(QPalette::Text, ScoreSafeColor);
        m_scoreLabel->setPalette(paTmp);

        m_securRightLabel->setText(tr("Safe"));
        m_scoreProg->setValue(score);
        m_scoreProg->setColor(ScoreSafeColor);
        m_progressBarShadow->setColor(ScoreSafeShadowColor);
    }else if (59 < score) {
        QPalette paTmp = m_scoreLabel->palette();
        paTmp.setColor(QPalette::Text, ScoreHaveDangersColor);
        m_scoreLabel->setPalette(paTmp);

        m_securRightLabel->setText(tr("Risky"));
        m_scoreProg->setValue(score);
        m_scoreProg->setColor(ScoreHaveDangersColor);
        m_progressBarShadow->setColor(ScoreHaveDangersShadowColor);
    }else{
        QPalette paTmp = m_scoreLabel->palette();
        paTmp.setColor(QPalette::Text, ScoreUnderIllnessColor);
        m_scoreLabel->setPalette(paTmp);

        m_securRightLabel->setText(tr("At High Risk"));
        m_securRightLabel->setPalette(paTmp);
        m_scoreProg->setValue(score);
        m_scoreProg->setColor(ScoreUnderIllnessColor);
        m_progressBarShadow->setColor(ScoreUnderIllnessShadowColor);
    }

    //show fix items
    if (0 == fixItemIdLst.size()) { //如果没有待修复项
        m_itemsArea->hide();
        m_noNeedFixItemImg->show();
    } else {
        m_itemsArea->show();
        m_noNeedFixItemImg->hide();
    }

    for (FixItem fixItem : m_fixItemLst) {
        fixItem.widget->hide();
    }
    for (int i : fixItemIdLst) {
        m_fixItemLst[i - 1].widget->show();
    }

}
