/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "window/namespace.h"
#include "dtkwidget_global.h"
#include "widgets/scoreprogressbar.h"
#include "window/modules/homepage/homepagework.h"
#include "window/modules/homepage/examworker.h"

#include <DCommandLinkButton>
#include <DProgressBar>

#include <QWidget>
#include <QLabel>
#include <QProgressBar>
#include <QScrollArea>
#include <QGSettings>
#include <QGraphicsDropShadowEffect>

DWIDGET_BEGIN_NAMESPACE
class DSuggestButton;
class DCommandLinkButton;
DWIDGET_END_NAMESPACE

QT_BEGIN_NAMESPACE
class QHBoxLayout;
class QVBoxLayout;
class QLabel;
class QPushButton;
QT_END_NAMESPACE

DWIDGET_USE_NAMESPACE

DEF_NAMESPACE_BEGIN
DEF_ANTIVIRUS_NAMESPACE_BEGIN

class ExamDetailPage : public QWidget
{
    Q_OBJECT
public:
    enum FixItemStyle {safetylevel, virus, firewalls};
    struct FixItemInfo {
        QString id; //修复项标示
        FixItemStyle style;
        QString iconPath;
        QString title;
        QString tip;
        QString btnStr;
    };
    struct FixItem {
        QString id; //修复项标示
        QWidget *widget;
        DCommandLinkButton *btn;
    };

public:
    explicit ExamDetailPage(QWidget *parent = nullptr);

    void setModelAndWork(HomePageModel *mode, HomePageWork *work);
    void setScoreValue(int value);
    void showResult(QList<int> fixItemIdLst);

private:
    void initFixItems(QList<FixItemInfo> InfoLst);
    void showLastResult();

Q_SIGNALS:
    void requestBack();
    void requestShowSafetyLevelWidgt();
    void requestShowAntivirusWidgt();
    void requestReEaxamine();

public Q_SLOT:
    void examThreadStart();
    void updateScoreValue();
    void examining(int score, QString tip); //系统体检

private:
    HomePageModel *m_mode;
    HomePageWork *m_work;
    QVBoxLayout *m_leftLayout;
    QLabel *m_scoreLabel;
    int m_currentValue;
    int m_value;
    int m_moveDir;
    QTimer *m_scoreTimer;
    QLabel *m_centLabel;
    ScoreProgressBar *m_scoreProg;
    QGraphicsDropShadowEffect *m_progressBarShadow;
    QLabel *m_securLeftLabel;
    QLabel *m_securRightLabel;
    QLabel *m_scoreTip;
    QList<FixItemInfo> m_fixItemInfoLst;
    QList<FixItem> m_fixItemLst;
    QScrollArea *m_itemsArea;
    QLabel *m_noNeedFixItemImg;
    bool m_isbExamining;
};

DEF_ANTIVIRUS_NAMESPACE_END
DEF_NAMESPACE_END
