/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "homepagemodule.h"
#include "window/interface/moduleinterface.h"
#include "window/interface/frameproxyinterface.h"
#include "window/modules/homepage/homepagewidget.h"

#include <QModelIndex>
#include <QStandardItemModel>

HomePageModule::HomePageModule(FrameProxyInterface *frame, QObject *parent)
    : QObject(parent)
    , ModuleInterface(frame)
    , m_homePageModel(nullptr)
    , m_homePageWork(nullptr)
    , m_homePageWidget(nullptr)
    , m_examDetailPage(nullptr)
{

}

void HomePageModule::initialize()
{

}

void HomePageModule::preInitialize()
{
    m_homePageModel = new HomePageModel;
    m_homePageWork = new HomePageWork(m_homePageModel);
}

const QString HomePageModule::name() const
{
    return "homepage";
}

void HomePageModule::active()
{
    m_homePageWidget = new HomePageWidget;
    m_homePageWidget->setModelAndWork(m_homePageModel, m_homePageWork);
    m_homePageWidget->updateUI();
    connect(m_homePageWidget, &HomePageWidget::requestShowScoreDetailPage, this, &HomePageModule::showScoreDetailPage);

    m_frameProxy->pushWidget(this, m_homePageWidget);
}

void HomePageModule::deactive()
{

}

void HomePageModule::showScoreDetailPage(QList<int> fixItemIdLst)
{
    m_examDetailPage = new ExamDetailPage;
    m_examDetailPage->setModelAndWork(m_homePageModel, m_homePageWork);
    m_examDetailPage->showResult(fixItemIdLst);
    connect(m_examDetailPage, &ExamDetailPage::requestShowSafetyLevelWidgt, this, [=]() {
        m_frameProxy->setCurrentModule(1);
    });
    connect(m_examDetailPage, &ExamDetailPage::requestShowAntivirusWidgt, this, [=]() {
//        m_frameProxy->setCurrentModule(2);
    });
    connect(m_examDetailPage, &ExamDetailPage::requestReEaxamine, this, [=]() {
        //添加m_homePageWidget前，先将旧的页面移除然后再添加，保证页面栈中只有一个homePage
        m_frameProxy->popWidget(this);
        m_frameProxy->pushWidget(this, m_homePageWidget);
        m_homePageWidget->reExamine();
    });
    m_frameProxy->pushWidget(this, m_examDetailPage);

}
