/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "homepagewidget.h"
#include "widgets/scorebar.h"

#include <DFontSizeManager>
#include <DApplicationHelper>
#include <DSuggestButton>

#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QLabel>
#include <QPushButton>
#include <QButtonGroup>

DWIDGET_USE_NAMESPACE

//score stages
const int ScoreVerySafeAbove = 94;
const int ScoreSafeAbove = 79;
const int ScoreHaveDangersAbove = 59;

//score stages color
const QColor ScoreVerySafeColor = QColor(35, 196, 0, 255);
const QColor ScoreSafeColor = QColor(158, 202, 0, 255);
const QColor ScoreHaveDangersColor = QColor(255, 93, 0, 255);
const QColor ScoreUnderIllnessColor = QColor(216, 49, 108, 255);

HomePageWidget::HomePageWidget(QWidget *parent)
    : QWidget(parent)
    , m_mode(nullptr)
    , m_work(nullptr)
    , m_examThread(nullptr)
    , m_examWorker(nullptr)
    , m_scoreTipLeft(nullptr)
    , m_scoreTipRight(nullptr)
    , m_examTip(nullptr)
    , m_examBtn(nullptr)
{
    m_examThread = new QThread;
    m_examWorker = new ExamWorker;
    m_examWorker->moveToThread(m_examThread); //创建体检线程

    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->setAlignment(Qt::AlignHCenter);
    setLayout(mainLayout);

    QLabel *scorePic = new QLabel;
    scorePic->setAlignment(Qt::AlignHCenter);
    QIcon icon= QIcon::fromTheme("dcc_home_image");
    scorePic->setPixmap(icon.pixmap(300, 300));
    mainLayout->addWidget(scorePic);

    //分数等级提示标签
    QHBoxLayout *scoreTipLayout = new QHBoxLayout;
    mainLayout->addLayout(scoreTipLayout);

    m_scoreTipLeft = new QLabel;
    m_scoreTipLeft->setAlignment(Qt::AlignHCenter);
    DFontSizeManager::instance()->bind(m_scoreTipLeft, DFontSizeManager::T1);
    scoreTipLayout->addWidget(m_scoreTipLeft);

    m_scoreTipRight = new QLabel;
    m_scoreTipRight->setAlignment(Qt::AlignLeft);
    DFontSizeManager::instance()->bind(m_scoreTipRight, DFontSizeManager::T1);
    scoreTipLayout->addWidget(m_scoreTipRight);

    //体检提示标签
    m_examTip = new DTipLabel;
    m_examTip->setAlignment(Qt::AlignHCenter);
    DFontSizeManager::instance()->bind(m_examTip, DFontSizeManager::T5);
    mainLayout->addWidget(m_examTip);

    mainLayout->addSpacing(25);
    // examine button
    m_examBtn = new DSuggestButton;
    m_examBtn->setFixedWidth(200);
    mainLayout->addWidget(m_examBtn, 0, Qt::AlignHCenter);

    mainLayout->addStretch(1);

    //connect
    connect(m_examBtn, &DSuggestButton::clicked, this, [=]() {
        switch (m_examineFlag) {
        case normal:
            //start examining
            m_examWorker->run();
            m_examThread->start();
            m_examineFlag = examining;
            break;
        case examining:
            //cancel
            m_examWorker->quit();
            m_examThread->quit();
            m_examineFlag = normal;
            break;
        case pause:
            break;
        }
        this->examineUpdate(""); //根据检测状态更新按钮文字
    });
    connect(m_examThread, &QThread::started, m_examWorker, &ExamWorker::examining); //需要首页体检按钮点击信号 连接 线程m_examThread->start()
    connect(m_examWorker, &ExamWorker::sendExamineProcess, this, &HomePageWidget::examineUpdate);
}

HomePageWidget::~HomePageWidget()
{
    m_examWorker->quit();
    m_examWorker->deleteLater();

    m_examThread->quit();
    connect(m_examThread, &QThread::finished, m_examThread, &QThread::deleteLater); //线程结束后销毁
}

void HomePageWidget::setModelAndWork(HomePageModel *mode, HomePageWork *work)
{
     m_mode = mode;
     m_work = work;
     m_examWorker->setModelAndWork(mode, work);
}

void HomePageWidget::updateUI()
{
    //home page UI control
    m_scoreTipLeft->hide();
    m_scoreTipRight->hide();
    m_examTip->hide();
    m_examBtn->hide();

    int score = m_mode->getSafetyScore();
    if (-1 == score) {
        m_scoreTipLeft->show();
        m_scoreTipLeft->setText(tr("Your computer has not been checked yet"));
        m_examTip->show();
        m_examTip->setText(tr("Fully check security issues with one click"));
        m_examBtn->show();
        m_examBtn->setText(tr("Check Now"));
        return;
    } else if (ScoreVerySafeAbove < score) {
        m_scoreTipLeft->show();
        m_scoreTipLeft->setText(tr("Last Result:"));
        m_scoreTipRight->show();
        QPalette pa = m_scoreTipRight->palette();
        pa.setColor(QPalette::Text, ScoreVerySafeColor);
        m_scoreTipRight->setPalette(pa);
        m_scoreTipRight->setText(tr("Very Safe"));
        m_examTip->show();
        m_examBtn->show();
        m_examBtn->setText(tr("Check Now"));
    }else if (ScoreSafeAbove < score) {
        m_scoreTipLeft->show();
        m_scoreTipLeft->setText(tr("Last Result:"));
        m_scoreTipRight->show();
        QPalette pa = m_scoreTipRight->palette();
        pa.setColor(QPalette::Text, ScoreSafeColor);
        m_scoreTipRight->setPalette(pa);
        m_scoreTipRight->setText(tr("Safe"));
        m_examTip->show();
        m_examBtn->show();
        m_examBtn->setText(tr("Check Now"));
    }else if (ScoreHaveDangersAbove < score) {
        m_scoreTipLeft->show();
        m_scoreTipLeft->setText(tr("Last Result:"));
        m_scoreTipRight->show();
        QPalette pa = m_scoreTipRight->palette();
        pa.setColor(QPalette::Text, ScoreHaveDangersColor);
        m_scoreTipRight->setPalette(pa);
        m_scoreTipRight->setText(tr("Risky"));
        m_examTip->show();
        m_examBtn->show();
        m_examBtn->setText(tr("Check Now"));
    }else{
        m_scoreTipLeft->show();
        m_scoreTipLeft->setText(tr("Last Result:"));
        m_scoreTipRight->show();
        QPalette pa = m_scoreTipRight->palette();
        pa.setColor(QPalette::Text, ScoreUnderIllnessColor);
        m_scoreTipRight->setPalette(pa);
        m_scoreTipRight->setText(tr("At High Risk"));
        m_examTip->show();
        m_examBtn->show();
        m_examBtn->setText(tr("Check Now"));
    }

    //根据上次体检时间，调整体检提示标签文字
    QDateTime lastCheckTime = m_mode->getLastCheckTime();
    qint64 deltaTime = lastCheckTime.secsTo(QDateTime::currentDateTime());

    float minutes = deltaTime / 60;
    float hours = minutes / 60;
    float days = hours / 24;

    qDebug() << "last check time to today: " << days << "days " << hours << "hours " << minutes << "minutes";
    if (lastCheckTime.date() == QDate::currentDate()) {
        if (2 > hours) m_examTip->setText(tr("Checked just now"));
        else m_examTip->setText(tr("Checked %1 hours ago").arg(int(hours)));
    } else {
        if (0 == int(days)) m_examTip->setText(tr("Checked %1 days ago").arg(1));
        else m_examTip->setText(tr("Checked %1 days ago").arg(int(days)));
    }
}

void HomePageWidget::examineUpdate(QString place, int fixItemId)
{
    if (normal == m_examineFlag) {
        updateUI();
        return;
    }

    if ("finished" == place) {
        m_examThread->quit();
        m_examineFlag = normal;
        m_work->setLastCheckTime(QDateTime::currentDateTime());
        updateUI();
        Q_EMIT requestShowScoreDetailPage(m_fixItemIdLst); //检测结束，发送打开评分详细页消息
        return;
    }

    //根据检测状态修改按钮文字
    if (pause == m_examineFlag) {
        m_examBtn->setText("Continue");
    }
    if (examining == m_examineFlag) {
        m_examBtn->setText(tr("Cancel"));
    }
    if ("" == place) return;

    //m_fixItemIdLst
    if (0 != fixItemId)
        m_fixItemIdLst.append(fixItemId);

    if (examining == m_examineFlag) {
        m_scoreTipLeft->show();
        m_scoreTipLeft->setText(tr("Checking..."));
        m_scoreTipRight->hide();
        m_examTip->show();
        m_examTip->setText(place);
        m_examBtn->show();
        m_examBtn->setText(tr("Cancel"));
    }

}

void HomePageWidget::reExamine()
{
   Q_EMIT m_examBtn->clicked();
}
