/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     luokai <luokai_cm@deepin.com>
 *
 * Maintainer: luokai <luokai_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "flowwidget.h"

#include <DPinyin>

#include <QVBoxLayout>
#include <QHeaderView>
#include <QApplication>

FlowWidget::FlowWidget(NetworkProtectionModel *model, NetworkProtectionWorker *worker, QWidget *parent)
    : DFrame(parent)
    , m_model(model)
    , m_worker(worker)
    , m_layout(new QVBoxLayout)
    , m_tableflowdrame(new DFrame)
    , m_tableflowlayout(new QVBoxLayout)
    , m_flowtableview(new QTableView)
    , m_flowitemmodel(new QStandardItemModel)
    , m_flowitem(new FlowItemDelegate(this, this))
{
    //点击表头排序
    connect(m_flowtableview->horizontalHeader(), &QHeaderView::sectionClicked, this, &FlowWidget::onGetlist);
    connect(this, &FlowWidget::notifyflowName, this, &FlowWidget::onShowDataDetailDialog);
    m_layout->setContentsMargins(0, 0, 0, 0);
    m_flowRangeStyle = FlowRangeStyle::today;
    m_tableflowdrame->setLayout(m_tableflowlayout);
    m_tableflowlayout->addWidget(m_flowtableview);
    // 设置表格布局

    m_layout->setSpacing(0);
    m_layout->addWidget(m_tableflowdrame);
    setLayout(m_layout);
    m_displayflowlist = m_worker->GetFlowList();
    SetFlowAttributes();
    DrawFlowItem();
}

void FlowWidget::onsetdata()
{
    int row = 0;
    int i = 1;
    m_displayflowlist = m_worker->GetFlowList();
    for (auto at : m_displaylist) { //UI的list
        at.status = 1;
        for (auto it : m_displayflowlist) { //读到数据的list
            if (at.name.compare(it.name) == 0) {
                m_flowitemmodel->setData((m_flowitemmodel->index(row, 8)), i); //替换序号
                m_flowitemmodel->setData((m_flowitemmodel->index(row, 2)), QVariant(it.download.toDouble()));
                m_flowitemmodel->setData((m_flowitemmodel->index(row, 3)), QVariant(it.update.toDouble()));
                m_flowitemmodel->setData((m_flowitemmodel->index(row, 4)), QVariant(it.total.toDouble()));
                if (KB_SECTION(it.download.toDouble())) {
                    m_flowitemmodel->setData((m_flowitemmodel->index(row, 9)), it.download + "KB"); //替换下行数据
                } else if (MB_SECTION(it.download.toDouble())) {
                    double download = it.download.toDouble() / (MB);
                    m_flowitemmodel->setData((m_flowitemmodel->index(row, 9)), QString::number(download, 'f', 2) + "MB"); //替换下行数据
                } else if (GB_SECTION(it.download.toDouble())) {
                    double download = it.download.toDouble() / (GB);
                    m_flowitemmodel->setData((m_flowitemmodel->index(row, 9)), QString::number(download, 'f', 2) + "GB"); //替换下行数据
                } else if (TB_SECTION(it.download.toDouble())) {
                    double download = it.download.toDouble() / (TB);
                    m_flowitemmodel->setData((m_flowitemmodel->index(row, 9)), QString::number(download, 'f', 2) + "TB"); //替换下行数据
                }

                if (KB_SECTION(it.update.toDouble())) {
                    m_flowitemmodel->setData((m_flowitemmodel->index(row, 10)), it.update + "KB"); //替换下行数据
                } else if (MB_SECTION(it.update.toDouble())) {
                    double upload = it.update.toDouble() / (MB);
                    m_flowitemmodel->setData((m_flowitemmodel->index(row, 10)), QString::number(upload, 'f', 2) + "MB"); //替换下行数据
                } else if (GB_SECTION(it.update.toDouble())) {
                    double upload = it.update.toDouble() / (GB);
                    m_flowitemmodel->setData((m_flowitemmodel->index(row, 10)), QString::number(upload, 'f', 2) + "GB"); //替换下行数据
                } else if (TB_SECTION(it.update.toDouble())) {
                    double upload = it.update.toDouble() / (TB);
                    m_flowitemmodel->setData((m_flowitemmodel->index(row, 10)), QString::number(upload, 'f', 2) + "TB"); //替换下行数据
                }

                if (KB_SECTION(it.total.toDouble())) {
                    m_flowitemmodel->setData((m_flowitemmodel->index(row, 11)), it.total + "KB"); //替换下行数据
                } else if (MB_SECTION(it.total.toDouble())) {
                    double total = it.total.toDouble() / (MB);
                    m_flowitemmodel->setData((m_flowitemmodel->index(row, 11)), QString::number(total, 'f', 2) + "MB"); //替换下行数据
                } else if (GB_SECTION(it.total.toDouble())) {
                    double total = it.total.toDouble() / (GB);
                    m_flowitemmodel->setData((m_flowitemmodel->index(row, 11)), QString::number(total, 'f', 2) + "GB"); //替换下行数据
                } else if (TB_SECTION(it.total.toDouble())) {
                    double total = it.total.toDouble() / (TB);
                    m_flowitemmodel->setData((m_flowitemmodel->index(row, 11)), QString::number(total, 'f', 2) + "TB"); //替换下行数据
                }
                i++;
                at.status = 0;
            }
        }
        if (at.status == 1)
            m_flowtableview->setRowHidden(row, 1);
        else {
            m_flowtableview->setRowHidden(row, 0);
        }
        row++;
    }
}

void FlowWidget::onGetlist(int logicalIndex)
{
    int row = m_flowitemmodel->rowCount();
    DefenderDisplay list;
    {
        DEFENDER_MUTEX_LOCKER(&SMutex::getInstance().m_mutex);
        for (int i = 0; i < row; i++) {
            list.name = m_flowitemmodel->data(m_flowitemmodel->index(i, 12), Qt::DisplayRole).toString();
            m_displaylist.replace(i, list);
        }
    }

    if (logicalIndex == 0)
        return;

    {
        DEFENDER_MUTEX_LOCKER(&SMutex::getInstance().m_mutex);
        onsetdata();
    }
}

void FlowWidget::onUpdateLauncherList()
{
    DEFENDER_MUTEX_LOCKER(&SMutex::getInstance().m_mutex);
    qDebug() << "FlowWidget onUpdateLauncherList";
    int nRow = 0;
    m_flowRangeStyle = FlowRangeStyle::today;
    m_displaylist.clear();
    QString downloadflow;
    QString updateflow;
    QString sName;
    QString sIcon;
    QString sId;
    QString sPath;
    QString appname;
    //遍历所有应用
    for (const auto &it : m_worker->GetFlowapp()) {
        sName = it.Name;
        sIcon = it.Icon;
        sId = it.ID;
        sPath = it.Path;
        m_showflow.name = sName;
        m_showflow.icon = sIcon;
        m_showflow.path = sPath;
        m_showflow.download = "0KB";
        m_showflow.update = "0KB";
        m_showflow.total = "0KB";
        m_showflow.status = 1;
        //遍历流量list
        for (const auto &at : m_displayflowlist) {
            appname = at.pidname;
            if (sId.contains(appname) && !appname.isEmpty()) { //名称包含
                m_showflow.download = at.download;
                m_showflow.update = at.update;
                m_showflow.total = at.total;
                m_showflow.pidname = at.pidname;
                m_showflow.status = 0;
                SetFLow(&m_showflow);
            }
        }
        QString name = DTK_CORE_NAMESPACE::Chinese2Pinyin(it.Name);
        m_flowitemmodel->setItem(nRow, 0, new QStandardItem());
        m_flowitemmodel->setItem(nRow, 1, new QStandardItem(name));
        m_flowitemmodel->setItem(nRow, 2, new QStandardItem(m_showflow.download));
        m_flowitemmodel->setItem(nRow, 3, new QStandardItem(m_showflow.update));
        m_flowitemmodel->setItem(nRow, 4, new QStandardItem(m_showflow.total));
        m_flowitemmodel->setItem(nRow, 5, new QStandardItem());
        m_flowitemmodel->setItem(nRow, 6, new QStandardItem(m_showflow.icon));
        m_flowitemmodel->setItem(nRow, 7, new QStandardItem(m_showflow.path));
        m_flowitemmodel->setItem(nRow, 12, new QStandardItem(m_showflow.name));

        //详情按钮
        DCommandLinkButton *iconlabel = new DCommandLinkButton(tr("Details"));
        QWidget *widget = new QWidget;
        QVBoxLayout *buttonlayout = new QVBoxLayout;
        buttonlayout->addWidget(iconlabel);
        widget->setLayout(buttonlayout);
        connect(iconlabel, &DCommandLinkButton::clicked, this, [ = ] {
            Q_EMIT notifyflowName(sName);
        });
        m_flowtableview->setIndexWidget(m_flowitemmodel->index(nRow, 5), widget);
        m_displaylist << m_showflow;
        nRow++;
    }
    m_flowtableview->setColumnHidden(6, true);
    m_flowtableview->setColumnHidden(7, true);
    m_flowtableview->setColumnHidden(8, true);
    m_flowtableview->setColumnHidden(9, true);
    m_flowtableview->setColumnHidden(10, true);
    m_flowtableview->setColumnHidden(11, true);
    m_flowtableview->setColumnHidden(12, true);

    m_flowtableview->setColumnWidth(0, 65);
    m_flowtableview->setColumnWidth(1, 200);
    m_flowtableview->setColumnWidth(2, 125);
    m_flowtableview->setColumnWidth(3, 125);
    m_flowtableview->setColumnWidth(4, 125);
    m_flowtableview->setColumnWidth(5, 70);
    m_flowtableview->horizontalHeader()->setHighlightSections(false);
    m_flowtableview->sortByColumn(0, Qt::AscendingOrder);
    onGetlist(0);
}

void FlowWidget::onShowDataDetailDialog(QString sName)
{
    for (auto at : m_displayflowlist) {
        if (sName == at.name) {
            AppDataDetailDialog *appDataDetailDialog = new AppDataDetailDialog(sName, at.pidname, at.icon,
                                                                               m_model, m_worker);
            appDataDetailDialog->show();
        }
    }
}

//设置网速view属性
void FlowWidget::SetFlowAttributes()
{
    m_flowtableview->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_flowtableview->setSelectionMode(QAbstractItemView::NoSelection);
    m_flowtableview->setEditTriggers(QAbstractItemView::NoEditTriggers);
    m_flowtableview->setFrameShape(QTableView::NoFrame);
    m_flowtableview->horizontalHeader()->setStretchLastSection(false);
    m_flowtableview->verticalHeader()->setHidden(true);
    m_flowtableview->setShowGrid(false);
    m_flowtableview->setSortingEnabled(true);
    m_flowtableview->verticalHeader()->setDefaultSectionSize(48);
    // 连接表格与模型
    m_flowtableview->setModel(m_flowitemmodel);
    //设置代理
    m_flowtableview->setItemDelegate(m_flowitem);
    // 初始化表格设置
    // 设置表头
    QStandardItem *standaritem0 = new QStandardItem(tr("No."));
    standaritem0->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QStandardItem *standaritem1 = new QStandardItem(tr("Name"));
    standaritem1->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QStandardItem *standaritem2 = new QStandardItem(tr("Downloaded"));
    standaritem2->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QStandardItem *standaritem3 = new QStandardItem(tr("Uploaded"));
    standaritem3->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QStandardItem *standaritem4 = new QStandardItem(tr("Data Used"));
    standaritem4->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QStandardItem *standaritem5 = new QStandardItem(tr("Details"));
    standaritem5->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);

    m_flowitemmodel->clear();
    m_flowitemmodel->setHorizontalHeaderItem(0, standaritem0);
    m_flowitemmodel->setHorizontalHeaderItem(1, standaritem1);
    m_flowitemmodel->setHorizontalHeaderItem(2, standaritem2);
    m_flowitemmodel->setHorizontalHeaderItem(3, standaritem3);
    m_flowitemmodel->setHorizontalHeaderItem(4, standaritem4);
    m_flowitemmodel->setHorizontalHeaderItem(5, standaritem5);
    m_flowitemmodel->setHorizontalHeaderItem(6, new QStandardItem(QString("Icon")));
    m_flowitemmodel->setHorizontalHeaderItem(7, new QStandardItem(QString("Path")));
    m_flowitemmodel->setHorizontalHeaderItem(8, new QStandardItem(QString("RealNO")));
    m_flowitemmodel->setHorizontalHeaderItem(9, new QStandardItem(QString("DownloadValue")));
    m_flowitemmodel->setHorizontalHeaderItem(10, new QStandardItem(QString("UploadValue")));
    m_flowitemmodel->setHorizontalHeaderItem(11, new QStandardItem(QString("TotalValue")));
    m_flowitemmodel->setHorizontalHeaderItem(12, new QStandardItem(QString("Nametopinyin")));

    m_flowtableview->setColumnHidden(6, true);
    m_flowtableview->setColumnHidden(7, true);
    m_flowtableview->setColumnHidden(8, true);
    m_flowtableview->setColumnHidden(9, true);
    m_flowtableview->setColumnHidden(10, true);
    m_flowtableview->setColumnHidden(11, true);
    m_flowtableview->setColumnHidden(12, true);

    m_flowtableview->setColumnWidth(0, 65);
    m_flowtableview->setColumnWidth(1, 200);
    m_flowtableview->setColumnWidth(2, 125);
    m_flowtableview->setColumnWidth(3, 125);
    m_flowtableview->setColumnWidth(4, 125);
    m_flowtableview->setColumnWidth(5, 70);
}

void FlowWidget::DrawFlowItem()
{
    DEFENDER_MUTEX_LOCKER(&SMutex::getInstance().m_mutex);
    int nRow = 0;
    m_flowRangeStyle = FlowRangeStyle::today;
    QString downloadflow;
    QString updateflow;
    QString sName;
    QString sIcon;
    QString sId;
    QString sPath;
    QString appname;
    //遍历所有应用
    for (const auto &it : m_worker->GetFlowapp()) {
        sName = it.Name;
        sIcon = it.Icon;
        sId = it.ID;
        sPath = it.Path;
        m_showflow.name = sName;
        m_showflow.icon = sIcon;
        m_showflow.path = sPath;
        m_showflow.download = "0KB";
        m_showflow.update = "0KB";
        m_showflow.total = "0KB";
        m_showflow.status = 1;
        QString name = DTK_CORE_NAMESPACE::Chinese2Pinyin(it.Name);
        m_flowitemmodel->setItem(nRow, 0, new QStandardItem());
        m_flowitemmodel->setItem(nRow, 1, new QStandardItem(name));
        m_flowitemmodel->setItem(nRow, 2, new QStandardItem(m_showflow.download));
        m_flowitemmodel->setItem(nRow, 3, new QStandardItem(m_showflow.update));
        m_flowitemmodel->setItem(nRow, 4, new QStandardItem(m_showflow.total));
        m_flowitemmodel->setItem(nRow, 5, new QStandardItem());
        m_flowitemmodel->setItem(nRow, 6, new QStandardItem(m_showflow.icon));
        m_flowitemmodel->setItem(nRow, 7, new QStandardItem(m_showflow.path));
        m_flowitemmodel->setItem(nRow, 12, new QStandardItem(m_showflow.name));

        //详情按钮
        DCommandLinkButton *iconlabel = new DCommandLinkButton(tr("Details"));
        QWidget *widget = new QWidget;
        QVBoxLayout *buttonlayout = new QVBoxLayout;
        buttonlayout->addWidget(iconlabel);
        widget->setLayout(buttonlayout);
        connect(iconlabel, &DCommandLinkButton::clicked, this, [ = ] {
            Q_EMIT notifyflowName(sName);
        });
        m_flowtableview->setIndexWidget(m_flowitemmodel->index(nRow, 5), widget);
        m_displaylist << m_showflow;
        nRow++;
    }
    m_flowtableview->horizontalHeader()->setHighlightSections(false);
    m_flowtableview->sortByColumn(0, Qt::AscendingOrder);
    onGetlist(0);
}

void FlowWidget::SetFLow(DefenderDisplay *showflow)
{
    if (KB_SECTION(showflow->update.toDouble())) {
        showflow->update = QString::number(showflow->update.toDouble(), 'f', 2) + "KB";
    } else if (MB_SECTION(showflow->update.toDouble())) {//MB
        showflow->update = QString::number(showflow->update.toDouble() / (MB), 'f', 2) + "MB";
    } else if (GB_SECTION(showflow->update.toDouble())) { //GB
        showflow->update = QString::number(showflow->update.toDouble() / (GB), 'f', 2) + "GB";
    } else if (TB_SECTION(showflow->update.toDouble())) { //TB
        showflow->update = QString::number(showflow->update.toDouble() / (TB), 'f', 2) + "TB";
    }

    if (KB_SECTION(showflow->download.toDouble())) {//KB
        showflow->download = QString::number(showflow->download.toDouble(), 'f', 2) + "KB";
    } else if (MB_SECTION(showflow->download.toDouble())) {//MB
        showflow->download = QString::number(showflow->download.toDouble() / (MB), 'f', 2) + "MB";
    } else if (GB_SECTION(showflow->download.toDouble())) { //GB
        showflow->download = QString::number(showflow->download.toDouble() / (GB), 'f', 2) + "GB";
    } else if (TB_SECTION(showflow->download.toDouble())) { //TB
        showflow->download = QString::number(showflow->download.toDouble() / (TB), 'f', 2) + "TB";
    }

    if (KB_SECTION(showflow->total.toDouble())) {//KB
        showflow->total = QString::number(showflow->total.toDouble(), 'f', 2) + "KB";
    } else if (MB_SECTION(showflow->total.toDouble())) {//MB
        showflow->total = QString::number(showflow->total.toDouble() / (MB), 'f', 2) + "MB";
    } else if (GB_SECTION(showflow->total.toDouble())) {//GB
        showflow->total = QString::number(showflow->total.toDouble() / (GB), 'f', 2) + "GB";
    } else if (TB_SECTION(showflow->total.toDouble())) {//TB
        showflow->total = QString::number(showflow->total.toDouble() / (TB), 'f', 2) + "TB";
    }
}

// 网速代理函数
FlowItemDelegate::FlowItemDelegate(FlowWidget *widget, QObject *parent)
    : QItemDelegate(parent)
{
    m_widget = widget;
}

// 网速重绘操作
void FlowItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    int idx = index.model()->data(index.model()->index(index.row(), 8), Qt::DisplayRole).toInt();
    if (idx % 2  == 0) {
        QBrush brush(QColor(0, 0, 0, 8));
        painter->fillRect(option.rect, brush);
    }
    if (index.column() == 1) {
        // 设置图片
        QRect rect = option.rect;
        int pix_x = rect.x() + 4;
        int pix_y = rect.y() + 4;
        int height = rect.height() - 4 * 2;

        QFont font1;
        font1.setFamily("SourceHanSansSC-Medium");
        font1.setPixelSize(13);//文字像素大小
        QPen pen1;
        pen1.setColor(QColor("#001a2e"));

        QFont font2;
        font2.setFamily("SourceHanSansSC-Normal");//字体
        font2.setPixelSize(12);//文字像素大小
        QPen pen2;
        pen2.setColor(QColor("#52607f"));

        // 读取该应用执行信息
        QString sAppsSources;
        QString sPath = index.model()->data(index.model()->index(index.row(), 7), Qt::DisplayRole).toString();
        QFile fileInfo(sPath);
        if (QFileInfo(sPath).isSymLink()) {
            sAppsSources = tr("Third-party");
        } else if (fileInfo.open(QIODevice::ReadOnly | QIODevice::Text)) {
            QTextStream in(&fileInfo);
            in.setCodec("UTF-8");
            QString sAllInfo = in.readAll();
            int nIndex = sAllInfo.indexOf("Exec");
            QString sOptInfo = sAllInfo.mid(nIndex, 10);
            if (sOptInfo.contains("opt"))
                sAppsSources = tr("Third-party");
            else
                sAppsSources = tr("System");
        }
        // 图标路径
        QString sIconPath = index.model()->data(index.model()->index(index.row(), 6), Qt::DisplayRole).toString();
        QFile file(sIconPath);
        if (file.open(QIODevice::ReadOnly)) {
            // 绘画图片坐标
            QPixmap pixmap = QPixmap(sIconPath);
            painter->drawPixmap(pix_x, pix_y, height, height, pixmap);
        } else {
            // 绘画图片坐标
            QIcon icon = QIcon::fromTheme(sIconPath);
            if (icon.isNull()) {
                QFile file(":/imageresource/dcc_application_executable.svg");
                if (file.open(QIODevice::ReadOnly)) {
                    // 绘画图片坐标
                    QPixmap pixmap = QPixmap(":/imageresource/dcc_application_executable.svg");
                    painter->drawPixmap(pix_x, pix_y, height, height, pixmap);
                }
                file.close();
            } else {
                QRect rectIcon(pix_x, pix_y, height, height);
                icon.paint(painter, rectIcon);
            }
        }
        file.close();

        // 绘画文字的坐标
        int textWidth = rect.width() - height - 4 * 8;
        int x = pix_x + height + 4 * 2;
        int y = pix_y + rect.height() / 4 + height / 8;
        int y1 = pix_y + rect.height() / 2 + height / 4 + 4;
        QString sName = index.model()->data(index.model()->index(index.row(), 12), Qt::DisplayRole).toString();
        QString sElidedName = painter->fontMetrics().elidedText(sName, Qt::ElideRight, textWidth, Qt::TextShowMnemonic);
        painter->setFont(font1);
        painter->drawText(x, y, sElidedName);

        // 绘画应用来源
        QString sElidedSource = painter->fontMetrics().elidedText(sAppsSources, Qt::ElideRight, textWidth, Qt::TextShowMnemonic);
        painter->setFont(font2);
        painter->setPen(pen2);
        painter->drawText(x, y1, sElidedSource);
    } else if (index.column() == 0) {
        // 设置text
        QString realno = index.model()->data(index.model()->index(index.row(), 8), Qt::DisplayRole).toString();
        QFont font1;
        font1.setPixelSize(13);//文字像素大小
        painter->setFont(font1);

        // 绘画文字的坐标
        QRect rect = option.rect;
        int x = rect.x();
        int y = rect.y() + rect.height() / 2 + 5;
        painter->drawText(x, y, realno);
    } else if (index.column() == 2) {
        // 设置text
        QString download = index.model()->data(index.model()->index(index.row(), 9), Qt::DisplayRole).toString();
        QFont font1;
        font1.setPixelSize(13);//文字像素大小
        painter->setFont(font1);
        // 绘画文字的坐标
        QRect rect = option.rect;
        int x = rect.x();
        int y = rect.y() + rect.height() / 2 + 5;
        painter->drawText(x, y, download);
    } else if (index.column() == 3) {
        // 设置text
        QString upload = index.model()->data(index.model()->index(index.row(), 10), Qt::DisplayRole).toString();
        QFont font1;
        font1.setPixelSize(13);//文字像素大小
        painter->setFont(font1);
        // 绘画文字的坐标
        QRect rect = option.rect;
        int x = rect.x();
        int y = rect.y() + rect.height() / 2 + 5;
        painter->drawText(x, y, upload);
    } else if (index.column() == 4) {
        // 设置text
        QString upload = index.model()->data(index.model()->index(index.row(), 11), Qt::DisplayRole).toString();
        QFont font1;
        font1.setPixelSize(13);//文字像素大小
        painter->setFont(font1);
        // 绘画文字的坐标
        QRect rect = option.rect;
        int x = rect.x();
        int y = rect.y() + rect.height() / 2 + 5;
        painter->drawText(x, y, upload);
    } else {

    }
}

QPixmap FlowWidget::getPixmap(QPixmap pixmap)
{
    const qreal ratio = devicePixelRatioF();
    pixmap.scaled(pixmap.size() *ratio, Qt::KeepAspectRatio, Qt::SmoothTransformation);
    pixmap.setDevicePixelRatio(ratio);

    return pixmap;
}
