﻿/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     wubowen <wubowen_cm@deepin.com>
 *
 * Maintainer: wubowen <wubowen_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "networkprotectionwidget.h"
#include "netprowidget/menuwidget.h"
#include "networkprotectionmodel.h"
#include "networkprotectionworker.h"
#include "trafficdetailwidget.h"
#include "window/utils.h"

#include <DFontSizeManager>
#include <DApplicationHelper>
#include <DListView>

#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QListView>
#include <QStandardItemModel>
#include <QPushButton>
#include <QDebug>

DWIDGET_USE_NAMESPACE
DEF_USING_ANTIVIRUS_NAMESPACE

NetworkProtectionWidget::NetworkProtectionWidget(NetworkProtectionModel *model,
                                                 NetworkProtectionWorker *worker, QWidget *parent)
    : QWidget(parent)
    , m_mainLayout(new QHBoxLayout)
    , m_layout(new QVBoxLayout)
    , m_model(model)
    , m_worker(worker)
    , m_NetControlServerInter(new NetControlServer("com.deepin.defender.netcontrol", "/com/deepin/defender/netcontrol", QDBusConnection::sessionBus(), this))
{
    installEventFilter(this);
    initUI();
}

void NetworkProtectionWidget::initUI()
{
    QLabel *title = new QLabel(tr("Firewall helps protect your network security"));
    QFont ft = title->font();
    ft.setBold(true);
    title->setFont(ft);
    DFontSizeManager::instance()->bind(title, DFontSizeManager::T5);
#if bIs_Complete
    QPushButton *resetBtn = new QPushButton(tr("Restore Defaults"));
    resetBtn->setMinimumSize(141, 36);
    connect(resetBtn, &QPushButton::clicked, [this] {
        qDebug() << " click restore defaults";
        m_worker->reset();
    });
#endif
    m_layout->setSpacing(0);
    m_layout->setContentsMargins(leftSpace, 0, 0, 0);
    m_layout->addSpacing(34);
    m_layout->addWidget(title, 0, Qt::AlignLeft);
    m_layout->addSpacing(30);
    m_layout->addLayout(initMenu());
    m_layout->addSpacing(28);
#if bIs_Complete
    m_layout->addWidget(resetBtn, 0, Qt::AlignLeft);
    m_layout->addSpacing(28);
#endif
    m_mainLayout->setMargin(0);
    m_mainLayout->setSpacing(0);
    m_mainLayout->addLayout(m_layout);
    m_mainLayout->addStretch();
    QLabel *logo = new QLabel(this);
    logo->setMinimumSize(QSize(307, 300));
    logo->setPixmap(QIcon::fromTheme("dcc_network_protection_detection_image").pixmap(QSize(307, 300)));
    logo->move(414, 135);
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addLayout(m_mainLayout);

    QWidget *bottomWidget = new QWidget;
    bottomWidget->setFixedHeight(100);
    bottomWidget->setLayout(initBottomButton());
    mainLayout->addStretch();
    mainLayout->addWidget(bottomWidget, 0, Qt::AlignBottom | Qt::AlignLeft);

    setLayout(mainLayout);
}

QVBoxLayout *NetworkProtectionWidget::initMenu()
{
    QVBoxLayout *menuLayout = new QVBoxLayout;

    MenuWidget *canFindWidget = new MenuWidget;
    DTipLabel *shareTxt = new DTipLabel(tr("If visible, your shared folders and files are viewable to computers in LAN"));
    shareTxt->setWordWrap(true);
    shareTxt->setAlignment(Qt::AlignLeft);
    canFindWidget->setVisible(false);
    shareTxt->setVisible(false);

    MenuWidget *funConn = new MenuWidget;
    DTipLabel *npTxt = new DTipLabel(tr("Allow or disable Internet access of applications and services"));
    npTxt->setWordWrap(true);
    npTxt->setAlignment(Qt::AlignLeft);
#if bIs_Complete
    MenuWidget *remoteConn = new MenuWidget;
    DTipLabel *remoteTxt = new DTipLabel(tr("Specifies whether remote access to your system through applications and services is permitted or not"));
    remoteTxt->setWordWrap(true);
    remoteTxt->setAlignment(Qt::AlignLeft);
#endif
    menuLayout->setContentsMargins(0, 0, 0, 0);
    menuLayout->setSpacing(0);

    QList<MenuWidget *> menus;
#if bIs_Complete
    menus << canFindWidget << funConn << remoteConn;
#else
    menus << canFindWidget << funConn;
#endif
    QList<DTipLabel *> titles;
#if bIs_Complete
    titles << shareTxt << npTxt << remoteTxt;
#else
    titles << shareTxt << npTxt;
#endif
    for (int i = 0; i < menus.length(); ++i) {
        QVBoxLayout *layout = new QVBoxLayout;
        layout->setSpacing(8);
        layout->addWidget(menus[i]);
        titles[i]->setMargin(10);
        layout->addWidget(titles[i], 0, Qt::AlignLeft);
        menuLayout->addLayout(layout);
        menuLayout->addSpacing(10);
    }

    connect(canFindWidget, &MenuWidget::notifyClicked, [ = ](int index) {
        qDebug() << " canFindWidget index : " << index;
        m_model->setShareOption(static_cast<NetworkProtectionModel::NetworkShareOption>(index));
    });

    connect(funConn, &MenuWidget::notifyClicked, [ = ](int index) {
        qDebug() << " funConn index : " << index;
        m_NetControlServerInter->setConnCunrrentIndex(index);
    });
#if bIs_Complete
    connect(remoteConn, &MenuWidget::notifyClicked, [ = ](int index) {
        qDebug() << " remoteConn index : " << index;
        m_model->setRemoteAccessedOption(static_cast<NetworkProtectionModel::NetworkAccessOption>(index));
    });
#endif

    QStringList list;

    canFindWidget->setTitle(tr("Network Sharing"));
    list.clear();
    list << tr("Hide") << tr("Visible");
    canFindWidget->setMenuData(list);

    funConn->setTitle(tr("Internet Access"));
    list.clear();
    list << tr("Ask me every time") << tr("Disable always") << tr("Allow always");
    funConn->setMenuData(list);
#if bIs_Complete
    remoteConn->setTitle(tr("Remote Connection"));
    list.clear();
    list << tr("Ask me every time") << tr("Disable always") << tr("Allow always");
    remoteConn->setMenuData(list);
#endif

    int shareOpt = m_model->getShareOption();
    int networkAccessOpt = m_NetControlServerInter->getConnCunrrentIndex();
#if bIs_Complete
    int remoteAccessedOpt = m_model->getRemoteAccessedOption(); 
#endif
    canFindWidget->setMenuIndex(shareOpt);
    funConn->setMenuIndex(networkAccessOpt);
#if bIs_Complete
    remoteConn->setMenuIndex(remoteAccessedOpt);
#endif
    connect(m_model, &NetworkProtectionModel::shareOptionChanged, canFindWidget, &MenuWidget::setMenuIndex);
    //connect(m_model, &NetworkProtectionModel::networkAccessOptionChanged, funConn, &MenuWidget::setMenuIndex);
#if bIs_Complete
    connect(m_model, &NetworkProtectionModel::remoteAccessedOptionChanged, remoteConn, &MenuWidget::setMenuIndex);
#endif
    //connect(m_worker, &NetworkProtectionWorker::requestShowNetworkSettings, this, &NetworkProtectionWidget::showFuncConnectNetControl);

    return menuLayout;
}

QHBoxLayout *NetworkProtectionWidget::initBottomButton()
{
    QHBoxLayout *layout = new QHBoxLayout;
    layout->setMargin(0);
    layout->setSpacing(0);

    DListView *buttonView = new DListView;

    //Initialize top page view and model
    QStandardItemModel *buttonModel = new QStandardItemModel(buttonView);
    buttonView->setModel(buttonModel);

    buttonView->setFrameShape(QFrame::Shape::NoFrame);
    buttonView->setEditTriggers(QListView::NoEditTriggers);
    buttonView->setResizeMode(QListView::Adjust);
    buttonView->setViewMode(QListView::IconMode);
    buttonView->setDragEnabled(false);
    buttonView->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    buttonView->setIconSize(QSize(38, 38));
    buttonView->setSpacing(10);
    buttonView->setItemSize(QSize(100, 80));
    buttonView->clearSelection();
    buttonView->clearFocus();
    buttonView->clearMask();
    buttonView->setSelectionMode(QAbstractItemView::NoSelection);
    buttonView->setTextElideMode(Qt::ElideNone);
    buttonView->setBackgroundType(DStyledItemDelegate::BackgroundType(DStyledItemDelegate::RoundedBackground | DStyledItemDelegate::NoNormalState));

    //退出模态页面(子页面)后，清除选中效果
    connect(buttonView, &QAbstractItemView::clicked, [ = ] {
        buttonView->clearFocus();
    });

    QStringList titles;
#if bIs_Complete
    titles << tr("Internet Control") << tr("Data Usage") << tr("Remote Connection");
#else
    titles << tr("Internet Control") << tr("Data Usage");
#endif
    QStringList icons;
#if bIs_Complete
    icons << "dcc_network_control" << "dcc_flow_detail" << "dcc_remote_access";
#else
    icons << "dcc_network_control" << "dcc_flow_detail";
#endif
    for (int i =  0; i < titles.size(); ++i) {
        QStandardItem *item = new QStandardItem;
        item->setIcon(QIcon::fromTheme(icons[i]));
        item->setText(titles[i]);
        item->setSizeHint(QSize(100, 60));
        buttonModel->appendRow(item);
    }

    layout->addWidget(buttonView);

    connect(buttonView, &QListView::clicked, this, &NetworkProtectionWidget::displayWidget);

    return layout;
}

void NetworkProtectionWidget::displayWidget(QModelIndex index)
{
    switch (static_cast<WidgetType>(index.row())) {
    case FuncConnectNetControl:
        showFuncConnectNetControl(QStringList());
        break;
    case NetworkLog:
        showNetworkLog();
        break;
    case ProfessionMode:
        showProfessionMode();
        break;
    default:
        break;
    }
}

void NetworkProtectionWidget::showFuncConnectNetControl(const QStringList &appNamePath)
{
    int sAppColor = DGuiApplicationHelper::instance()->paletteType();
    qDebug() << " APP BackGround Color : " << sAppColor;
    m_NetControlServerInter->setColor(QString::number(sAppColor));

    QString sNamePath;
    for (int i = 0; i < appNamePath.count(); ++i) {
        if(i == 0)
            sNamePath = appNamePath.at(0);
        else
            sNamePath = sNamePath + "|" + appNamePath.at(i);
    }
    // 联网管控弹框
    m_NetControlServerInter->showFuncConnectNetControlInter(sNamePath);
}

void NetworkProtectionWidget::showNetworkLog()
{
    widget = new TrafficDetailWidget(m_model, m_worker);
    Q_EMIT notifyShowNetFlowMonitor();
}

void NetworkProtectionWidget::showProfessionMode()
{

}

bool NetworkProtectionWidget::eventFilter(QObject *obj, QEvent *event)
{
    if (this == obj) {
        if (QEvent::PaletteChange == event->type()) {
            int sAppColor = DGuiApplicationHelper::instance()->paletteType();
            qDebug() << " APP BackGround Color : " << sAppColor;
            m_NetControlServerInter->setColor(QString::number(sAppColor));
        }
    }
    return QWidget::eventFilter(obj, event);
}
