/*
 * Copyright (C) 2017 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caimengci <caimengci_cm@deepin.com>
 *
 * Maintainer: caimengci <caimengci_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#pragma once

#include "window/namespace.h"
#include "networkprotectionmodel.h"
#include "com_deepin_dde_daemon_launcherd.h"
#include "com_deepin_defender_monitornetflow.h"
#include "com_deepin_defender_trafficdetails.h"
#include "csingleteon.h"
#include "../common/common.h"

#include <QObject>
#include <QMap>

#define Work_Debug_Flag 0

struct AppParameter {
    QString name;
    double recv;
    double send;
    int pid;
    QString device;
    QString type;
    QString pidname;//desktop
    double apptotal = 0;//单个APP总流量
    QString sName;//显示的名字
};

enum FlowRangeStyle {today, yesterday, thisMonth, lastMonth};

struct allApp {
    QString Path;
    QString Name;
    QString ID;
    QString Icon;
};
Q_DECLARE_METATYPE(allApp);

Q_DECLARE_METATYPE(AppParameter);

using Launcherd = com::deepin::dde::daemon::Launcher;
using MonitorNetFlow = com::deepin::defender::MonitorNetFlow;
using Trafficdetails = com::deepin::defender::trafficdetails;

DEF_NAMESPACE_BEGIN
class NetworkProtectionWorker : public QObject
{
    Q_OBJECT
public:
    explicit NetworkProtectionWorker(NetworkProtectionModel *model, QObject *parent = nullptr);
    void reset();
    QList<NetworkProtectionModel::AppNetworkAccessStruct> getRunningProcessOption();
    void setRunningProcessOption(QList<NetworkProtectionModel::AppNetworkAccessStruct> list);

    inline Launcherd *getlauncherd()
    {
        return m_launcherInter;
    }
    inline DefenderDisplayList GetSpeedList()
    {
        return m_speedlist;
    }
    inline DefenderDisplayList GetFlowList()
    {
        return m_flowlist;
    }
    inline QList<allApp> GetAllapp()
    {
        return m_allapp;
    }
    inline QList<allApp> GetFlowapp()
    {
        return m_flowallapp;
    }
    inline Trafficdetails *GetTrafficdetails()
    {
        return m_trafficdetails;
    }
    void setRunningProcessOption(int index, NetworkProtectionModel::AppsAccessOption option);
    static NetworkProtectionModel::AppType getAppType(const QString &path);
    static QString getAppTypeName(NetworkProtectionModel::AppType type);

    void showNetworkAccessSettings(const QStringList &appNamePath);
    //请求获取某个app流量数据
    void requestGetAppYesterdayFlowData(QString appname);
    void requestGetAppTodayFlowData(QString appname);
    void requestGetAppLastMonthFlowData(QString appname);
    void requestGetAppThisMonthFlowData(QString appname);

    struct AppIcon {
        QString appName;
        QString icon;
        QString path;
        NetworkProtectionModel::AppType type;
    };

private:
    QList<AppIcon> getAllApps();
    QDBusPendingReply<LauncherItemInfoList> getLauncherList();

Q_SIGNALS:
    void requestShowNetworkSettings(const QStringList &appName);
    void requestShowNetworkModuleSettings(const QStringList &appNamePath);
    void notifyUpdateLauncherList();
    void notifyUpdateSpeed();
    void notifyUpdateFlow();
    //向应用流量详情对话框实例发送显示的数据列表
    void sendAppFlowData(DefenderAppFlowList flowLst);
    void notifyUpdate();

private Q_SLOTS:
    void onReceiveSpeedList(DefenderDisplayList speedlist);
    void onReceoveFLowList(DefenderDisplayList flowlist);
    void onUpdateLauncherList();

private:
    NetworkProtectionModel *m_model;
    Launcherd *m_launcherInter;
    MonitorNetFlow *m_monitornetflow;
    Trafficdetails *m_trafficdetails;//流量监控dbus类
    DefenderDisplayList m_speedlist;//网速list
    DefenderDisplayList m_flowlist;//流量list
    QList<allApp> m_allapp;
    QList<allApp> m_flowallapp;
};
DEF_NAMESPACE_END
