/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     luokai <luokai_cm@deepin.com>
 *
 * Maintainer: luokai <luokai_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "speedwidget.h"

#include <DPinyin>

#include <QVBoxLayout>
#include <QHeaderView>
#include <QApplication>

SpeedWidget::SpeedWidget(NetworkProtectionModel *model, NetworkProtectionWorker *worker, QWidget *parent)
    : DFrame(parent)
    , m_model(model)
    , m_worker(worker)
    , m_layout(new QVBoxLayout)
    , m_tablespeeddrame(new DFrame)
    , m_tablespeedlayout(new QVBoxLayout)
    , m_speedtableview(new QTableView)
    , m_speeditemmodel(new QStandardItemModel)
    , m_speeditem(new SpeedItemDelegate(this, this))
    , m_NetControlServerInter(new NetControlServer("com.deepin.defender.netcontrol", "/com/deepin/defender/netcontrol", QDBusConnection::sessionBus(), this))
{
    //点击表头排序
    connect(m_speedtableview->horizontalHeader(), &QHeaderView::sectionClicked, this, &SpeedWidget::onGetlist);

    m_layout->setContentsMargins(0, 0, 0, 0);
    m_tablespeeddrame->setLayout(m_tablespeedlayout);
    m_tablespeedlayout->addWidget(m_speedtableview);
    // 设置表格布局

    m_layout->setSpacing(0);
    m_layout->addWidget(m_tablespeeddrame);
    setLayout(m_layout);
    SetSpeedAttributes();
    m_speedlist = m_worker->GetSpeedList();
    DrawSpeedItem();
}

void SpeedWidget::onsetdata()
{
    int row = 0;
    int i = 1;
    m_speedlist = m_worker->GetSpeedList();
    if (m_speedlist.size() == 0)
        QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));//设置鼠标等待
    else
        QApplication::restoreOverrideCursor();
    for (auto at : m_displaylist) { //UI的list
        at.status = 1;
        for (auto it : m_speedlist) { //读到数据的list
            if (at.name.compare(it.name) == 0) {
                m_speeditemmodel->setData((m_speeditemmodel->index(row, 8)), i); //替换序号
                m_speeditemmodel->setData((m_speeditemmodel->index(row, 1)), QVariant(it.download.toDouble())); //替换下行数据
                m_speeditemmodel->setData((m_speeditemmodel->index(row, 2)), QVariant(it.update.toDouble())); //替换下行数据
                if (MB_SECTION(it.download.toDouble())) {
                    double download = it.download.toDouble() / (MB);
                    m_speeditemmodel->setData((m_speeditemmodel->index(row, 6)), QString::number(download, 'f', 2) + "MB/s"); //替换下行数据
                } else {
                    m_speeditemmodel->setData((m_speeditemmodel->index(row, 6)), it.download + "KB/s"); //替换下行数据
                }
                if (MB_SECTION(it.update.toDouble())) {
                    double upload = it.update.toDouble() / (MB);
                    m_speeditemmodel->setData((m_speeditemmodel->index(row, 7)), QString::number(upload, 'f', 2) + "MB/s"); //替换上行数据
                } else {
                    m_speeditemmodel->setData((m_speeditemmodel->index(row, 7)), it.update + "KB/s"); //替换上行数据
                }
                at.status = 0;
                i++;
            }
        }
        if (at.status == 1)
            m_speedtableview->setRowHidden(row, 1);
        else {
            m_speedtableview->setRowHidden(row, 0);
        }
        row++;
    }
}

void SpeedWidget::onGetlist(int logicalIndex)
{
    Q_UNUSED(logicalIndex);
    qDebug() << "onGetlist";
    int row = m_speeditemmodel->rowCount();
    DefenderDisplay list;

    {
        DEFENDER_MUTEX_LOCKER(&SMutex::getInstance().m_mutex);
        for (int i = 0; i < row; i++) {
            list.name = m_speeditemmodel->data(m_speeditemmodel->index(i, 9), Qt::DisplayRole).toString();
            m_displaylist.replace(i, list);
        }
    }

    {
        DEFENDER_MUTEX_LOCKER(&SMutex::getInstance().m_mutex);
        onsetdata();
    }
}

void SpeedWidget::onUpdateLauncherList()
{
    DEFENDER_MUTEX_LOCKER(&SMutex::getInstance().m_mutex);
    qDebug() << "SpeedWidget onUpdateLauncherList";
    int nRow = 0;
    totalupload = 0;
    totaldownload = 0;
    QString sName;
    QString sIcon;
    QString sId;
    QString sPath;
    QString appname;
    //遍历所有应用
    for (const auto &it : m_worker->GetAllapp()) {
        sName = it.Name;
        sIcon = it.Icon;
        sId = it.ID;
        sPath = it.Path;
        m_showspeed.name = sName;
        m_showspeed.icon = sIcon;
        m_showspeed.path = sPath;
        m_showspeed.download = "0.0KB/s";
        m_showspeed.update = "0.0KB/s";
        m_showspeed.total = "0.0KB/s";
        m_showspeed.status = 1;
        QString name = DTK_CORE_NAMESPACE::Chinese2Pinyin(it.Name);
        m_speeditemmodel->setItem(nRow, 0, new QStandardItem(name));
        m_speeditemmodel->setItem(nRow, 1, new QStandardItem(m_showspeed.download));
        m_speeditemmodel->setItem(nRow, 2, new QStandardItem(m_showspeed.update));
        m_speeditemmodel->setItem(nRow, 3, new QStandardItem());
        m_speeditemmodel->setItem(nRow, 4, new QStandardItem(m_showspeed.icon));
        m_speeditemmodel->setItem(nRow, 5, new QStandardItem(m_showspeed.path));
        m_speeditemmodel->setItem(nRow, 9, new QStandardItem(m_showspeed.name));

        buttonlabel = new ButtonLabel;
        if (sName != "https") {
            connect(buttonlabel, &ButtonLabel::clicked, this, [ = ] {
                QStringList list;
                list << sName;

                QString sNamePath;
                for (int i = 0; i < list.count(); ++i)
                {
                    if (i == 0)
                        sNamePath = list.at(0);
                    else
                        sNamePath = sNamePath + "|" + list.at(i);
                }
                m_NetControlServerInter->showFuncConnectNetControlInter(sNamePath);
            });
        }
        m_speedtableview->setIndexWidget(m_speeditemmodel->index(nRow, 3), buttonlabel);
        m_displaylist << m_showspeed;
        nRow++;
    }
    m_speedtableview->setColumnHidden(4, true);
    m_speedtableview->setColumnHidden(5, true);
    m_speedtableview->setColumnHidden(6, true);
    m_speedtableview->setColumnHidden(7, true);
    m_speedtableview->setColumnHidden(8, true);
    m_speedtableview->setColumnHidden(9, true);

    m_speedtableview->setColumnWidth(0, 220);
    m_speedtableview->setColumnWidth(1, 215);
    m_speedtableview->setColumnWidth(2, 215);
    m_speedtableview->setColumnWidth(3, 64);
    m_speedtableview->horizontalHeader()->setHighlightSections(false);
    int row = m_speeditemmodel->rowCount();
    DefenderDisplay list;
    for (int i = 0; i < row; i++) {
        list.name = m_speeditemmodel->data(m_speeditemmodel->index(i, 9), Qt::DisplayRole).toString();
        m_displaylist.replace(i, list);
    }
    m_speedtableview->sortByColumn(0, Qt::AscendingOrder);
    onGetlist(0);
}

//设置网速view属性
void SpeedWidget::SetSpeedAttributes()
{
    m_speedtableview->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_speedtableview->setSelectionMode(QAbstractItemView::NoSelection);
    m_speedtableview->setEditTriggers(QAbstractItemView::NoEditTriggers);
    m_speedtableview->setFrameShape(QTableView::NoFrame);
    m_speedtableview->horizontalHeader()->setStretchLastSection(false);
    m_speedtableview->verticalHeader()->setHidden(true);
    m_speedtableview->setShowGrid(false);
    m_speedtableview->setSortingEnabled(true);
    m_speedtableview->verticalHeader()->setDefaultSectionSize(48);
    // 连接表格与模型
    m_speedtableview->setModel(m_speeditemmodel);
    //设置代理
    m_speedtableview->setItemDelegate(m_speeditem);
    // 初始化表格设置
    // 设置表头
    QStandardItem *standaritem0 = new QStandardItem(tr("Name"));
    standaritem0->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QStandardItem *standaritem1 = new QStandardItem(tr("Download"));
    standaritem1->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QStandardItem *standaritem2 = new QStandardItem(tr("Upload"));
    standaritem2->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QStandardItem *standaritem3 = new QStandardItem(tr("Settings"));
    standaritem3->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    m_speeditemmodel->setHorizontalHeaderItem(0, standaritem0);
    m_speeditemmodel->setHorizontalHeaderItem(1, standaritem1);
    m_speeditemmodel->setHorizontalHeaderItem(2, standaritem2);
    m_speeditemmodel->setHorizontalHeaderItem(3, standaritem3);
    m_speeditemmodel->setHorizontalHeaderItem(4, new QStandardItem(QString("Icon")));
    m_speeditemmodel->setHorizontalHeaderItem(5, new QStandardItem(QString("Path")));
    m_speeditemmodel->setHorizontalHeaderItem(6, new QStandardItem(QString("DownloadValue")));
    m_speeditemmodel->setHorizontalHeaderItem(7, new QStandardItem(QString("UploadValue")));
    m_speeditemmodel->setHorizontalHeaderItem(8, new QStandardItem(QString("Nunber")));
    m_speeditemmodel->setHorizontalHeaderItem(9, new QStandardItem(QString("Nametopinyin")));

    m_speedtableview->setColumnHidden(4, true);
    m_speedtableview->setColumnHidden(5, true);
    m_speedtableview->setColumnHidden(6, true);
    m_speedtableview->setColumnHidden(7, true);
    m_speedtableview->setColumnHidden(8, true);
    m_speedtableview->setColumnHidden(9, true);

    m_speedtableview->setColumnWidth(0, 220);
    m_speedtableview->setColumnWidth(1, 215);
    m_speedtableview->setColumnWidth(2, 215);
    m_speedtableview->setColumnWidth(3, 64);
    m_speedtableview->horizontalHeader()->setHighlightSections(false);
}

void SpeedWidget::DrawSpeedItem()
{
    DEFENDER_MUTEX_LOCKER(&SMutex::getInstance().m_mutex);
    int nRow = 0;
    totalupload = 0;
    totaldownload = 0;
    QString sName;
    QString sIcon;
    QString sId;
    QString sPath;
    QString appname;
    //遍历所有应用
    for (const auto &it : m_worker->GetAllapp()) {
        sName = it.Name;
        sIcon = it.Icon;
        sId = it.ID;
        sPath = it.Path;
        m_showspeed.name = sName;
        m_showspeed.icon = sIcon;
        m_showspeed.path = sPath;
        m_showspeed.download = "0.0KB/s";
        m_showspeed.update = "0.0KB/s";
        m_showspeed.total = "0.0KB/s";
        m_showspeed.status = 1;
        //遍历网速list
        for (auto &at : m_sppedlist) {
            appname = at.pidname;
            if (sId.contains(appname) && !appname.isEmpty()) { //名称包含重新赋值
                m_showspeed.download = QString::number(at.recv, 'f', 2);
                m_showspeed.update = QString::number(at.send, 'f', 2);
                m_showspeed.total = QString::number(at.apptotal, 'f', 2);
                totalupload += at.send;
                totaldownload += at.recv;
                m_showspeed.status = 0;
                at.send = 0;
                at.recv = 0;
                SetSpeed(&m_showspeed);
            }
        }
        QString name = DTK_CORE_NAMESPACE::Chinese2Pinyin(it.Name);
        m_speeditemmodel->setItem(nRow, 0, new QStandardItem(name));
        m_speeditemmodel->setItem(nRow, 1, new QStandardItem(m_showspeed.download));
        m_speeditemmodel->setItem(nRow, 2, new QStandardItem(m_showspeed.update));
        m_speeditemmodel->setItem(nRow, 3, new QStandardItem());
        m_speeditemmodel->setItem(nRow, 4, new QStandardItem(m_showspeed.icon));
        m_speeditemmodel->setItem(nRow, 5, new QStandardItem(m_showspeed.path));
        m_speeditemmodel->setItem(nRow, 9, new QStandardItem(m_showspeed.name));

        buttonlabel = new ButtonLabel;
        if (sName != "https") {
            connect(buttonlabel, &ButtonLabel::clicked, this, [ = ] {
                QStringList list;
                list << sName;

                QString sNamePath;
                for (int i = 0; i < list.count(); ++i)
                {
                    if (i == 0)
                        sNamePath = list.at(0);
                    else
                        sNamePath = sNamePath + "|" + list.at(i);
                }
                m_NetControlServerInter->showFuncConnectNetControlInter(sNamePath);
            });
        }
        m_speedtableview->setIndexWidget(m_speeditemmodel->index(nRow, 3), buttonlabel);
        m_displaylist << m_showspeed;
        nRow++;
    }
    m_speedtableview->sortByColumn(0, Qt::AscendingOrder);
    onGetlist(0);
}

void SpeedWidget::SetSpeed(DefenderDisplay *showspeed)
{
    if (KB_SECTION(showspeed->update.toDouble()))
        showspeed->update = QString::number(showspeed->update.toDouble(), 'f', 2) + "KB/s";
    else
        showspeed->update = QString::number(showspeed->update.toDouble() / (MB), 'f', 2) + "MB/s";

    if (KB_SECTION(showspeed->download.toDouble()))
        showspeed->download = QString::number(showspeed->download.toDouble(), 'f', 2) + "KB/s";
    else
        showspeed->download = QString::number(showspeed->download.toDouble() / (MB), 'f', 2) + "MB/s";
}

// 网速代理函数
SpeedItemDelegate::SpeedItemDelegate(SpeedWidget *widget, QObject *parent)
    : QItemDelegate(parent)
{
    m_widget = widget;
}

// 网速重绘操作
void SpeedItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    int idx = index.model()->data(index.model()->index(index.row(), 8), Qt::DisplayRole).toInt();
    if (idx % 2  == 0) {
        QBrush brush(QColor(0, 0, 0, 8));
        painter->fillRect(option.rect, brush);
    }
    // 第一列
    if (index.column() == 0) {
        // 设置图片
        QRect rect = option.rect;
        int pix_x = rect.x() + 4;
        int pix_y = rect.y() + 4;
        int height = rect.height() - 4 * 2;

        QFont font1;
        font1.setFamily("SourceHanSansSC-Medium");
        font1.setPixelSize(13);//文字像素大小
        QPen pen1;
        pen1.setColor(QColor("#001a2e"));

        QFont font2;
        font2.setFamily("SourceHanSansSC-Normal");//字体
        font2.setPixelSize(12);//文字像素大小
        QPen pen2;
        pen2.setColor(QColor("#52607f"));

        // 读取该应用执行信息
        QString sAppsSources;
        QString sPath = index.model()->data(index.model()->index(index.row(), 5), Qt::DisplayRole).toString();
        if (sPath != "https") {
            QFile fileInfo(sPath);
            if (QFileInfo(sPath).isSymLink()) {
                sAppsSources = tr("Third-party");
            } else if (fileInfo.open(QIODevice::ReadOnly | QIODevice::Text)) {
                QTextStream in(&fileInfo);
                in.setCodec("UTF-8");
                QString sAllInfo = in.readAll();
                int nIndex = sAllInfo.indexOf("Exec");
                QString sOptInfo = sAllInfo.mid(nIndex, 10);
                if (sOptInfo.contains("opt"))
                    sAppsSources = tr("Third-party");
                else
                    sAppsSources = tr("System");
            }
        } else {
            sAppsSources = tr("Third-party");
        }
        painter->save();
        // 图标路径
        QString sIconPath = index.model()->data(index.model()->index(index.row(), 4), Qt::DisplayRole).toString();
        QFile file(sIconPath);
        if (file.open(QIODevice::ReadOnly) && sIconPath.contains("/")) {
            // 绘画图片坐标
            QPixmap pixmap = QPixmap(sIconPath);
            painter->drawPixmap(pix_x, pix_y, height, height, pixmap);
            file.close();
        } else {
            // 绘画图片坐标
            QIcon icon = QIcon::fromTheme(sIconPath);
            if (icon.isNull()) {
                QFile file(":/imageresource/dcc_application_executable.svg");
                if (file.open(QIODevice::ReadOnly) && sIconPath.contains("/")) {
                    // 绘画图片坐标
                    QPixmap pixmap = QPixmap(":/imageresource/dcc_application_executable.svg");
                    painter->drawPixmap(pix_x, pix_y, height, height, pixmap);
                }
                file.close();
            } else {
                QRect rectIcon(pix_x, pix_y, height, height);
                icon.paint(painter, rectIcon);
            }
        }
        // 绘画文字的坐标
        int textWidth = rect.width() - height - 4 * 8;
        int x = pix_x + height + 4 * 2;
        int y = pix_y + rect.height() / 4 + height / 8;
        int y1 = pix_y + rect.height() / 2 + height / 4 + 4;
        QString sName = index.model()->data(index.model()->index(index.row(), 9), Qt::DisplayRole).toString();
        QString sElidedName = painter->fontMetrics().elidedText(sName, Qt::ElideRight, textWidth, Qt::TextShowMnemonic);
        painter->setFont(font1);
        painter->drawText(x, y, sElidedName);

        // 绘画应用来源
        QString sElidedSource = painter->fontMetrics().elidedText(sAppsSources, Qt::ElideRight, textWidth, Qt::TextShowMnemonic);
        painter->setFont(font2);
        painter->setPen(pen2);
        painter->drawText(x, y1, sElidedSource);
        painter->restore();

    } else if (index.column() == 1) {
        // 设置text
        QString download = index.model()->data(index.model()->index(index.row(), 6), Qt::DisplayRole).toString();

        // 绘画图片和文字的坐标
        QRect rect = option.rect;
        int x = rect.x();
        int y = rect.y() + rect.height() / 2 + 5;
        painter->drawText(x, y, download);
    } else if (index.column() == 2) {
        // 设置text
        QString upload = index.model()->data(index.model()->index(index.row(), 7), Qt::DisplayRole).toString();

        // 绘画文字的坐标
        QRect rect = option.rect;
        int x = rect.x();
        int y = rect.y() + rect.height() / 2 + 5;
        painter->drawText(x, y, upload);
    } else if (index.column() == 3) {
        // text
        QPixmap pixmap;
        QString sPath = index.model()->data(index.model()->index(index.row(), 5), Qt::DisplayRole).toString();
        if (sPath == "https") {
            const QPixmap &star = QPixmap(":/imageresource/dcc_control_button_disabled");
            pixmap = m_widget->getPixmap(star);
        } else {
            const QPixmap &star = QPixmap(":/imageresource/dcc_control_button");
            pixmap = m_widget->getPixmap(star);
        }
        // 绘画和文字的坐标
        int height = pixmap.height();
        QRect rect = option.rect;
        int x = rect.x();
        int y = rect.y() + rect.height() / 2 - height / 2;
        painter->drawPixmap(x, y, pixmap);
    } else {

    }
}

QPixmap SpeedWidget::getPixmap(QPixmap pixmap)
{
    const qreal ratio = devicePixelRatioF();
    pixmap.scaled(pixmap.size() *ratio, Qt::KeepAspectRatio, Qt::SmoothTransformation);
    pixmap.setDevicePixelRatio(ratio);

    return pixmap;
}

//管控qlabel
ButtonLabel::ButtonLabel()
{

}
//实现qlabel 鼠标点击函数
void ButtonLabel::mouseReleaseEvent(QMouseEvent *ev)
{
    Q_UNUSED(ev)
    emit clicked();
}
