/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     luokai <luokai_cm@deepin.com>
 *
 * Maintainer: luokai <luokai_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#pragma once

#include "window/interface/moduleinterface.h"
#include "window/interface/frameproxyinterface.h"
#include "networkprotectionworker.h"
#include "com_deepin_defender_trafficdetails.h"
#include "com_deepin_defender_netcontrol.h"

#include <DButtonBox>
#include <DTipLabel>
#include <DCommandLinkButton>
#include <DBackgroundGroup>
#include <DFrame>

#include <QItemDelegate>
#include <QStandardItemModel>
#include <QTableView>
#include <QVBoxLayout>

DWIDGET_USE_NAMESPACE

#define SpeedWidget_Debug 1

using NetControlServer = com::deepin::defender::netcontrol;
class SpeedWidget;
class SpeedItemDelegate: public QItemDelegate
{
    Q_OBJECT
public:
    SpeedItemDelegate(SpeedWidget *widget, QObject *parent = nullptr);
    virtual ~ SpeedItemDelegate() {}

    void paint(QPainter *painter,
               const QStyleOptionViewItem &option,
               const QModelIndex &index) const;

private:
    QPixmap m_pixmap;
    SpeedWidget *m_widget;
};

class ButtonLabel: public QLabel
{
    Q_OBJECT
public:
    ButtonLabel();

protected:
    virtual void mouseReleaseEvent(QMouseEvent *ev);
signals:
    void clicked(void);
};

class NetworkProtectionModel;
class NetworkProtectionWorker;
class SpeedWidget : public DFrame
{
    Q_OBJECT
public:
    explicit SpeedWidget(NetworkProtectionModel *model = nullptr,
                         NetworkProtectionWorker *worker = nullptr, QWidget *parent = nullptr);
    QPixmap getPixmap(QPixmap);
    inline double gettotalupload()
    {
        return totalupload;
    }
    inline double gettotaldownload()
    {
        return totaldownload;
    }

public Q_SLOTS:
    //更新界面
    void onsetdata();
    void onGetlist(int logicalIndex);
    void onUpdateLauncherList();

private:
    void SetSpeedAttributes();
    void DrawSpeedItem();
    void SetSpeed(DefenderDisplay *showspeed);

private:
    NetworkProtectionModel *m_model;
    NetworkProtectionWorker *m_worker;
    QDBusPendingReply<LauncherItemInfoList> m_reply;
    QVBoxLayout *m_layout;
    DFrame *m_tablespeeddrame;//圆角widget
    QVBoxLayout *m_tablespeedlayout;
    QTableView *m_speedtableview;
    QStandardItemModel *m_speeditemmodel;
    SpeedItemDelegate *m_speeditem;
    QList<AppParameter> m_sppedlist;
    DefenderDisplayList m_displaylist;
    DefenderDisplay m_showspeed;
    ButtonLabel *buttonlabel;
    double totalupload;
    double totaldownload;
    DefenderDisplayList m_speedlist;

    NetControlServer  *m_NetControlServerInter = nullptr; // 联网管控接口
};
DEF_NAMESPACE_END
DEF_NETPROTECTION_NAMESPACE_END
