/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     luokai <luokai_cm@deepin.com>
 *
 * Maintainer: luokai <luokai_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "trafficdetailwidget.h"
#include "netprowidget/menuwidget.h"
#include "networkprotectionmodel.h"
#include "networkprotectionworker.h"
#include "netprowidget/flowmonitorcontrol.h"
#include "speedwidget.h"
#include "window/modules/networkprotection/appdatadetaildialog.h"

#include <DFontSizeManager>
#include <DApplicationHelper>
#include <DListView>
#include <DSimpleListView>
#include <DTipLabel>

#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QPushButton>
#include <QDebug>
#include <QComboBox>
#include <QTableView>
#include <QHeaderView>
#include <functional>
#include <QToolButton>
#include <QMenu>

DWIDGET_USE_NAMESPACE
DEF_USING_ANTIVIRUS_NAMESPACE

TrafficDetailWidget::TrafficDetailWidget(NetworkProtectionModel *model, NetworkProtectionWorker *worker, QWidget *parent)
    : DFrame(parent)
    , m_model(model)
    , m_worker(worker)
    , m_detailInfo(new DTipLabel)
    , m_layout(new QVBoxLayout)
    , m_totalLayout(new QHBoxLayout)
    , m_widget(new QWidget)
    , m_vlayout(new QVBoxLayout)
    , m_timebox(new QComboBox)
    , m_topSwitchWidgetBtn(new Dtk::Widget::DButtonBox)
    , m_tablelayout(new QVBoxLayout)
    , m_timeRangeSelect(new DPushButton(tr("Today")))
{
    initUI();
    qRegisterMetaType<QList<allApp>>("QList<allApp>");
    qRegisterMetaType<QList<AppParameter>>("QList<AppParameter");
    connect(m_worker, &NetworkProtectionWorker::notifyUpdateSpeed, m_speedwidget, &SpeedWidget::onsetdata);
    connect(m_worker, &NetworkProtectionWorker::notifyUpdateSpeed, this, &TrafficDetailWidget::onsetdata);
    connect(m_worker, &NetworkProtectionWorker::notifyUpdateFlow, this, &TrafficDetailWidget::onsetdata);
    connect(m_worker, &NetworkProtectionWorker::notifyUpdateFlow, m_flowwidget, &FlowWidget::onsetdata);
    connect(m_worker, &NetworkProtectionWorker::notifyUpdate, m_speedwidget, &SpeedWidget::onUpdateLauncherList);
    connect(m_worker, &NetworkProtectionWorker::notifyUpdate, m_flowwidget, &FlowWidget::onUpdateLauncherList);
}

//初始化ui界面
void TrafficDetailWidget::initUI()
{
    //将　widget　加入到一个新建的　DBackgroundGroup中
    QVBoxLayout *bglayout = new QVBoxLayout;
    bglayout->setContentsMargins(0, 0, 0, 0);
    m_bgGroup = new DBackgroundGroup(bglayout, this);
    m_bgGroup->setBackgroundRole(QPalette::Window);
    bglayout->addWidget(m_widget);

    //将 m_bgGroup 沉底
    m_bgGroup->lower();

    //设置m_bgGroup 的大小
    m_bgGroup->setFixedSize(size());

    // 标题栏布局
    QVBoxLayout *tileLayout = new QVBoxLayout;
    tileLayout->setContentsMargins(0, 0, 0, 0);
    QLabel *title = new QLabel(tr("Data Usage"));
    DFontSizeManager::instance()->bind(title, DFontSizeManager::T5);
    tileLayout->addWidget(title, 0, Qt::AlignTop);

    //按钮item
    DButtonBoxButton *btnUpdate = new DButtonBoxButton(tr("Apps and Services"));
    DButtonBoxButton *btnSetting = new DButtonBoxButton(tr("Rankings"));
    m_btnlist.append(btnUpdate);
    m_btnlist.append(btnSetting);
    m_topSwitchWidgetBtn->setButtonList(m_btnlist, true);
    m_topSwitchWidgetBtn->setMaximumSize(240, 36);
    m_btnlist.first()->setChecked(true);
    m_topSwitchWidgetBtn->setId(btnUpdate, 0);
    m_topSwitchWidgetBtn->setId(btnSetting, 1);
    m_topSwitchWidgetBtn->setMinimumSize(240, 36);
    m_btnlist.at(0)->setChecked(true);
    connect(m_topSwitchWidgetBtn, &DButtonBox::buttonClicked, [this](QAbstractButton * value) {
        settype(static_cast<TabelType>(m_topSwitchWidgetBtn->id(value)));
    });

    m_layout->setMargin(0);
    m_layout->addLayout(tileLayout);
    m_layout->setAlignment(Qt::AlignTop);
    m_layout->addWidget(m_topSwitchWidgetBtn, 0, Qt::AlignHCenter);
    m_layout->addLayout(m_totalLayout);

    m_timeRangeSelect->setFlat(true);
    m_timeRangeSelect->setMinimumWidth(100);

    QMenu *timeRangeMenu = new QMenu;
    timeRangeMenu->setMinimumWidth(100);
    m_timeRangeSelect->setMenu(timeRangeMenu);
    QAction *todayAction = new QAction(tr("Today"));
    todayAction->setObjectName("today");
    todayAction->setCheckable(true);
    timeRangeMenu->addAction(todayAction);
    QAction *yesterdayAction = new QAction(tr("Yesterday"));
    yesterdayAction->setObjectName("yesterday");
    yesterdayAction->setCheckable(true);
    timeRangeMenu->addAction(yesterdayAction);
    QAction *thisMonthAction = new QAction(tr("This month"));
    thisMonthAction->setObjectName("thisMonth");
    thisMonthAction->setCheckable(true);
    timeRangeMenu->addAction(thisMonthAction);
    QAction *lastMonthAction = new QAction(tr("Last month"));
    lastMonthAction->setObjectName("lastMonth");
    lastMonthAction->setCheckable(true);
    timeRangeMenu->addAction(lastMonthAction);
    m_totalLayout->addWidget(m_detailInfo, 0, Qt::AlignLeft);
    m_totalLayout->addWidget(m_timeRangeSelect, 0, Qt::AlignRight);
    Q_EMIT m_worker->GetTrafficdetails()->notifyGetTodayFlowData();
    connect(timeRangeMenu, &QMenu::triggered, this, [this](QAction * action) {
        for (QAction *ttaction : m_timeRangeSelect->menu()->actions()) {
            ttaction->setChecked(false);
        }
        m_timeRangeSelect->setText(action->text());
        action->setChecked(true);
        if ("today" == action->objectName()) {
            Q_EMIT m_worker->GetTrafficdetails()->notifyGetTodayFlowData();
        } else if ("yesterday" == action->objectName()) {
            Q_EMIT m_worker->GetTrafficdetails()->notifyGetYesterdayFlowData();
        } else if ("thisMonth" == action->objectName()) {
            Q_EMIT m_worker->GetTrafficdetails()->notifyGetThisMonthFlowData();
        } else if ("lastMonth" == action->objectName()) {
            Q_EMIT m_worker->GetTrafficdetails()->notifyGetLastMonthFlowData();
        }
    });

    m_speedwidget = new SpeedWidget(m_model, m_worker);
    m_speedwidget->setContentsMargins(0, 0, 0, 0);
    m_flowwidget = new FlowWidget(m_model, m_worker);
    m_speedwidget->setContentsMargins(0, 0, 0, 0);
    m_tablelayout->addWidget(m_speedwidget);
    m_tablelayout->addWidget(m_flowwidget);
    m_layout->addLayout(m_tablelayout);
    setLayout(m_layout);
    showAppsServices();
}

//根据按钮显示对应界面
void TrafficDetailWidget::settype(TabelType type)
{
    qDebug() << "----- click index : " << type;
    m_tabeltype = type;
    switch (type) {
    case AppsServices:
        showAppsServices();
        break;
    case Trafficranking:
        showTrafficranking();
        break;
    }
}
//网速界面
void TrafficDetailWidget::showAppsServices()
{
    QString upload = "";
    QString download = "";
    double uploadspeed = 0;
    double downloadspeed = 0;
    m_displayspeedlist = m_worker->GetSpeedList();
    for (auto at : m_displayspeedlist) {
        uploadspeed += at.update.toDouble();
        downloadspeed += at.download.toDouble();
    }
    m_timeRangeSelect->hide();
    m_flowwidget->hide();
    m_speedwidget->show();
    if (KB_SECTION(uploadspeed))
        upload = tr("Total Upload") + QString("：%1KB/s").arg(QString::number(uploadspeed, 'f', 2));
    else if (MB_SECTION(uploadspeed))
        upload = tr("Total Upload") + QString("：%1MB/s").arg(QString::number(uploadspeed / (MB), 'f', 2));

    if (KB_SECTION(downloadspeed))
        download = tr("Total Download") + QString("：%1KB/s").arg(QString::number(downloadspeed, 'f', 2));
    else if (MB_SECTION(downloadspeed))
        download = tr("Total Download") + QString("：%1MB/s").arg(QString::number(downloadspeed / (MB), 'f', 2));

    m_detailInfo->setText(download + "   " + upload);
}

//流量界面
void TrafficDetailWidget::showTrafficranking()
{
    QString upload = "";
    QString download = "";
    QString total = "";
    m_displayflowlist = m_worker->GetFlowList();
    double uploadflow = 0;
    double downloadflow = 0;
    for (auto at : m_displayflowlist) {
        uploadflow += at.update.toDouble();
        downloadflow += at.download.toDouble();
    }
    double totalflow = uploadflow + downloadflow;
    m_timeRangeSelect->show();
    m_speedwidget->hide();
    m_flowwidget->show();
    if (m_displayflowlist.isEmpty()) {
        m_detailInfo->setText(tr("Downloaded") + "：0.00KB" + "   " + tr("Uploaded") + "：0.00KB"  + "   " + tr("Data Used") + "：0.00KB");
    } else {
        if (KB_SECTION(uploadflow))
            upload = tr("Uploaded") + QString("：%1KB").arg(QString::number(uploadflow, 'f', 2));
        else if (MB_SECTION(uploadflow))
            upload = tr("Uploaded") + QString("：%1MB").arg(QString::number(uploadflow / (MB), 'f', 2));
        else if (GB_SECTION(uploadflow))
            upload = tr("Uploaded") + QString("：%1GB").arg(QString::number(uploadflow / (GB), 'f', 2));
        else if (TB_SECTION(uploadflow))
            upload = tr("Uploaded") + QString("：%1TB").arg(QString::number(uploadflow / (TB), 'f', 2));

        if (KB_SECTION(downloadflow))
            download = tr("Downloaded") + QString("：%1KB").arg(QString::number(downloadflow, 'f', 2));
        else if (MB_SECTION(downloadflow))
            download = tr("Downloaded") + QString("：%1MB").arg(QString::number(downloadflow / (MB), 'f', 2));
        else if (GB_SECTION(downloadflow))
            download = tr("Downloaded") + QString("：%1GB").arg(QString::number(downloadflow / (GB), 'f', 2));
        else if (TB_SECTION(downloadflow))
            download = tr("Downloaded") + QString("：%1TB").arg(QString::number(downloadflow / (TB), 'f', 2));

        if (KB_SECTION(totalflow))
            total = tr("Data Used") + QString("：%1KB").arg(QString::number(totalflow, 'f', 2));
        else if (MB_SECTION(totalflow))
            total = tr("Data Used") + QString("：%1MB").arg(QString::number(totalflow / (MB), 'f', 2));
        else if (GB_SECTION(totalflow))
            total = tr("Data Used") + QString("：%1GB").arg(QString::number(totalflow / (GB), 'f', 2));
        else if (TB_SECTION(totalflow))
            total = tr("Data Used") + QString("：%1TB").arg(QString::number(totalflow / (TB), 'f', 2));

        m_detailInfo->setText(download + "   " + upload + "   " + total);
    }

    connect(m_timebox, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged), this, [this](int index) {
        switch (index) {
        case 0:
            Q_EMIT m_worker->GetTrafficdetails()->notifyGetTodayFlowData();
            break;
        case 1:
            Q_EMIT m_worker->GetTrafficdetails()->notifyGetYesterdayFlowData();
            break;
        case 2:
            Q_EMIT m_worker->GetTrafficdetails()->notifyGetThisMonthFlowData();
            break;
        case 3:
            Q_EMIT m_worker->GetTrafficdetails()->notifyGetLastMonthFlowData();
            break;
        }
    });
}

//接收界面显示需要的数据
void TrafficDetailWidget::onsetdata()
{
    QString upload = "";
    QString download = "";
    QString total = "";
    m_displayflowlist = m_worker->GetFlowList();
    double uploadflow = 0;
    double downloadflow = 0;
    double uploadspeed = 0;
    double downloadspeed = 0;
    m_displayspeedlist = m_worker->GetSpeedList();
    for (auto at : m_displayspeedlist) {
        uploadspeed += at.update.toDouble();
        downloadspeed += at.download.toDouble();
    }
    for (auto at : m_displayflowlist) {
        uploadflow += at.update.toDouble();
        downloadflow += at.download.toDouble();
    }
    double totalflow = uploadflow + downloadflow;
    if (!m_tabeltype) {
        if (KB_SECTION(uploadspeed))
            upload = tr("Total Upload") + QString("：%1KB/s").arg(QString::number(uploadspeed, 'f', 2));
        else if (MB_SECTION(uploadspeed))
            upload = tr("Total Upload") + QString("：%1MB/s").arg(QString::number(uploadspeed / (MB), 'f', 2));

        if (KB_SECTION(downloadspeed))
            download = tr("Total Download") + QString("：%1KB/s").arg(QString::number(downloadspeed, 'f', 2));
        else if (MB_SECTION(downloadspeed))
            download = tr("Total Download") + QString("：%1MB/s").arg(QString::number(downloadspeed / (MB), 'f', 2));

        m_detailInfo->setText(download + "   " + upload);
    } else {
        if (KB_SECTION(uploadflow))
            upload = tr("Uploaded") + QString("：%1KB").arg(QString::number(uploadflow, 'f', 2));
        else if (MB_SECTION(uploadflow))
            upload = tr("Uploaded") + QString("：%1MB").arg(QString::number(uploadflow / (MB), 'f', 2));
        else if (GB_SECTION(uploadflow))
            upload = tr("Uploaded") + QString("：%1GB").arg(QString::number(uploadflow / (GB), 'f', 2));
        else if (TB_SECTION(uploadflow))
            upload = tr("Uploaded") + QString("：%1TB").arg(QString::number(uploadflow / (TB), 'f', 2));

        if (KB_SECTION(downloadflow))
            download = tr("Downloaded") + QString("：%1KB").arg(QString::number(downloadflow, 'f', 2));
        else if (MB_SECTION(downloadflow))
            download = tr("Downloaded") + QString("：%1MB").arg(QString::number(downloadflow / (MB), 'f', 2));
        else if (GB_SECTION(downloadflow))
            download = tr("Downloaded") + QString("：%1GB").arg(QString::number(downloadflow / (GB), 'f', 2));
        else if (TB_SECTION(downloadflow))
            download = tr("Downloaded") + QString("：%1TB").arg(QString::number(downloadflow / (TB), 'f', 2));

        if (KB_SECTION(totalflow))
            total = tr("Data Used") + QString("：%1KB").arg(QString::number(totalflow, 'f', 2));
        else if (MB_SECTION(totalflow))
            total = tr("Data Used") + QString("：%1MB").arg(QString::number(totalflow / (MB), 'f', 2));
        else if (GB_SECTION(totalflow))
            total = tr("Data Used") + QString("：%1GB").arg(QString::number(totalflow / (GB), 'f', 2));
        else if (TB_SECTION(totalflow))
            total = tr("Data Used") + QString("：%1TB").arg(QString::number(totalflow / (TB), 'f', 2));

        m_detailInfo->setText(download + "   " + upload + "   " + total);
    }
}

//设置背景大小
void TrafficDetailWidget::resizeEvent(QResizeEvent *event)
{
    QFrame::resizeEvent(event);

    //设置m_bgGroup 的大小
    if (m_bgGroup)
        m_bgGroup->setFixedSize(size());
}
