#include "loginsecuritywidget.h"
#include <QLabel>
#include <pwd.h>
#include <unistd.h>

#include <DCommandLinkButton>
#include <DFontSizeManager>

LoginSecurityWidget::LoginSecurityWidget(SystemSafetyModel *model, SystemSafetyWork *work, QWidget *parent)
    : QWidget(parent)
    , m_model(model)
    , m_work(work)
    , m_layout(new QVBoxLayout)
    , m_PasswodUpdateTips(new QLabel)
    , m_pButtonGroup(new QButtonGroup(this))
{
    struct passwd *pws;
    pws = getpwuid(getuid());
    QString m_currentUserName = QString(pws->pw_name);
    qDebug() << "m_currentUserName = " << m_currentUserName;
    QString user = m_work->GetAccounts()->FindUserByName(m_currentUserName);
    qDebug() << "user = " << user;
    m_usermanage = new DbususerManager(user);
    int date =  m_usermanage->passwordLastChange();
    const QDateTime datetime(QDateTime::currentDateTime());
    QDateTime dt = QDateTime(QDate(1970, 1, 1));
    m_day = dt.daysTo(datetime) - date;
    initui();
    connect(m_work->GetDaemonService(), &DaemonService::UpdateUI, [this] {
        qDebug() << "PasswordLastChangeChanged";
        Setlastchage();
    });
}

LoginSecurityWidget::~LoginSecurityWidget()
{

}

void LoginSecurityWidget::initui()
{
    QLabel *title = new QLabel(tr("Password Policy"));
    QFont ft = title->font();
    ft.setBold(true);
    title->setFont(ft);
    DFontSizeManager::instance()->bind(title, DFontSizeManager::T5);
    m_layout->setSpacing(0);
    m_layout->addWidget(title, 0, Qt::AlignLeft | Qt::AlignTop);
    m_layout->setAlignment(Qt::AlignTop);
    m_layout->addSpacing(15);
    QVBoxLayout *vlayout = new QVBoxLayout;

    // 设置互斥
    m_pButtonGroup->setExclusive(true);

    QHBoxLayout *highlayout = new QHBoxLayout();
    highlayout->setContentsMargins(0, 0, 0, 0);
    highlayout->setAlignment(Qt::AlignLeft);
    QRadioButton *highButton = new QRadioButton(this);
    m_highLevelItem = new SecurityLevelItem;
    m_highLevelItem->setIcon(QIcon::fromTheme("dcc_security_level_high"));
    m_highLevelItem->setLevelText(tr("High"));
    m_highLevelItem->setTipText(tr("The password must have at least 8 characters, and contain at least 3 of the four available character types: lowercase letters, uppercase letters, numbers, and symbols"));
    highlayout->addWidget(highButton, 0, Qt::AlignCenter);
    highlayout->addWidget(m_highLevelItem);
    vlayout->addLayout(highlayout);
    m_pButtonGroup->addButton(highButton, LoginSecurity::high);
    vlayout->addSpacing(15);

    QHBoxLayout *mediumlayout = new QHBoxLayout();
    mediumlayout->setContentsMargins(0, 0, 0, 0);
    mediumlayout->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QRadioButton *mediumButton = new QRadioButton(this);
    m_mediumLevelItem = new SecurityLevelItem;
    m_mediumLevelItem->setIcon(QIcon::fromTheme("dcc_security_level_medium"));
    m_mediumLevelItem->setLevelText(tr("Medium"));
    m_mediumLevelItem->setTipText(tr("The password must have at least 6 characters, and contain at least 2 of the four available character types: lowercase letters, uppercase letters, numbers, and symbols"));
    mediumlayout->addWidget(mediumButton, 0, Qt::AlignCenter);
    mediumlayout->addWidget(m_mediumLevelItem);
    vlayout->addLayout(mediumlayout);
    m_pButtonGroup->addButton(mediumButton, LoginSecurity::middle);
    vlayout->addSpacing(15);

    QHBoxLayout *lowlayout = new QHBoxLayout();
    lowlayout->setContentsMargins(0, 0, 0, 0);
    lowlayout->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QRadioButton *lowButton = new QRadioButton(this);
    m_lowLevelItem = new SecurityLevelItem;
    m_lowLevelItem->setIcon(QIcon::fromTheme("dcc_security_level_low"));
    m_lowLevelItem->setLevelText(tr("Low"));
    m_lowLevelItem->setTipText(tr("No restrictions"));
    lowlayout->addWidget(lowButton, 0, Qt::AlignCenter);
    lowlayout->addWidget(m_lowLevelItem);
    vlayout->addLayout(lowlayout);
    m_pButtonGroup->addButton(lowButton, LoginSecurity::low);

    vlayout->addSpacing(15);
    m_layout->addLayout(vlayout);
    m_layout->addSpacing(15);

    m_menu = new PassWordMenu;
    m_menu->setTitle(tr("Change Password"));
    m_menu->setMenuData(tr("Go to Settings"));
    m_layout->addWidget(m_menu, 0, Qt::AlignLeft | Qt::AlignTop);
    connect(m_menu, &PassWordMenu::notifyClicked, this, [ = ] {
        m_work->GetControlCenter()->ShowModule("accounts");
    });
    m_layout->addSpacing(15);
    if (m_day >= 90) {
        m_PasswodUpdateTips->setText(tr("You have not changed the password for %1 days, please change it regularly to improve system security.").arg(m_day));
    } else {
        m_PasswodUpdateTips->hide();
    }
    m_PasswodUpdateTips->setWordWrap(true);
    m_PasswodUpdateTips->setAlignment(Qt::AlignLeft);
    QFont ft1 = m_PasswodUpdateTips->font();
    ft1.setBold(true);
    m_PasswodUpdateTips->setFont(ft1);
    DFontSizeManager::instance()->bind(m_PasswodUpdateTips, DFontSizeManager::T8);
    m_layout->addWidget(m_PasswodUpdateTips);
    // 连接信号槽
    connect(m_pButtonGroup, QOverload<int>::of(&QButtonGroup::buttonPressed), this, &LoginSecurityWidget::onButtonClicked);
    connect(m_highLevelItem, &SecurityLevelItem::clicked, highButton, &QRadioButton::click);
    connect(m_mediumLevelItem, &SecurityLevelItem::clicked, mediumButton, &QRadioButton::click);
    connect(m_lowLevelItem, &SecurityLevelItem::clicked, lowButton, &QRadioButton::click);

    setLayout(m_layout);
    int safetygrade = m_work->GetDaemonService()->safetygrade();
    setStyle(safetygrade);
}

void LoginSecurityWidget::Setlastchage()
{
    int date =  m_usermanage->passwordLastChange();
    const QDateTime datetime(QDateTime::currentDateTime());
    QDateTime dt = QDateTime(QDate(1970, 1, 1));
    m_day = dt.daysTo(datetime) - date;
    if (m_day >= 90) {
        m_PasswodUpdateTips->setText(tr("You have not changed the password for %1 days, please change it regularly to improve system security.").arg(m_day));
    } else {
        m_PasswodUpdateTips->hide();
    }
}

void LoginSecurityWidget::onButtonClicked(int button)
{
    setStyle(button);
    m_work->GetDaemonService()->SetGsetting(button);
}

void LoginSecurityWidget::setStyle(int grade)
{
    switch (grade) {
    case LoginSecurity::high :
        m_pButtonGroup->button(LoginSecurity::high)->setChecked(true); //设置按钮
        m_highLevelItem->setDisableStyle(false);
        m_mediumLevelItem->setDisableStyle(true);
        m_lowLevelItem->setDisableStyle(true);
        break;
    case LoginSecurity::middle :
        m_pButtonGroup->button(LoginSecurity::middle)->setChecked(true); //设置按钮
        m_highLevelItem->setDisableStyle(true);
        m_mediumLevelItem->setDisableStyle(false);
        m_lowLevelItem->setDisableStyle(true);
        break;
    case LoginSecurity::low :
        m_pButtonGroup->button(LoginSecurity::low)->setChecked(true); //设置按钮
        m_highLevelItem->setDisableStyle(true);
        m_mediumLevelItem->setDisableStyle(true);
        m_lowLevelItem->setDisableStyle(false);
        break;
    }
}
